@@ -1,5 +1,5 @@
 #include <string>
-#include <string.h>
+#include <cstring>
 
 #include <Poco/UTF8Encoding.h>
 #include <Poco/NumberParser.h>
@@ -7,15 +7,16 @@
 #include <base/find_symbols.h>
 #include <base/preciseExp10.h>
 
-#include <iostream>
-
-#define JSON_MAX_DEPTH 100
+constexpr size_t JSON_MAX_DEPTH = 100;
 
 
-POCO_IMPLEMENT_EXCEPTION(JSONException, Poco::Exception, "JSONException")
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-dynamic-exception-spec"
+POCO_IMPLEMENT_EXCEPTION(JSONException, Poco::Exception, "JSONException") // NOLINT(cert-err60-cpp, modernize-use-noexcept, hicpp-use-noexcept)
+#pragma clang diagnostic pop
 
 
-/// Прочитать беззнаковое целое в простом формате из не-0-terminated строки.
+/// Read unsigned integer in a simple form from a non-0-terminated string.
 static UInt64 readUIntText(const char * buf, const char * end)
 {
     UInt64 x = 0;
@@ -52,7 +53,7 @@ static UInt64 readUIntText(const char * buf, const char * end)
 }
 
 
-/// Прочитать знаковое целое в простом формате из не-0-terminated строки.
+/// Read signed integer in a simple form from a non-0-terminated string.
 static Int64 readIntText(const char * buf, const char * end)
 {
     bool negative = false;
@@ -95,7 +96,7 @@ static Int64 readIntText(const char * buf, const char * end)
 }
 
 
-/// Прочитать число с плавающей запятой в простом формате, с грубым округлением, из не-0-terminated строки.
+/// Read floating point number in simple format, imprecisely, from a non-0-terminated string.
 static double readFloatText(const char * buf, const char * end)
 {
     bool negative = false;
@@ -144,8 +145,8 @@ static double readFloatText(const char * buf, const char * end)
             case 'E':
             {
                 ++buf;
-                Int32 exponent = readIntText(buf, end);
-                x *= preciseExp10(exponent);
+                auto exponent = readIntText(buf, end);
+                x *= preciseExp10(static_cast<double>(exponent));
 
                 run = false;
                 break;
@@ -200,12 +201,11 @@ JSON::ElementType JSON::getType() const
             return TYPE_NUMBER;
         case '"':
         {
-            /// Проверим - это просто строка или name-value pair
+            /// Is it a string or a name-value pair?
             Pos after_string = skipString();
             if (after_string < ptr_end && *after_string == ':')
                 return TYPE_NAME_VALUE_PAIR;
-            else
-                return TYPE_STRING;
+            return TYPE_STRING;
         }
         default:
             throw JSONException(std::string("JSON: unexpected char ") + *ptr_begin + ", expected one of '{[tfn-0123456789\"'");
@@ -222,15 +222,13 @@ void JSON::checkPos(Pos pos) const
 
 JSON::Pos JSON::skipString() const
 {
-    //std::cerr << "skipString()\t" << data() << std::endl;
-
     Pos pos = ptr_begin;
     checkPos(pos);
     if (*pos != '"')
         throw JSONException(std::string("JSON: expected \", got ") + *pos);
     ++pos;
 
-    /// fast path: находим следующую двойную кавычку. Если перед ней нет бэкслеша - значит это конец строки (при допущении корректности JSON).
+    /// fast path: find next double quote. If it is not escaped by backslash - then it's an end of string (assuming JSON is valid).
     Pos closing_quote = reinterpret_cast<const char *>(memchr(reinterpret_cast<const void *>(pos), '\"', ptr_end - pos));
     if (nullptr != closing_quote && closing_quote[-1] != '\\')
         return closing_quote + 1;
@@ -262,8 +260,6 @@ JSON::Pos JSON::skipString() const
 
 JSON::Pos JSON::skipNumber() const
 {
-    //std::cerr << "skipNumber()\t" << data() << std::endl;
-
     Pos pos = ptr_begin;
 
     checkPos(pos);
@@ -289,8 +285,6 @@ JSON::Pos JSON::skipNumber() const
 
 JSON::Pos JSON::skipBool() const
 {
-    //std::cerr << "skipBool()\t" << data() << std::endl;
-
     Pos pos = ptr_begin;
     checkPos(pos);
 
@@ -307,16 +301,12 @@ JSON::Pos JSON::skipBool() const
 
 JSON::Pos JSON::skipNull() const
 {
-    //std::cerr << "skipNull()\t" << data() << std::endl;
-
     return ptr_begin + 4;
 }
 
 
 JSON::Pos JSON::skipNameValuePair() const
 {
-    //std::cerr << "skipNameValuePair()\t" << data() << std::endl;
-
     Pos pos = skipString();
     checkPos(pos);
 
@@ -331,8 +321,6 @@ JSON::Pos JSON::skipNameValuePair() const
 
 JSON::Pos JSON::skipArray() const
 {
-    //std::cerr << "skipArray()\t" << data() << std::endl;
-
     if (!isArray())
         throw JSONException("JSON: expected [");
     Pos pos = ptr_begin;
@@ -363,8 +351,6 @@ JSON::Pos JSON::skipArray() const
 
 JSON::Pos JSON::skipObject() const
 {
-    //std::cerr << "skipObject()\t" << data() << std::endl;
-
     if (!isObject())
         throw JSONException("JSON: expected {");
     Pos pos = ptr_begin;
@@ -395,8 +381,6 @@ JSON::Pos JSON::skipObject() const
 
 JSON::Pos JSON::skipElement() const
 {
-    //std::cerr << "skipElement()\t" << data() << std::endl;
-
     ElementType type = getType();
 
     switch (type)
@@ -475,9 +459,8 @@ JSON::Pos JSON::searchField(const char * data, size_t size) const
     {
         if (!it->hasEscapes())
         {
-            if (static_cast<int>(size) + 2 > it->dataEnd() - it->data())
-                continue;
-            if (!strncmp(data, it->data() + 1, size))
+            const auto current_name = it->getRawName();
+            if (current_name.size() == size && 0 == memcmp(current_name.data(), data, size))
                 break;
         }
         else
@@ -490,8 +473,7 @@ JSON::Pos JSON::searchField(const char * data, size_t size) const
 
     if (it == end())
         return nullptr;
-    else
-        return it->data();
+    return it->data();
 }
 
 
@@ -503,7 +485,7 @@ bool JSON::hasEscapes() const
 
     if (*pos == '"')
         return false;
-    else if (*pos == '\\')
+    if (*pos == '\\')
         return true;
     throw JSONException("JSON: unexpected end of data.");
 }
@@ -519,7 +501,7 @@ bool JSON::hasSpecialChars() const
 
     if (*pos == '"')
         return false;
-    else if (pos < ptr_end)
+    if (pos < ptr_end)
         return true;
     throw JSONException("JSON: unexpected end of data.");
 }
@@ -633,7 +615,7 @@ std::string JSON::getString() const
                         {
                             throw JSONException("JSON: incorrect syntax: incorrect HEX code.");
                         }
-                        buf.resize(buf.size() + 6);    /// максимальный размер UTF8 многобайтовой последовательности
+                        buf.resize(buf.size() + 6);    /// Max size of UTF-8 sequence, including pre-standard mapping of UCS-4 to UTF-8.
                         int res = utf8.convert(unicode,
                             reinterpret_cast<unsigned char *>(const_cast<char*>(buf.data())) + buf.size() - 6, 6);
                         if (!res)
@@ -662,18 +644,20 @@ std::string JSON::getName() const
     return getString();
 }
 
-StringRef JSON::getRawString() const
+std::string_view JSON::getRawString() const
 {
     Pos s = ptr_begin;
     if (*s != '"')
         throw JSONException(std::string("JSON: expected \", got ") + *s);
-    while (++s != ptr_end && *s != '"');
+    ++s;
+    while (s != ptr_end && *s != '"')
+        ++s;
     if (s != ptr_end)
-        return StringRef(ptr_begin + 1, s - ptr_begin - 1);
+        return std::string_view(ptr_begin + 1, s - ptr_begin - 1);
     throw JSONException("JSON: incorrect syntax (expected end of string, found end of JSON).");
 }
 
-StringRef JSON::getRawName() const
+std::string_view JSON::getRawName() const
 {
     return getRawString();
 }
@@ -696,10 +680,9 @@ double JSON::toDouble() const
 
     if (type == TYPE_NUMBER)
         return getDouble();
-    else if (type == TYPE_STRING)
+    if (type == TYPE_STRING)
         return JSON(ptr_begin + 1, ptr_end, level + 1).getDouble();
-    else
-        throw JSONException("JSON: cannot convert value to double.");
+    throw JSONException("JSON: cannot convert value to double.");
 }
 
 Int64 JSON::toInt() const
@@ -708,10 +691,9 @@ Int64 JSON::toInt() const
 
     if (type == TYPE_NUMBER)
         return getInt();
-    else if (type == TYPE_STRING)
+    if (type == TYPE_STRING)
         return JSON(ptr_begin + 1, ptr_end, level + 1).getInt();
-    else
-        throw JSONException("JSON: cannot convert value to signed integer.");
+    throw JSONException("JSON: cannot convert value to signed integer.");
 }
 
 UInt64 JSON::toUInt() const
@@ -720,10 +702,9 @@ UInt64 JSON::toUInt() const
 
     if (type == TYPE_NUMBER)
         return getUInt();
-    else if (type == TYPE_STRING)
+    if (type == TYPE_STRING)
         return JSON(ptr_begin + 1, ptr_end, level + 1).getUInt();
-    else
-        throw JSONException("JSON: cannot convert value to unsigned integer.");
+    throw JSONException("JSON: cannot convert value to unsigned integer.");
 }
 
 std::string JSON::toString() const
@@ -732,11 +713,9 @@ std::string JSON::toString() const
 
     if (type == TYPE_STRING)
         return getString();
-    else
-    {
-        Pos pos = skipElement();
-        return std::string(ptr_begin, pos - ptr_begin);
-    }
+
+    Pos pos = skipElement();
+    return std::string(ptr_begin, pos - ptr_begin);
 }
 
 
@@ -747,8 +726,6 @@ JSON::iterator JSON::iterator::begin() const
     if (type != TYPE_ARRAY && type != TYPE_OBJECT)
         throw JSONException("JSON: not array or object when calling begin() method.");
 
-    //std::cerr << "begin()\t" << data() << std::endl;
-
     Pos pos = ptr_begin + 1;
     checkPos(pos);
     if (*pos == '}' || *pos == ']')
@@ -839,4 +816,3 @@ bool JSON::isType<bool>() const
 {
     return isBool();
 }
-
