@@ -3,21 +3,21 @@
 #include <vector>
 #include <algorithm>
 #include <cctype>
+#include <iostream>
 #include <unordered_set>
 #include <unordered_map>
 #include <list>
 #include <thread>
 #include <filesystem>
 
-#include <re2/re2.h>
-
 #include <boost/program_options.hpp>
 
 #include <Common/TerminalSize.h>
 #include <Common/Exception.h>
 #include <Common/SipHash.h>
-#include <Common/StringUtils/StringUtils.h>
+#include <Common/StringUtils.h>
 #include <Common/ShellCommand.h>
+#include <Common/re2.h>
 #include <base/find_symbols.h>
 
 #include <IO/copyData.h>
@@ -26,7 +26,6 @@
 #include <IO/WriteBufferFromFile.h>
 #include <IO/WriteBufferFromFileDescriptor.h>
 
-
 static constexpr auto documentation = R"(
 A tool to extract information from Git repository for analytics.
 
@@ -67,7 +66,7 @@ Run this tool inside your git repository. It will create .tsv files that can be
 The tool can process large enough repositories in a reasonable time.
 It has been tested on:
 - ClickHouse: 31 seconds; 3 million rows;
-- LLVM: 8 minues; 62 million rows;
+- LLVM: 8 minutes; 62 million rows;
 - Linux - 12 minutes; 85 million rows;
 - Chromium - 67 minutes; 343 million rows;
 (the numbers as of Sep 2020)
@@ -174,6 +173,7 @@ clickhouse-client --query "INSERT INTO git.commits FORMAT TSV" < commits.tsv
 clickhouse-client --query "INSERT INTO git.file_changes FORMAT TSV" < file_changes.tsv
 clickhouse-client --query "INSERT INTO git.line_changes FORMAT TSV" < line_changes.tsv
 
+Check out this presentation: https://presentations.clickhouse.com/matemarketing_2020/
 )";
 
 namespace po = boost::program_options;
@@ -234,7 +234,7 @@ struct Commit
 };
 
 
-enum class FileChangeType
+enum class FileChangeType : uint8_t
 {
     Add,
     Delete,
@@ -244,7 +244,7 @@ enum class FileChangeType
     Type,
 };
 
-void writeText(FileChangeType type, WriteBuffer & out)
+static void writeText(FileChangeType type, WriteBuffer & out)
 {
     switch (type)
     {
@@ -292,7 +292,7 @@ struct FileChange
 };
 
 
-enum class LineType
+enum class LineType : uint8_t
 {
     Empty,
     Comment,
@@ -300,7 +300,7 @@ enum class LineType
     Code,
 };
 
-void writeText(LineType type, WriteBuffer & out)
+static void writeText(LineType type, WriteBuffer & out)
 {
     switch (type)
     {
@@ -351,7 +351,7 @@ struct LineChange
             ++pos;
         }
 
-        indent = std::max(255U, num_spaces);
+        indent = std::min(255U, num_spaces);
         line.assign(pos, end);
 
         if (pos == end)
@@ -430,7 +430,7 @@ using CommitDiff = std::map<std::string /* path */, FileDiff>;
 
 /** Parsing helpers */
 
-void skipUntilWhitespace(ReadBuffer & buf)
+static void skipUntilWhitespace(ReadBuffer & buf)
 {
     while (!buf.eof())
     {
@@ -445,7 +445,7 @@ void skipUntilWhitespace(ReadBuffer & buf)
     }
 }
 
-void skipUntilNextLine(ReadBuffer & buf)
+static void skipUntilNextLine(ReadBuffer & buf)
 {
     while (!buf.eof())
     {
@@ -463,7 +463,7 @@ void skipUntilNextLine(ReadBuffer & buf)
     }
 }
 
-void readStringUntilNextLine(std::string & s, ReadBuffer & buf)
+static void readStringUntilNextLine(std::string & s, ReadBuffer & buf)
 {
     s.clear();
     while (!buf.eof())
@@ -530,6 +530,13 @@ struct ResultWriter
             }
         }
     }
+
+    void finalize()
+    {
+        commits.finalize();
+        file_changes.finalize();
+        line_changes.finalize();
+    }
 };
 
 
@@ -604,6 +611,8 @@ struct FileBlame
     /// This is important when file was copied or renamed.
     FileBlame & operator=(const FileBlame & rhs)
     {
+        if (&rhs == this)
+            return *this;
         lines = rhs.lines;
         it = lines.begin();
         current_idx = 1;
@@ -674,7 +683,7 @@ using Snapshot = std::map<std::string /* path */, FileBlame>;
   * - the author, time and commit of the previous change to every found line (blame).
   * And update the snapshot.
   */
-void updateSnapshot(Snapshot & snapshot, const Commit & commit, CommitDiff & file_changes)
+static void updateSnapshot(Snapshot & snapshot, const Commit & commit, CommitDiff & file_changes)
 {
     /// Renames and copies.
     for (auto & elem : file_changes)
@@ -715,7 +724,7 @@ void updateSnapshot(Snapshot & snapshot, const Commit & commit, CommitDiff & fil
                 uint32_t this_line_in_prev_commit = line_change.hunk_start_line_number_old
                     + (line_change.line_number_new - line_change.hunk_start_line_number_new);
 
-                if (deleted_lines.count(this_line_in_prev_commit))
+                if (deleted_lines.contains(this_line_in_prev_commit))
                 {
                     const auto & prev_commit = deleted_lines[this_line_in_prev_commit];
                     if (prev_commit.time <= commit.time)
@@ -749,7 +758,7 @@ void updateSnapshot(Snapshot & snapshot, const Commit & commit, CommitDiff & fil
   */
 using DiffHashes = std::unordered_set<UInt128>;
 
-UInt128 diffHash(const CommitDiff & file_changes)
+static UInt128 diffHash(const CommitDiff & file_changes)
 {
     SipHash hasher;
 
@@ -785,7 +794,7 @@ UInt128 diffHash(const CommitDiff & file_changes)
   * :100644 100644 828dedf6b5 828dedf6b5 R100       dbms/src/Functions/GeoUtils.h   dbms/src/Functions/PolygonUtils.h
   * according to the output of 'git show --raw'
   */
-void processFileChanges(
+static void processFileChanges(
     ReadBuffer & in,
     const Options & options,
     Commit & commit,
@@ -877,7 +886,7 @@ void processFileChanges(
   * - we expect some specific format of the diff; but it may actually depend on git config;
   * - non-ASCII file names are not processed correctly (they will not be found and will be ignored).
   */
-void processDiffs(
+static void processDiffs(
     ReadBuffer & in,
     std::optional<size_t> size_limit,
     Commit & commit,
@@ -1049,7 +1058,7 @@ void processDiffs(
 
 /** Process the "git show" result for a single commit. Append the result to tables.
   */
-void processCommit(
+static void processCommit(
     ReadBuffer & in,
     const Options & options,
     size_t commit_num,
@@ -1117,7 +1126,7 @@ void processCommit(
 /** Runs child process and allows to read the result.
   * Multiple processes can be run for parallel processing.
   */
-auto gitShow(const std::string & hash)
+static auto gitShow(const std::string & hash)
 {
     std::string command = fmt::format(
         "git show --raw --pretty='format:%ct%x00%aN%x00%P%x00%s%x00' --patch --unified=0 {}",
@@ -1129,7 +1138,7 @@ auto gitShow(const std::string & hash)
 
 /** Obtain the list of commits and process them.
   */
-void processLog(const Options & options)
+static void processLog(const Options & options)
 {
     ResultWriter result;
 
@@ -1150,7 +1159,7 @@ void processLog(const Options & options)
         readString(hash, in);
         assertChar('\n', in);
 
-        if (!options.skip_commits.count(hash))
+        if (!options.skip_commits.contains(hash))
             hashes.emplace_back(std::move(hash));
     }
 
@@ -1160,7 +1169,7 @@ void processLog(const Options & options)
     /// Will run multiple processes in parallel
     size_t num_threads = options.threads;
     if (num_threads == 0)
-        throw Exception("num-threads cannot be zero", ErrorCodes::INCORRECT_DATA);
+        throw Exception(ErrorCodes::INCORRECT_DATA, "num-threads cannot be zero");
 
     std::vector<std::unique_ptr<ShellCommand>> show_commands(num_threads);
     for (size_t i = 0; i < num_commits && i < num_threads; ++i)
@@ -1179,6 +1188,8 @@ void processLog(const Options & options)
         if (i + num_threads < num_commits)
             show_commands[i % num_threads] = gitShow(hashes[i + num_threads]);
     }
+
+    result.finalize();
 }
 
 
@@ -1231,5 +1242,5 @@ try
 catch (...)
 {
     std::cerr << DB::getCurrentExceptionMessage(true) << '\n';
-    throw;
+    return DB::getCurrentExceptionCode();
 }
