@@ -1,6 +1,7 @@
 #include <Access/Credentials.h>
 #include <Common/Exception.h>
-
+#include <Poco/Net/HTTPRequest.h>
+#include <Common/Crypto/X509Certificate.h>
 
 namespace DB
 {
@@ -29,7 +30,7 @@ bool Credentials::isReady() const
 
 void Credentials::throwNotReady()
 {
-    throw Exception("Credentials are not ready", ErrorCodes::LOGICAL_ERROR);
+    throw Exception(ErrorCodes::LOGICAL_ERROR, "Credentials are not ready");
 }
 
 AlwaysAllowCredentials::AlwaysAllowCredentials()
@@ -48,6 +49,22 @@ void AlwaysAllowCredentials::setUserName(const String & user_name_)
     user_name = user_name_;
 }
 
+#if USE_SSL
+SSLCertificateCredentials::SSLCertificateCredentials(const String & user_name_, X509Certificate::Subjects && subjects_)
+    : Credentials(user_name_)
+    , certificate_subjects(subjects_)
+{
+    is_ready = true;
+}
+
+const X509Certificate::Subjects & SSLCertificateCredentials::getSSLCertificateSubjects() const
+{
+    if (!isReady())
+        throwNotReady();
+    return certificate_subjects;
+}
+#endif
+
 BasicCredentials::BasicCredentials()
 {
     is_ready = true;
