@@ -1,16 +1,13 @@
-#include "AggregateFunctionMLMethod.h"
-
+#include <AggregateFunctions/AggregateFunctionFactory.h>
+#include <AggregateFunctions/AggregateFunctionMLMethod.h>
+#include <AggregateFunctions/FactoryHelpers.h>
+#include <Columns/ColumnArray.h>
 #include <IO/ReadHelpers.h>
 #include <IO/WriteHelpers.h>
 #include <Interpreters/castColumn.h>
-#include <Columns/ColumnArray.h>
-#include <Columns/ColumnTuple.h>
 #include <Common/FieldVisitorConvertToNumber.h>
-#include <Common/typeid_cast.h>
 #include <Common/assert_cast.h>
-#include "AggregateFunctionFactory.h"
-#include "FactoryHelpers.h"
-#include "Helpers.h"
+#include <Common/typeid_cast.h>
 
 
 namespace DB
@@ -22,7 +19,8 @@ namespace ErrorCodes
     extern const int BAD_ARGUMENTS;
     extern const int LOGICAL_ERROR;
     extern const int ILLEGAL_TYPE_OF_ARGUMENT;
-    extern const int NUMBER_OF_ARGUMENTS_DOESNT_MATCH;
+    extern const int TOO_FEW_ARGUMENTS_FOR_FUNCTION;
+    extern const int TOO_MANY_ARGUMENTS_FOR_FUNCTION;
 }
 
 namespace
@@ -34,30 +32,26 @@ namespace
         const std::string & name, const DataTypes & argument_types, const Array & parameters, const Settings *)
     {
         if (parameters.size() > 4)
-            throw Exception(
-                "Aggregate function " + name
-                    + " requires at most four parameters: learning_rate, l2_regularization_coef, mini-batch size and weights_updater "
-                      "method",
-                ErrorCodes::NUMBER_OF_ARGUMENTS_DOESNT_MATCH);
+            throw Exception(ErrorCodes::TOO_MANY_ARGUMENTS_FOR_FUNCTION,
+                "Aggregate function {} requires at most four parameters: "
+                "learning_rate, l2_regularization_coef, mini-batch size and weights_updater method", name);
 
         if (argument_types.size() < 2)
-            throw Exception(
-                "Aggregate function " + name + " requires at least two arguments: target and model's parameters",
-                ErrorCodes::NUMBER_OF_ARGUMENTS_DOESNT_MATCH);
+            throw Exception(ErrorCodes::TOO_FEW_ARGUMENTS_FOR_FUNCTION,
+                "Aggregate function {} requires at least two arguments: target and model's parameters", name);
 
         for (size_t i = 0; i < argument_types.size(); ++i)
         {
             if (!isNativeNumber(argument_types[i]))
-                throw Exception(
-                    "Argument " + std::to_string(i) + " of type " + argument_types[i]->getName()
-                        + " must be numeric for aggregate function " + name,
-                    ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT);
+                throw Exception(ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT,
+                                "Argument {} of type {} must be numeric for aggregate function {}",
+                                i, argument_types[i]->getName(), name);
         }
 
         /// Such default parameters were picked because they did good on some tests,
         /// though it still requires to fit parameters to achieve better result
-        auto learning_rate = Float64(1.0);
-        auto l2_reg_coef = Float64(0.5);
+        auto learning_rate = static_cast<Float64>(1.0);
+        auto l2_reg_coef = static_cast<Float64>(0.5);
         UInt64 batch_size = 15;
 
         std::string weights_updater_name = "Adam";
@@ -79,21 +73,21 @@ namespace
         {
             weights_updater_name = parameters[3].safeGet<String>();
             if (weights_updater_name != "SGD" && weights_updater_name != "Momentum" && weights_updater_name != "Nesterov" && weights_updater_name != "Adam")
-                throw Exception("Invalid parameter for weights updater. The only supported are 'SGD', 'Momentum' and 'Nesterov'",
-                    ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT);
+                throw Exception(ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT, "Invalid parameter for weights updater. "
+                                "The only supported are 'SGD', 'Momentum' and 'Nesterov'");
         }
 
-        if (std::is_same<Method, FuncLinearRegression>::value)
+        if constexpr (std::is_same_v<Method, FuncLinearRegression>)
         {
             gradient_computer = std::make_unique<LinearRegression>();
         }
-        else if (std::is_same<Method, FuncLogisticRegression>::value)
+        else if constexpr (std::is_same_v<Method, FuncLogisticRegression>)
         {
             gradient_computer = std::make_unique<LogisticRegression>();
         }
         else
         {
-            throw Exception("Such gradient computer is not implemented yet", ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT);
+            []<bool flag = false>() {static_assert(flag, "Such gradient computer is not implemented yet");}(); // delay static_assert in constexpr if until template instantiation
         }
 
         return std::make_shared<Method>(
@@ -110,8 +104,229 @@ namespace
 
 void registerAggregateFunctionMLMethod(AggregateFunctionFactory & factory)
 {
-    factory.registerFunction("stochasticLinearRegression", createAggregateFunctionMLMethod<FuncLinearRegression>);
-    factory.registerFunction("stochasticLogisticRegression", createAggregateFunctionMLMethod<FuncLogisticRegression>);
+    // stochasticLinearRegression documentation
+    FunctionDocumentation::Description description_linear = R"(
+This function implements stochastic linear regression.
+It supports custom parameters for:
+- learning rate
+- L2 regularization coefficient
+- mini-batch size
+
+It also has a few methods for updating weights:
+- Adam (used by default)
+- simple SGD
+- Momentum
+- Nesterov
+
+**Usage**
+
+The function is used in two steps: fitting the model and predicting on new data.
+
+1. Fitting
+
+For fitting a query like this can be used:
+
+```sql
+CREATE TABLE IF NOT EXISTS train_data
+(
+    param1 Float64,
+    param2 Float64,
+    target Float64
+) ENGINE = Memory;
+
+CREATE TABLE your_model ENGINE = Memory AS SELECT
+stochasticLinearRegressionState(0.1, 0.0, 5, 'SGD')(target, x1, x2)
+AS state FROM train_data;
+```
+
+Here, we also need to insert data into the `train_data` table.
+The number of parameters is not fixed, it depends only on the number of arguments passed into `linearRegressionState`.
+They all must be numeric values.
+Note that the column with target value (which we would like to learn to predict) is inserted as the first argument.
+
+2. Predicting
+
+After saving a state into the table, we may use it multiple times for prediction or even merge with other states and create new, even better models.
+
+```sql
+WITH (SELECT state FROM your_model) AS model SELECT
+evalMLMethod(model, x1, x2) FROM test_data
+```
+
+The query will return a column of predicted values.
+Note that first argument of `evalMLMethod` is `AggregateFunctionState` object, next are columns of features.
+
+`test_data` is a table like `train_data` but may not contain target value.
+
+**Notes**
+
+1. To merge two models user may create such query:
+
+```sq;
+SELECT state1 + state2 FROM your_models
+```
+
+where the `your_models` table contains both models.
+This query will return a new `AggregateFunctionState` object.
+
+2. You may fetch weights of the created model for its own purposes without saving the model if no `-State` combinator is used.
+
+```sql
+SELECT stochasticLinearRegression(0.01)(target, param1, param2)
+FROM train_data
+```
+
+A query like this will fit the model and return its weights - first are weights, which correspond to the parameters of the model, the last one is bias.
+So in the example above the query will return a column with 3 values.
+)";
+    FunctionDocumentation::Syntax syntax_linear = "stochasticLinearRegression([learning_rate, l2_regularization_coef, mini_batch_size, method])(target, x1, x2, ...)";
+    FunctionDocumentation::Arguments arguments_linear = {
+        {"learning_rate", "The coefficient on step length when gradient descent step is performed. A learning rate that is too big may cause infinite weights of the model. Default is `0.00001`.", {"Float64"}},
+        {"l2_regularization_coef", "L2 regularization coefficient which may help to prevent overfitting. Default is `0.1`.", {"Float64"}},
+        {"mini_batch_size", "Sets the number of elements which gradients will be computed and summed to perform one step of gradient descent. Pure stochastic descent uses one element, however having small batches (about 10 elements) makes gradient steps more stable. Default is `15`.", {"UInt64"}},
+        {"method", "Method for updating weights: `Adam` (by default), `SGD`, `Momentum`, `Nesterov`. `Momentum` and `Nesterov` require slightly more computations and memory, however they happen to be useful in terms of speed of convergence and stability of stochastic gradient methods.", {"const String"}},
+        {"target", "Target value (dependent variable) to learn to predict. Must be numeric.", {"Float*"}},
+        {"x1, x2, ...", "Feature values (independent variables). All must be numeric.", {"Float*"}}
+    };
+    FunctionDocumentation::ReturnedValue returned_value_linear = {"Returns the trained linear regression model weights. First values correspond to the parameters of the model, the last one is bias. Use `evalMLMethod` for predictions.", {"Array(Float64)"}};
+    FunctionDocumentation::Examples examples_linear = {
+    {
+        "Training a model",
+        R"(
+CREATE TABLE your_model
+ENGINE = Memory
+AS SELECT
+stochasticLinearRegressionState(0.1, 0.0, 5, 'SGD')(target, x1, x2)
+AS state FROM train_data
+        )",
+        "Saves trained model state to table"
+    },
+    {
+        "Making predictions",
+         R"(
+WITH (SELECT state FROM your_model) AS model SELECT
+evalMLMethod(model, x1, x2) FROM test_data
+        )",
+        "Returns predicted values for test data"
+    },
+    {
+        "Getting model weights",
+        "SELECT stochasticLinearRegression(0.01)(target, x1, x2) FROM train_data",
+        "Returns model weights without saving state"
+    }
+    };
+    FunctionDocumentation::IntroducedIn introduced_in_linear = {20, 1};
+    FunctionDocumentation::Category category_linear = FunctionDocumentation::Category::MachineLearning;
+    FunctionDocumentation documentation_linear = {description_linear, syntax_linear, arguments_linear, returned_value_linear, examples_linear, introduced_in_linear, category_linear};
+
+    factory.registerFunction("stochasticLinearRegression", {createAggregateFunctionMLMethod<FuncLinearRegression>, {}, documentation_linear});
+
+    // stochasticLogisticRegression documentation
+    FunctionDocumentation::Description description_logistic = R"(
+This function implements stochastic logistic regression.
+It can be used for binary classification problem, supports the same custom parameters as [`stochasticLinearRegression`](/sql-reference/aggregate-functions/reference/stochasticlinearregression) and works the same way.
+
+**Usage**
+
+The function is used in two steps:
+
+1. Fitting
+
+For fitting a query like this can be used:
+
+```sql
+CREATE TABLE IF NOT EXISTS train_data
+(
+    param1 Float64,
+    param2 Float64,
+    target Float64
+) ENGINE = Memory;
+
+CREATE TABLE your_model ENGINE = Memory AS SELECT
+stochasticLogisticRegression(0.1, 0.0, 5, 'SGD')(target, x1, x2)
+AS state FROM train_data;
+```
+
+Here, we also need to insert data into the `train_data` table.
+The number of parameters is not fixed, it depends only on the number of arguments passed into `logisticRegressionState`.
+They all must be numeric values.
+Note that the column with target value (which we would like to learn to predict) is inserted as the first argument.
+
+Predicted labels have to be in [-1, 1].
+
+2. Predicting
+
+Using saved state we can predict the probability of an object having label `1`.
+
+```sql
+WITH (SELECT state FROM your_model) AS model SELECT
+evalMLMethod(model, param1, param2) FROM test_data
+```
+
+The query will return a column of probabilities.
+Note that first argument of `evalMLMethod` is an `AggregateFunctionState` object, next are columns of features.
+
+We can also set a bound of probability, which assigns elements to different labels.
+
+```sql
+SELECT result < 1.1 AND result > 0.5 FROM
+(WITH (SELECT state FROM your_model) AS model SELECT
+evalMLMethod(model, param1, param2) AS result FROM test_data)
+```
+
+Then the result will be labels.
+
+`test_data` is a table like `train_data` but may not contain target value.
+)";
+    FunctionDocumentation::Syntax syntax_logistic = "stochasticLogisticRegression([learning_rate, l2_regularization_coef, mini_batch_size, method])(target, x1, x2, ...)";
+    FunctionDocumentation::Arguments arguments_logistic = {
+        {"learning_rate", "The coefficient on step length when gradient descent step is performed. A learning rate that is too big may cause infinite weights of the model. Default is `0.00001`.", {"Float64"}},
+        {"l2_regularization_coef", "L2 regularization coefficient which may help to prevent overfitting. Default is `0.1`.", {"Float64"}},
+        {"mini_batch_size", "Sets the number of elements which gradients will be computed and summed to perform one step of gradient descent. Pure stochastic descent uses one element, however having small batches (about 10 elements) makes gradient steps more stable. Default is `15`.", {"UInt64"}},
+        {"method", "Method for updating weights: `Adam` (by default), `SGD`, `Momentum`, `Nesterov`. `Momentum` and `Nesterov` require a little bit more computations and memory, however they happen to be useful in terms of speed of convergence and stability of stochastic gradient methods.", {"String"}},
+        {"target", "Target binary classification labels. Must be in range [-1, 1].", {"Float"}},
+        {"x1, x2, ...", "Feature values (independent variables). All must be numeric.", {"Float"}}
+    };
+    FunctionDocumentation::ReturnedValue returned_value_logistic = {"Returns the trained logistic regression model weights. Use `evalMLMethod` for predictions which returns probabilities of object having label `1`.", {"Array(Float64)"}};
+    FunctionDocumentation::Examples examples_logistic = {
+    {
+        "Training a model",
+        R"(
+CREATE TABLE your_model
+ENGINE = MergeTree
+ORDER BY tuple()
+AS SELECT
+stochasticLogisticRegressionState(1.0, 1.0, 10, 'SGD')(target, x1, x2)
+AS state FROM train_data
+        )",
+        "Saves trained model state to table"
+    },
+    {
+        "Making predictions",
+        R"(
+WITH (SELECT state FROM your_model) AS model
+SELECT
+evalMLMethod(model, x1, x2)
+FROM test_data
+        )",
+        "Returns probability values for test data"
+    },
+    {
+        "Classification with threshold",
+        R"(
+SELECT result < 1.1 AND result > 0.5
+FROM (
+WITH (SELECT state FROM your_model) AS model SELECT
+evalMLMethod(model, x1, x2) AS result FROM test_data)
+        )",
+        "Returns binary classification labels using probability threshold"
+    }
+    };
+    FunctionDocumentation::IntroducedIn introduced_in_logistic = {20, 1};
+    FunctionDocumentation::Category category_logistic = FunctionDocumentation::Category::MachineLearning;
+    FunctionDocumentation documentation_logistic = {description_logistic, syntax_logistic, arguments_logistic, returned_value_logistic, examples_logistic, introduced_in_logistic, category_logistic};
+
+    factory.registerFunction("stochasticLogisticRegression", {createAggregateFunctionMLMethod<FuncLogisticRegression>, {}, documentation_logistic});
 }
 
 LinearModelData::LinearModelData(
@@ -251,15 +466,8 @@ void Adam::merge(const IWeightsUpdater & rhs, Float64 frac, Float64 rhs_frac)
     if (adam_rhs.average_gradient.empty())
         return;
 
-    if (average_gradient.empty())
-    {
-        if (!average_squared_gradient.empty() ||
-                adam_rhs.average_gradient.size() != adam_rhs.average_squared_gradient.size())
-            throw Exception("Average_gradient and average_squared_gradient must have same size", ErrorCodes::LOGICAL_ERROR);
-
-        average_gradient.resize(adam_rhs.average_gradient.size(), Float64{0.0});
-        average_squared_gradient.resize(adam_rhs.average_squared_gradient.size(), Float64{0.0});
-    }
+    average_gradient.resize(adam_rhs.average_gradient.size(), Float64{0.0});
+    average_squared_gradient.resize(adam_rhs.average_squared_gradient.size(), Float64{0.0});
 
     for (size_t i = 0; i < average_gradient.size(); ++i)
     {
@@ -272,14 +480,8 @@ void Adam::merge(const IWeightsUpdater & rhs, Float64 frac, Float64 rhs_frac)
 
 void Adam::update(UInt64 batch_size, std::vector<Float64> & weights, Float64 & bias, Float64 learning_rate, const std::vector<Float64> & batch_gradient)
 {
-    if (average_gradient.empty())
-    {
-        if (!average_squared_gradient.empty())
-            throw Exception("Average_gradient and average_squared_gradient must have same size", ErrorCodes::LOGICAL_ERROR);
-
-        average_gradient.resize(batch_gradient.size(), Float64{0.0});
-        average_squared_gradient.resize(batch_gradient.size(), Float64{0.0});
-    }
+    average_gradient.resize(batch_gradient.size(), Float64{0.0});
+    average_squared_gradient.resize(batch_gradient.size(), Float64{0.0});
 
     for (size_t i = 0; i != average_gradient.size(); ++i)
     {
@@ -332,8 +534,7 @@ void Nesterov::write(WriteBuffer & buf) const
 void Nesterov::merge(const IWeightsUpdater & rhs, Float64 frac, Float64 rhs_frac)
 {
     const auto & nesterov_rhs = static_cast<const Nesterov &>(rhs);
-    if (accumulated_gradient.empty())
-        accumulated_gradient.resize(nesterov_rhs.accumulated_gradient.size(), Float64{0.0});
+    accumulated_gradient.resize(nesterov_rhs.accumulated_gradient.size(), Float64{0.0});
 
     for (size_t i = 0; i < accumulated_gradient.size(); ++i)
     {
@@ -343,10 +544,7 @@ void Nesterov::merge(const IWeightsUpdater & rhs, Float64 frac, Float64 rhs_frac
 
 void Nesterov::update(UInt64 batch_size, std::vector<Float64> & weights, Float64 & bias, Float64 learning_rate, const std::vector<Float64> & batch_gradient)
 {
-    if (accumulated_gradient.empty())
-    {
-        accumulated_gradient.resize(batch_gradient.size(), Float64{0.0});
-    }
+    accumulated_gradient.resize(batch_gradient.size(), Float64{0.0});
 
     for (size_t i = 0; i < batch_gradient.size(); ++i)
     {
@@ -406,10 +604,7 @@ void Momentum::merge(const IWeightsUpdater & rhs, Float64 frac, Float64 rhs_frac
 void Momentum::update(UInt64 batch_size, std::vector<Float64> & weights, Float64 & bias, Float64 learning_rate, const std::vector<Float64> & batch_gradient)
 {
     /// batch_size is already checked to be greater than 0
-    if (accumulated_gradient.empty())
-    {
-        accumulated_gradient.resize(batch_gradient.size(), Float64{0.0});
-    }
+    accumulated_gradient.resize(batch_gradient.size(), Float64{0.0});
 
     for (size_t i = 0; i < batch_gradient.size(); ++i)
     {
@@ -460,9 +655,9 @@ void LogisticRegression::predict(
     size_t rows_num = arguments.front().column->size();
 
     if (offset > rows_num || offset + limit > rows_num)
-        throw Exception("Invalid offset and limit for LogisticRegression::predict. "
-                        "Block has " + toString(rows_num) + " rows, but offset is " + toString(offset) +
-                        " and limit is " + toString(limit), ErrorCodes::LOGICAL_ERROR);
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Invalid offset and limit for LogisticRegression::predict. "
+                        "Block has {} rows, but offset is {} and limit is {}",
+                        rows_num, offset, toString(limit));
 
     std::vector<Float64> results(limit, bias);
 
@@ -471,7 +666,7 @@ void LogisticRegression::predict(
         const ColumnWithTypeAndName & cur_col = arguments[i];
 
         if (!isNativeNumber(cur_col.type))
-            throw Exception("Prediction arguments must have numeric type", ErrorCodes::BAD_ARGUMENTS);
+            throw Exception(ErrorCodes::BAD_ARGUMENTS, "Prediction arguments must have numeric type");
 
         const auto & features_column = cur_col.column;
 
@@ -527,15 +722,15 @@ void LinearRegression::predict(
 {
     if (weights.size() + 1 != arguments.size())
     {
-        throw Exception("In predict function number of arguments differs from the size of weights vector", ErrorCodes::LOGICAL_ERROR);
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "In predict function number of arguments differs from the size of weights vector");
     }
 
     size_t rows_num = arguments.front().column->size();
 
     if (offset > rows_num || offset + limit > rows_num)
-        throw Exception("Invalid offset and limit for LogisticRegression::predict. "
-                        "Block has " + toString(rows_num) + " rows, but offset is " + toString(offset) +
-                        " and limit is " + toString(limit), ErrorCodes::LOGICAL_ERROR);
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Invalid offset and limit for LogisticRegression::predict. "
+                        "Block has {} rows, but offset is {} and limit is {}",
+                        rows_num, offset, toString(limit));
 
     std::vector<Float64> results(limit, bias);
 
@@ -544,12 +739,12 @@ void LinearRegression::predict(
         const ColumnWithTypeAndName & cur_col = arguments[i];
 
         if (!isNativeNumber(cur_col.type))
-            throw Exception("Prediction arguments must have numeric type", ErrorCodes::BAD_ARGUMENTS);
+            throw Exception(ErrorCodes::BAD_ARGUMENTS, "Prediction arguments must have numeric type");
 
         auto features_column = cur_col.column;
 
         if (!features_column)
-            throw Exception("Unexpectedly cannot dynamically cast features column " + std::to_string(i), ErrorCodes::LOGICAL_ERROR);
+            throw Exception(ErrorCodes::LOGICAL_ERROR, "Unexpectedly cannot dynamically cast features column {}", i);
 
         for (size_t row_num = 0; row_num < limit; ++row_num)
             results[row_num] += weights[i - 1] * features_column->getFloat64(row_num + offset);
