@@ -17,7 +17,7 @@ static auto getKeyHolder(const IColumn & column, size_t row_num, Arena & arena)
     else
     {
         const char * begin = nullptr;
-        StringRef serialized = column.serializeValueIntoArena(row_num, arena, begin);
+        StringRef serialized = column.serializeAggregationStateValueIntoArena(row_num, arena, begin);
         assert(serialized.data != nullptr);
         return SerializedKeyHolder{serialized, arena};
     }
@@ -29,7 +29,7 @@ static void deserializeAndInsert(StringRef str, IColumn & data_to)
     if constexpr (is_plain_column)
         data_to.insertData(str.data, str.size);
     else
-        data_to.deserializeAndInsertFromArena(str.data);
+        std::ignore = data_to.deserializeAndInsertAggregationStateValueFromArena(str.data);
 }
 
 }
