@@ -16,6 +16,7 @@ struct Settings;
 namespace ErrorCodes
 {
     extern const int LOGICAL_ERROR;
+    extern const int INCORRECT_DATA;
 }
 
 /** Calculates quantile for time in milliseconds, less than 30 seconds.
@@ -34,7 +35,7 @@ namespace ErrorCodes
   * -- for values from 0 to 1023 - in increments of 1;
   * -- for values from 1024 to 30,000 - in increments of 16;
   *
-  * NOTE: 64-bit integer weight can overflow, see also QantileExactWeighted.h::get()
+  * NOTE: 64-bit integer weight can overflow, see also QuantileExactWeighted.h::get()
   */
 
 #define TINY_MAX_ELEMS 31
@@ -77,14 +78,18 @@ namespace detail
 
         void serialize(WriteBuffer & buf) const
         {
-            writeBinary(count, buf);
+            writeBinaryLittleEndian(count, buf);
             buf.write(reinterpret_cast<const char *>(elems), count * sizeof(elems[0]));
         }
 
         void deserialize(ReadBuffer & buf)
         {
-            readBinary(count, buf);
-            buf.readStrict(reinterpret_cast<char *>(elems), count * sizeof(elems[0]));
+            UInt16 new_count = 0;
+            readBinaryLittleEndian(new_count, buf);
+            if (new_count > TINY_MAX_ELEMS)
+                throw Exception(ErrorCodes::INCORRECT_DATA, "The number of elements {} for the 'tiny' kind of quantileTiming is exceeding the maximum of {}", new_count, TINY_MAX_ELEMS);
+            buf.readStrict(reinterpret_cast<char *>(elems), new_count * sizeof(elems[0]));
+            count = new_count;
         }
 
         /** This function must be called before get-functions. */
@@ -159,14 +164,17 @@ namespace detail
 
         void serialize(WriteBuffer & buf) const
         {
-            writeBinary(elems.size(), buf);
+            writeBinaryLittleEndian(elems.size(), buf);
             buf.write(reinterpret_cast<const char *>(elems.data()), elems.size() * sizeof(elems[0]));
         }
 
         void deserialize(ReadBuffer & buf)
         {
             size_t size = 0;
-            readBinary(size, buf);
+            readBinaryLittleEndian(size, buf);
+            if (size > 10'000)
+                throw Exception(ErrorCodes::INCORRECT_DATA, "The number of elements {} for the 'medium' kind of quantileTiming is too large", size);
+
             elems.resize(size);
             buf.readStrict(reinterpret_cast<char *>(elems.data()), size * sizeof(elems[0]));
         }
@@ -178,7 +186,7 @@ namespace detail
             if (!elems.empty())
             {
                 size_t n = level < 1
-                    ? level * elems.size()
+                    ? static_cast<size_t>(level * elems.size())
                     : (elems.size() - 1);
 
                 /// Sorting an array will not be considered a violation of constancy.
@@ -201,7 +209,7 @@ namespace detail
                 auto level = levels[level_index];
 
                 size_t n = level < 1
-                    ? level * elems.size()
+                    ? static_cast<size_t>(level * elems.size())
                     : (elems.size() - 1);
 
                 ::nth_element(array.begin() + prev_n, array.begin() + n, array.end());
@@ -254,7 +262,7 @@ namespace detail
         UInt64 count_big[BIG_SIZE];
 
         /// Get value of quantile by index in array `count_big`.
-        static inline UInt16 indexInBigToValue(size_t i)
+        static UInt16 indexInBigToValue(size_t i)
         {
             return (i * BIG_PRECISION) + SMALL_THRESHOLD
                 + (intHash32<0>(i) % BIG_PRECISION - (BIG_PRECISION / 2));    /// A small randomization so that it is not noticeable that all the values are even.
@@ -333,7 +341,7 @@ namespace detail
 
         void serialize(WriteBuffer & buf) const
         {
-            writeBinary(count, buf);
+            writeBinaryLittleEndian(count, buf);
 
             if (count * 2 > SMALL_THRESHOLD + BIG_SIZE)
             {
@@ -348,8 +356,8 @@ namespace detail
                 {
                     if (count_small[i])
                     {
-                        writeBinary(UInt16(i), buf);
-                        writeBinary(count_small[i], buf);
+                        writeBinaryLittleEndian(UInt16(i), buf);
+                        writeBinaryLittleEndian(count_small[i], buf);
                     }
                 }
 
@@ -357,19 +365,19 @@ namespace detail
                 {
                     if (count_big[i])
                     {
-                        writeBinary(UInt16(i + SMALL_THRESHOLD), buf);
-                        writeBinary(count_big[i], buf);
+                        writeBinaryLittleEndian(UInt16(i + SMALL_THRESHOLD), buf);
+                        writeBinaryLittleEndian(count_big[i], buf);
                     }
                 }
 
                 /// Symbolizes end of data.
-                writeBinary(UInt16(BIG_THRESHOLD), buf);
+                writeBinaryLittleEndian(UInt16(BIG_THRESHOLD), buf);
             }
         }
 
         void deserialize(ReadBuffer & buf)
         {
-            readBinary(count, buf);
+            readBinaryLittleEndian(count, buf);
 
             if (count * 2 > SMALL_THRESHOLD + BIG_SIZE)
             {
@@ -380,12 +388,12 @@ namespace detail
                 while (true)
                 {
                     UInt16 index = 0;
-                    readBinary(index, buf);
+                    readBinaryLittleEndian(index, buf);
                     if (index == BIG_THRESHOLD)
                         break;
 
                     UInt64 elem_count = 0;
-                    readBinary(elem_count, buf);
+                    readBinaryLittleEndian(elem_count, buf);
 
                     if (index < SMALL_THRESHOLD)
                         count_small[index] = elem_count;
@@ -563,8 +571,11 @@ public:
         }
     }
 
-    void add(UInt64 x)
+    template <typename T>
+    void add(T x_)
     {
+        UInt64 x = static_cast<UInt64>(x_);
+
         if (tiny.count < TINY_MAX_ELEMS)
         {
             tiny.insert(x);
@@ -589,8 +600,11 @@ public:
         }
     }
 
-    void add(UInt64 x, size_t weight)
+    template <typename T>
+    void add(T x_, size_t weight)
     {
+        UInt64 x = static_cast<UInt64>(x_);
+
         /// NOTE: First condition is to avoid overflow.
         if (weight < TINY_MAX_ELEMS && tiny.count + weight <= TINY_MAX_ELEMS)
         {
@@ -664,7 +678,7 @@ public:
                     large->insert(elem);
             }
             else
-                throw Exception("Logical error in QuantileTiming::merge function: not all cases are covered", ErrorCodes::LOGICAL_ERROR);
+                throw Exception(ErrorCodes::LOGICAL_ERROR, "Logical error in QuantileTiming::merge function: not all cases are covered");
 
             /// For determinism, we should always convert to `large` when size condition is reached
             ///  - regardless of merge order.
@@ -678,7 +692,7 @@ public:
     void serialize(WriteBuffer & buf) const
     {
         auto kind = which();
-        DB::writePODBinary(kind, buf);
+        writeBinaryLittleEndian(kind, buf);
 
         if (kind == Kind::Tiny)
             tiny.serialize(buf);
@@ -692,7 +706,7 @@ public:
     void deserialize(ReadBuffer & buf)
     {
         Kind kind;
-        DB::readPODBinary(kind, buf);
+        readBinaryLittleEndian(kind, buf);
 
         if (kind == Kind::Tiny)
         {
@@ -708,6 +722,8 @@ public:
             tinyToLarge();
             large->deserialize(buf);
         }
+        else
+            throw Exception(ErrorCodes::INCORRECT_DATA, "Incorrect kind of QuantileTiming");
     }
 
     /// Get the value of the `level` quantile. The level must be between 0 and 1.
@@ -720,14 +736,12 @@ public:
             tiny.prepare();
             return tiny.get(level);
         }
-        else if (kind == Kind::Medium)
+        if (kind == Kind::Medium)
         {
             return medium.get(level);
         }
-        else
-        {
-            return large->get(level);
-        }
+
+        return large->get(level);
     }
 
     /// Get the size values of the quantiles of the `levels` levels. Record `size` results starting with `result` address.
@@ -767,6 +781,16 @@ public:
             for (size_t i = 0; i < size; ++i)
                 result[i] = std::numeric_limits<float>::quiet_NaN();
     }
+
+    friend void writeBinary(const Kind & x, WriteBuffer & buf)
+    {
+        writePODBinary(x, buf);
+    }
+
+    friend void readBinary(Kind & x, ReadBuffer & buf)
+    {
+        readPODBinary(x, buf);
+    }
 };
 
 #undef SMALL_THRESHOLD
