@@ -1,11 +1,11 @@
 #pragma once
-#include <IO/WriteBuffer.h>
-#include <Core/Block.h>
-
 
 namespace DB
 {
 
+class Block;
+class WriteBuffer;
+
 /// Prints internal server logs or profile events with colored output (if requested).
 /// NOTE: IRowOutputFormat does not suite well for this case
 class InternalTextLogs
@@ -38,10 +38,7 @@ public:
     /// See also TCPHandler::sendProfileEvents() for block columns.
     void writeProfileEvents(const Block & block);
 
-    void flush()
-    {
-        wb.next();
-    }
+    void flush();
 
 private:
     WriteBuffer & wb;
