@@ -3,7 +3,7 @@
   * This file provides forward declarations for Allocator.
   */
 
-template <bool clear_memory_, bool mmap_populate = false>
+template <bool clear_memory_, bool populate = false>
 class Allocator;
 
 template <typename Base, size_t N = 64, size_t Alignment = 1>
