@@ -1,12 +1,10 @@
 #pragma once
 
-#include <Common/config.h>
-
-#include <string>
+#include "config.h"
 
+#include <base/types.h>
 #include <Poco/DOM/Document.h>
-#include "Poco/DOM/AutoPtr.h"
-#include <base/logger_useful.h>
+#include <Poco/DOM/AutoPtr.h>
 
 #if USE_YAML_CPP
 
@@ -29,21 +27,11 @@ using YAMLParser = YAMLParserImpl;
 namespace DB
 {
 
-namespace ErrorCodes
-{
-    extern const int CANNOT_PARSE_YAML;
-}
-
 /// Fake YAML parser: throws an exception if we try to parse YAML configs in a build without yaml-cpp
 class DummyYAMLParser
 {
 public:
-    static Poco::AutoPtr<Poco::XML::Document> parse(const String& path)
-    {
-        Poco::AutoPtr<Poco::XML::Document> xml = new Poco::XML::Document;
-        throw Exception(ErrorCodes::CANNOT_PARSE_YAML, "Unable to parse YAML configuration file {} without usage of yaml-cpp library", path);
-        return xml;
-    }
+    static Poco::AutoPtr<Poco::XML::Document> parse(const String & path);
 };
 
 using YAMLParser = DummyYAMLParser;
