@@ -5,7 +5,6 @@
 #include <filesystem>
 #include <Common/filesystemHelpers.h>
 
-namespace fs = std::filesystem;
 
 class FileUpdatesTracker
 {
@@ -14,7 +13,7 @@ private:
     Poco::Timestamp known_time;
 
 public:
-    FileUpdatesTracker(const std::string & path_)
+    explicit FileUpdatesTracker(const std::string & path_)
         : path(path_)
         , known_time(0)
     {}
