@@ -16,9 +16,9 @@ class WriteBuffer;
 class StatusFile : private boost::noncopyable
 {
 public:
-    using FillFunction = std::function<void(WriteBuffer&)>;
+    using FillFunction = std::function<void(WriteBuffer &)>;
 
-    StatusFile(std::string path_, FillFunction fill_);
+    StatusFile(std::string path_, FillFunction fill);
     ~StatusFile();
 
     /// You can use one of these functions to fill the file or provide your own.
@@ -27,7 +27,6 @@ public:
 
 private:
     const std::string path;
-    FillFunction fill;
     int fd = -1;
 };
 
