@@ -1,19 +1,16 @@
-#if defined(__ELF__) && !defined(__FreeBSD__)
+#include <base/defines.h>
 
+#include <base/MemorySanitizer.h>
+#include <base/hex.h>
+#include <base/sort.h>
+#include <Common/MemoryTrackerDebugBlockerInThread.h>
 #include <Common/SymbolIndex.h>
-#include <Common/hex.h>
 
 #include <algorithm>
 #include <optional>
-#include <cassert>
 
-#include <link.h>
-
-//#include <iostream>
 #include <filesystem>
 
-#include <base/sort.h>
-
 /**
 
 ELF object can contain three different places with symbol names and addresses:
@@ -37,7 +34,7 @@ But because ClickHouse is linked with most of the symbols exported (-rdynamic fl
 It allows to get source file names and line numbers from addresses. Only available if you use -g option for compiler.
 It is also used by default for ClickHouse builds, but because of its weight (about two gigabytes)
 it is split to separate binary and provided in clickhouse-common-static-dbg package.
-This separate binary is placed in /usr/lib/debug/usr/bin/clickhouse and is loaded automatically by tools like gdb, addr2line.
+This separate binary is placed in /usr/lib/debug/usr/bin/clickhouse.debug and is loaded automatically by tools like gdb, addr2line.
 When you build ClickHouse by yourself, debug info is not split and present in a single huge binary.
 
 What ClickHouse is using to provide good stack traces?
@@ -57,18 +54,21 @@ Otherwise you will get only exported symbols from program headers.
 
 */
 
-#if defined(__clang__)
-#   pragma clang diagnostic ignored "-Wreserved-id-macro"
-#   pragma clang diagnostic ignored "-Wunused-macros"
-#endif
+extern "C" struct dl_phdr_info
+{
+    uint64_t addr;
+    const char * name;
+    const ElfProgramHeader * phdr;
+    uint16_t phnum;
+    uint64_t adds;
+    uint64_t subs;
+    size_t tls_modid;
+    void * tls_data;
+};
 
-#define __msan_unpoison_string(X) // NOLINT
-#if defined(ch_has_feature)
-#    if ch_has_feature(memory_sanitizer)
-#        undef __msan_unpoison_string
-#        include <sanitizer/msan_interface.h>
-#    endif
-#endif
+using DynamicLinkingProgramHeaderInfo = dl_phdr_info;
+
+extern "C" int dl_iterate_phdr(int (*)(DynamicLinkingProgramHeaderInfo *, size_t, void *), void *);
 
 
 namespace DB
@@ -88,114 +88,91 @@ namespace
 /// https://stackoverflow.com/questions/32088140/multiple-string-tables-in-elf-object
 
 
-void updateResources(ElfW(Addr) base_address, std::string_view object_name, std::string_view name, const void * address, SymbolIndex::Resources & resources)
-{
-    const char * char_address = static_cast<const char *>(address);
-
-    if (name.starts_with("_binary_") || name.starts_with("binary_"))
-    {
-        if (name.ends_with("_start"))
-        {
-            name = name.substr((name[0] == '_') + strlen("binary_"));
-            name = name.substr(0, name.size() - strlen("_start"));
-
-            resources.emplace(name, SymbolIndex::ResourcesBlob{
-                base_address,
-                object_name,
-                std::string_view{char_address, 0}, // NOLINT
-            });
-        }
-        else if (name.ends_with("_end"))
-        {
-            name = name.substr((name[0] == '_') + strlen("binary_"));
-            name = name.substr(0, name.size() - strlen("_end"));
-
-            auto it = resources.find(name);
-            if (it != resources.end() && it->second.base_address == base_address && it->second.data.empty())
-            {
-                const char * start = it->second.data.data();
-                assert(char_address >= start);
-                it->second.data = std::string_view{start, static_cast<size_t>(char_address - start)};
-            }
-        }
-    }
-}
-
-
 /// Based on the code of musl-libc and the answer of Kanalpiroge on
 /// https://stackoverflow.com/questions/15779185/list-all-the-functions-symbols-on-the-fly-in-c-code-on-a-linux-architecture
 /// It does not extract all the symbols (but only public - exported and used for dynamic linking),
 /// but will work if we cannot find or parse ELF files.
 void collectSymbolsFromProgramHeaders(
-    dl_phdr_info * info,
-    std::vector<SymbolIndex::Symbol> & symbols,
-    SymbolIndex::Resources & resources)
+    DynamicLinkingProgramHeaderInfo * info,
+    std::vector<SymbolIndex::Symbol> & symbols)
 {
     /* Iterate over all headers of the current shared lib
      * (first call is for the executable itself)
      */
-    for (size_t header_index = 0; header_index < info->dlpi_phnum; ++header_index)
+    __msan_unpoison(&info->phnum, sizeof(info->phnum));
+    __msan_unpoison(&info->phdr, sizeof(info->phdr));
+    for (size_t header_index = 0; header_index < info->phnum; ++header_index)
     {
         /* Further processing is only needed if the dynamic section is reached
          */
-        if (info->dlpi_phdr[header_index].p_type != PT_DYNAMIC)
+        __msan_unpoison(&info->phdr[header_index], sizeof(info->phdr[header_index]));
+        if (info->phdr[header_index].type != ProgramHeaderType::DYNAMIC)
             continue;
 
         /* Get a pointer to the first entry of the dynamic section.
          * It's address is the shared lib's address + the virtual address
          */
-        const ElfW(Dyn) * dyn_begin = reinterpret_cast<const ElfW(Dyn) *>(info->dlpi_addr + info->dlpi_phdr[header_index].p_vaddr);
+        const ElfDyn * dyn_begin = reinterpret_cast<const ElfDyn *>(info->addr + info->phdr[header_index].vaddr);
+        __msan_unpoison(&dyn_begin, sizeof(dyn_begin));
 
         /// For unknown reason, addresses are sometimes relative sometimes absolute.
-        auto correct_address = [](ElfW(Addr) base, ElfW(Addr) ptr)
+        auto correct_address = [](uint64_t base, uint64_t ptr)
         {
             return ptr > base ? ptr : base + ptr;
         };
 
         /* Iterate over all entries of the dynamic section until the
          * end of the symbol table is reached. This is indicated by
-         * an entry with d_tag == DT_NULL.
+         * an entry with d_tag == ProgramHeaderType::Null.
          */
 
         size_t sym_cnt = 0;
-        for (const auto * it = dyn_begin; it->d_tag != DT_NULL; ++it)
         {
-            ElfW(Addr) base_address = correct_address(info->dlpi_addr, it->d_un.d_ptr);
-
-            // TODO: this branch leads to invalid address of the hash table. Need further investigation.
-            // if (it->d_tag == DT_HASH)
-            // {
-            //     const ElfW(Word) * hash = reinterpret_cast<const ElfW(Word) *>(base_address);
-            //     sym_cnt = hash[1];
-            //     break;
-            // }
-            if (it->d_tag == DT_GNU_HASH)
+            const auto * it = dyn_begin;
+            while (true)
             {
-                /// This code based on Musl-libc.
+                __msan_unpoison(it, sizeof(*it));
+                if (it->tag != DynamicTableTag::Null)
+                    break;
 
-                const uint32_t * buckets = nullptr;
-                const uint32_t * hashval = nullptr;
+                uint64_t base_address = correct_address(info->addr, it->ptr);
 
-                const ElfW(Word) * hash = reinterpret_cast<const ElfW(Word) *>(base_address);
-
-                buckets = hash + 4 + (hash[2] * sizeof(size_t) / 4);
-
-                for (ElfW(Word) i = 0; i < hash[0]; ++i)
-                    if (buckets[i] > sym_cnt)
-                        sym_cnt = buckets[i];
-
-                if (sym_cnt)
+                if (it->tag == DynamicTableTag::GNU_HASH)
                 {
-                    sym_cnt -= hash[1];
-                    hashval = buckets + hash[0] + sym_cnt;
-                    do
+                    /// This code based on Musl-libc.
+
+                    const uint32_t * buckets = nullptr;
+                    const uint32_t * hashval = nullptr;
+
+                    const uint32_t * hash = reinterpret_cast<const uint32_t *>(base_address);
+
+                    __msan_unpoison(&hash[0], sizeof(*hash));
+                    __msan_unpoison(&hash[1], sizeof(*hash));
+                    __msan_unpoison(&hash[2], sizeof(*hash));
+
+                    buckets = hash + 4 + (hash[2] * sizeof(size_t) / 4);
+
+                    __msan_unpoison(buckets, hash[0] * sizeof(buckets[0]));
+
+                    for (uint32_t i = 0; i < hash[0]; ++i)
+                        sym_cnt = std::max<size_t>(sym_cnt, buckets[i]);
+
+                    if (sym_cnt)
                     {
-                        ++sym_cnt;
+                        sym_cnt -= hash[1];
+                        hashval = buckets + hash[0] + sym_cnt;
+                        __msan_unpoison(&hashval, sizeof(hashval));
+                        do
+                        {
+                            ++sym_cnt;
+                        }
+                        while (!(*hashval++ & 1));
                     }
-                    while (!(*hashval++ & 1));
+
+                    break;
                 }
 
-                break;
+                ++it;
             }
         }
 
@@ -203,11 +180,11 @@ void collectSymbolsFromProgramHeaders(
             continue;
 
         const char * strtab = nullptr;
-        for (const auto * it = dyn_begin; it->d_tag != DT_NULL; ++it)
+        for (const auto * it = dyn_begin; it->tag != DynamicTableTag::Null; ++it)
         {
-            ElfW(Addr) base_address = correct_address(info->dlpi_addr, it->d_un.d_ptr);
+            uint64_t base_address = correct_address(info->addr, it->ptr);
 
-            if (it->d_tag == DT_STRTAB)
+            if (it->tag == DynamicTableTag::STRTAB)
             {
                 strtab = reinterpret_cast<const char *>(base_address);
                 break;
@@ -217,39 +194,39 @@ void collectSymbolsFromProgramHeaders(
         if (!strtab)
             continue;
 
-        for (const auto * it = dyn_begin; it->d_tag != DT_NULL; ++it)
+        for (const auto * it = dyn_begin; it->tag != DynamicTableTag::Null; ++it)
         {
-            ElfW(Addr) base_address = correct_address(info->dlpi_addr, it->d_un.d_ptr);
+            uint64_t base_address = correct_address(info->addr, it->ptr);
 
-            if (it->d_tag == DT_SYMTAB)
+            if (it->tag == DynamicTableTag::SYMTAB)
             {
                 /* Get the pointer to the first entry of the symbol table */
-                const ElfW(Sym) * elf_sym = reinterpret_cast<const ElfW(Sym) *>(base_address);
+                const ElfSymbol * elf_sym = reinterpret_cast<const ElfSymbol *>(base_address);
+
+                __msan_unpoison(elf_sym, sym_cnt * sizeof(*elf_sym));
 
                 /* Iterate over the symbol table */
-                for (ElfW(Word) sym_index = 0; sym_index < ElfW(Word)(sym_cnt); ++sym_index)
+                for (uint32_t sym_index = 0; sym_index < uint32_t(sym_cnt); ++sym_index)
                 {
                     /* Get the name of the sym_index-th symbol.
                      * This is located at the address of st_name relative to the beginning of the string table.
                      */
-                    const char * sym_name = &strtab[elf_sym[sym_index].st_name];
+                    const char * sym_name = &strtab[elf_sym[sym_index].name];
+                    __msan_unpoison_string(sym_name);
 
                     if (!sym_name)
                         continue;
 
                     SymbolIndex::Symbol symbol;
-                    symbol.address_begin = reinterpret_cast<const void *>(
-                        info->dlpi_addr + elf_sym[sym_index].st_value);
-                    symbol.address_end = reinterpret_cast<const void *>(
-                        info->dlpi_addr + elf_sym[sym_index].st_value + elf_sym[sym_index].st_size);
+                    symbol.offset_begin = reinterpret_cast<const void *>(
+                        elf_sym[sym_index].value);
+                    symbol.offset_end = reinterpret_cast<const void *>(
+                        elf_sym[sym_index].value + elf_sym[sym_index].size);
                     symbol.name = sym_name;
 
                     /// We are not interested in empty symbols.
-                    if (elf_sym[sym_index].st_size)
+                    if (elf_sym[sym_index].size)
                         symbols.push_back(symbol);
-
-                    /// But resources can be represented by a pair of empty symbols (indicating their boundaries).
-                    updateResources(base_address, info->dlpi_name, symbol.name, symbol.address_begin, resources);
                 }
 
                 break;
@@ -260,15 +237,22 @@ void collectSymbolsFromProgramHeaders(
 
 
 #if !defined USE_MUSL
-String getBuildIDFromProgramHeaders(dl_phdr_info * info)
+String getBuildIDFromProgramHeaders(DynamicLinkingProgramHeaderInfo * info)
 {
-    for (size_t header_index = 0; header_index < info->dlpi_phnum; ++header_index)
+    __msan_unpoison(&info->phnum, sizeof(info->phnum));
+    __msan_unpoison(&info->phdr, sizeof(info->phdr));
+    for (size_t header_index = 0; header_index < info->phnum; ++header_index)
     {
-        const ElfPhdr & phdr = info->dlpi_phdr[header_index];
-        if (phdr.p_type != PT_NOTE)
+        const ElfProgramHeader & phdr = info->phdr[header_index];
+        __msan_unpoison(&phdr, sizeof(phdr));
+        if (phdr.type != ProgramHeaderType::NOTE)
             continue;
 
-        return Elf::getBuildID(reinterpret_cast<const char *>(info->dlpi_addr + phdr.p_vaddr), phdr.p_memsz);
+        std::string_view view(reinterpret_cast<const char *>(info->addr + phdr.vaddr), phdr.memsz);
+        __msan_unpoison(view.data(), view.size());
+        String build_id = Elf::getBuildID(view.data(), view.size());
+        if (!build_id.empty()) // there may be multiple PT_NOTE segments
+            return build_id;
     }
     return {};
 }
@@ -276,63 +260,57 @@ String getBuildIDFromProgramHeaders(dl_phdr_info * info)
 
 
 void collectSymbolsFromELFSymbolTable(
-    dl_phdr_info * info,
     const Elf & elf,
     const Elf::Section & symbol_table,
     const Elf::Section & string_table,
-    std::vector<SymbolIndex::Symbol> & symbols,
-    SymbolIndex::Resources & resources)
+    std::vector<SymbolIndex::Symbol> & symbols)
 {
     /// Iterate symbol table.
-    const ElfSym * symbol_table_entry = reinterpret_cast<const ElfSym *>(symbol_table.begin());
-    const ElfSym * symbol_table_end = reinterpret_cast<const ElfSym *>(symbol_table.end());
+    const ElfSymbol * symbol_table_entry = reinterpret_cast<const ElfSymbol *>(symbol_table.begin());
+    const ElfSymbol * symbol_table_end = reinterpret_cast<const ElfSymbol *>(symbol_table.end());
 
     const char * strings = string_table.begin();
 
     for (; symbol_table_entry < symbol_table_end; ++symbol_table_entry)
     {
-        if (!symbol_table_entry->st_name
-            || !symbol_table_entry->st_value
-            || strings + symbol_table_entry->st_name >= elf.end())
+        if (!symbol_table_entry->name
+            || !symbol_table_entry->value
+            || strings + symbol_table_entry->name >= elf.end())
             continue;
 
         /// Find the name in strings table.
-        const char * symbol_name = strings + symbol_table_entry->st_name;
+        const char * symbol_name = strings + symbol_table_entry->name;
 
         if (!symbol_name)
             continue;
 
         SymbolIndex::Symbol symbol;
-        symbol.address_begin = reinterpret_cast<const void *>(
-            info->dlpi_addr + symbol_table_entry->st_value);
-        symbol.address_end = reinterpret_cast<const void *>(
-            info->dlpi_addr + symbol_table_entry->st_value + symbol_table_entry->st_size);
+        symbol.offset_begin = reinterpret_cast<const void *>(
+            symbol_table_entry->value);
+        symbol.offset_end = reinterpret_cast<const void *>(
+            symbol_table_entry->value + symbol_table_entry->size);
         symbol.name = symbol_name;
 
-        if (symbol_table_entry->st_size)
+        if (symbol_table_entry->size)
             symbols.push_back(symbol);
-
-        updateResources(info->dlpi_addr, info->dlpi_name, symbol.name, symbol.address_begin, resources);
     }
 }
 
 
 bool searchAndCollectSymbolsFromELFSymbolTable(
-    dl_phdr_info * info,
     const Elf & elf,
-    unsigned section_header_type,
+    SectionHeaderType section_header_type,
     const char * string_table_name,
-    std::vector<SymbolIndex::Symbol> & symbols,
-    SymbolIndex::Resources & resources)
+    std::vector<SymbolIndex::Symbol> & symbols)
 {
     std::optional<Elf::Section> symbol_table;
     std::optional<Elf::Section> string_table;
 
     if (!elf.iterateSections([&](const Elf::Section & section, size_t)
         {
-            if (section.header.sh_type == section_header_type)
+            if (section.header.type == section_header_type)
                 symbol_table.emplace(section);
-            else if (section.header.sh_type == SHT_STRTAB && 0 == strcmp(section.name(), string_table_name))
+            else if (section.header.type == SectionHeaderType::STRTAB && 0 == strcmp(section.name(), string_table_name))
                 string_table.emplace(section);
 
             return (symbol_table && string_table);
@@ -341,31 +319,31 @@ bool searchAndCollectSymbolsFromELFSymbolTable(
         return false;
     }
 
-    collectSymbolsFromELFSymbolTable(info, elf, *symbol_table, *string_table, symbols, resources);
+    collectSymbolsFromELFSymbolTable(elf, *symbol_table, *string_table, symbols);
     return true;
 }
 
 
 void collectSymbolsFromELF(
-    dl_phdr_info * info,
+    DynamicLinkingProgramHeaderInfo * info,
     std::vector<SymbolIndex::Symbol> & symbols,
     std::vector<SymbolIndex::Object> & objects,
-    SymbolIndex::Resources & resources,
-    String & build_id)
+    String & self_build_id)
 {
     String object_name;
-    String our_build_id;
+    String build_id;
 
 #if defined (USE_MUSL)
     object_name = "/proc/self/exe";
-    our_build_id = Elf(object_name).getBuildID();
-    build_id = our_build_id;
+    build_id = Elf(object_name).getBuildID();
+    self_build_id = build_id;
 #else
     /// MSan does not know that the program segments in memory are initialized.
-    __msan_unpoison_string(info->dlpi_name);
+    __msan_unpoison(info, sizeof(*info));
+    __msan_unpoison_string(info->name);
 
-    object_name = info->dlpi_name;
-    our_build_id = getBuildIDFromProgramHeaders(info);
+    object_name = info->name;
+    build_id = getBuildIDFromProgramHeaders(info);
 
     /// If the name is empty and there is a non-empty build-id - it's main executable.
     /// Find a elf file for the main executable and set the build-id.
@@ -373,14 +351,15 @@ void collectSymbolsFromELF(
     {
         object_name = "/proc/self/exe";
 
-        if (our_build_id.empty())
-            our_build_id = Elf(object_name).getBuildID();
-
         if (build_id.empty())
-            build_id = our_build_id;
+            build_id = Elf(object_name).getBuildID();
+
+        if (self_build_id.empty())
+            self_build_id = build_id;
     }
 #endif
 
+    /// Note: we load ELF from file; this doesn't work for vdso because it's only present in memory.
     std::error_code ec;
     std::filesystem::path canonical_path = std::filesystem::canonical(object_name, ec);
 
@@ -391,10 +370,22 @@ void collectSymbolsFromELF(
     std::filesystem::path local_debug_info_path = canonical_path.parent_path() / canonical_path.stem();
     local_debug_info_path += ".debug";
     std::filesystem::path debug_info_path = std::filesystem::path("/usr/lib/debug") / canonical_path.relative_path();
+    debug_info_path += ".debug";
 
-    if (std::filesystem::exists(local_debug_info_path))
+    /// NOTE: This is a workaround for current package system.
+    ///
+    /// Since nfpm cannot copy file only if it exists,
+    /// and so in cmake empty .debug file is created instead,
+    /// but if we will try to load empty Elf file, then the CANNOT_PARSE_ELF
+    /// exception will be thrown from the Elf::Elf.
+    auto exists_not_empty = [](const std::filesystem::path & path)
+    {
+        return std::filesystem::exists(path) && !std::filesystem::is_empty(path);
+    };
+
+    if (exists_not_empty(local_debug_info_path))
         object_name = local_debug_info_path;
-    else if (std::filesystem::exists(debug_info_path))
+    else if (exists_not_empty(debug_info_path))
         object_name = debug_info_path;
     else if (build_id.size() >= 2)
     {
@@ -412,7 +403,7 @@ void collectSymbolsFromELF(
 
         std::filesystem::path build_id_debug_info_path(
             fmt::format("/usr/lib/debug/.build-id/{}/{}.debug", build_id_hex.substr(0, 2), build_id_hex.substr(2)));
-        if (std::filesystem::exists(build_id_debug_info_path))
+        if (exists_not_empty(build_id_debug_info_path))
             object_name = build_id_debug_info_path;
         else
             object_name = canonical_path;
@@ -427,9 +418,10 @@ void collectSymbolsFromELF(
 
     String file_build_id = object.elf->getBuildID();
 
-    if (our_build_id != file_build_id)
+    if (build_id != file_build_id)
     {
-        /// If debug info doesn't correspond to our binary, fallback to the info in our binary.
+        /// If the separate debuginfo binary doesn't correspond to the loaded binary, fallback to
+        /// the info in the loaded binary.
         if (object_name != canonical_path)
         {
             object_name = canonical_path;
@@ -437,23 +429,23 @@ void collectSymbolsFromELF(
 
             /// But it can still be outdated, for example, if executable file was deleted from filesystem and replaced by another file.
             file_build_id = object.elf->getBuildID();
-            if (our_build_id != file_build_id)
+            if (build_id != file_build_id)
                 return;
         }
         else
             return;
     }
 
-    object.address_begin = reinterpret_cast<const void *>(info->dlpi_addr);
-    object.address_end = reinterpret_cast<const void *>(info->dlpi_addr + object.elf->size());
+    object.address_begin = reinterpret_cast<const void *>(info->addr);
+    object.address_end = reinterpret_cast<const void *>(info->addr + object.elf->size());
     object.name = object_name;
     objects.push_back(std::move(object));
 
-    searchAndCollectSymbolsFromELFSymbolTable(info, *objects.back().elf, SHT_SYMTAB, ".strtab", symbols, resources);
+    searchAndCollectSymbolsFromELFSymbolTable(*objects.back().elf, SectionHeaderType::SYMTAB, ".strtab", symbols);
 
     /// Unneeded if they were parsed from "program headers" of loaded objects.
 #if defined USE_MUSL
-    searchAndCollectSymbolsFromELFSymbolTable(info, *objects.back().elf, SHT_DYNSYM, ".dynstr", symbols, resources);
+    searchAndCollectSymbolsFromELFSymbolTable(*objects.back().elf, SectionHeaderType::DYNSYM, ".dynstr", symbols);
 #endif
 }
 
@@ -462,57 +454,70 @@ void collectSymbolsFromELF(
  * Is called by dl_iterate_phdr for every loaded shared lib until something
  * else than 0 is returned by one call of this function.
  */
-int collectSymbols(dl_phdr_info * info, size_t, void * data_ptr)
+int collectSymbols(DynamicLinkingProgramHeaderInfo * info, size_t, void * data_ptr)
 {
     SymbolIndex::Data & data = *reinterpret_cast<SymbolIndex::Data *>(data_ptr);
 
-    collectSymbolsFromProgramHeaders(info, data.symbols, data.resources);
-    collectSymbolsFromELF(info, data.symbols, data.objects, data.resources, data.build_id);
+    collectSymbolsFromProgramHeaders(info, data.symbols);
+    collectSymbolsFromELF(info, data.symbols, data.objects, data.self_build_id);
 
     /* Continue iterations */
     return 0;
 }
 
 
-template <typename T>
-const T * find(const void * address, const std::vector<T> & vec)
+const SymbolIndex::Symbol * find(const void * offset, const std::vector<SymbolIndex::Symbol> & vec)
+{
+    /// First range that has left boundary greater than address.
+
+    auto it = std::lower_bound(vec.begin(), vec.end(), offset,
+        [](const SymbolIndex::Symbol & symbol, const void * addr) { return symbol.offset_begin <= addr; });
+
+    if (it == vec.begin())
+        return nullptr;
+    --it; /// Last range that has left boundary less or equals than address.
+
+    if (offset >= it->offset_begin && offset < it->offset_end)
+        return &*it;
+    return nullptr;
+}
+
+const SymbolIndex::Object * find(const void * address, const std::vector<SymbolIndex::Object> & vec)
 {
     /// First range that has left boundary greater than address.
 
     auto it = std::lower_bound(vec.begin(), vec.end(), address,
-        [](const T & symbol, const void * addr) { return symbol.address_begin <= addr; });
+        [](const SymbolIndex::Object & object, const void * addr) { return object.address_begin <= addr; });
 
     if (it == vec.begin())
         return nullptr;
-    else
-        --it; /// Last range that has left boundary less or equals than address.
+    --it; /// Last range that has left boundary less or equals than address.
 
     if (address >= it->address_begin && address < it->address_end)
         return &*it;
-    else
-        return nullptr;
+    return nullptr;
 }
 
 }
 
 
-void SymbolIndex::update()
+void SymbolIndex::load()
 {
     dl_iterate_phdr(collectSymbols, &data);
 
     ::sort(data.objects.begin(), data.objects.end(), [](const Object & a, const Object & b) { return a.address_begin < b.address_begin; });
-    ::sort(data.symbols.begin(), data.symbols.end(), [](const Symbol & a, const Symbol & b) { return a.address_begin < b.address_begin; });
+    ::sort(data.symbols.begin(), data.symbols.end(), [](const Symbol & a, const Symbol & b) { return a.offset_begin < b.offset_begin; });
 
     /// We found symbols both from loaded program headers and from ELF symbol tables.
     data.symbols.erase(std::unique(data.symbols.begin(), data.symbols.end(), [](const Symbol & a, const Symbol & b)
     {
-        return a.address_begin == b.address_begin && a.address_end == b.address_end;
+        return a.offset_begin == b.offset_begin && a.offset_end == b.offset_end;
     }), data.symbols.end());
 }
 
-const SymbolIndex::Symbol * SymbolIndex::findSymbol(const void * address) const
+const SymbolIndex::Symbol * SymbolIndex::findSymbol(const void * offset) const
 {
-    return find(address, data.symbols);
+    return find(offset, data.symbols);
 }
 
 const SymbolIndex::Object * SymbolIndex::findObject(const void * address) const
@@ -520,14 +525,18 @@ const SymbolIndex::Object * SymbolIndex::findObject(const void * address) const
     return find(address, data.objects);
 }
 
+const SymbolIndex::Object * SymbolIndex::thisObject() const
+{
+    return findObject(reinterpret_cast<const void *>(+[]{}));
+}
+
 String SymbolIndex::getBuildIDHex() const
 {
-    String build_id_binary = getBuildID();
     String build_id_hex;
-    build_id_hex.resize(build_id_binary.size() * 2);
+    build_id_hex.resize(data.self_build_id.size() * 2);
 
     char * pos = build_id_hex.data();
-    for (auto c : build_id_binary)
+    for (auto c : data.self_build_id)
     {
         writeHexByteUppercase(c, pos);
         pos += 2;
@@ -536,24 +545,25 @@ String SymbolIndex::getBuildIDHex() const
     return build_id_hex;
 }
 
-MultiVersion<SymbolIndex> & SymbolIndex::instanceImpl()
+const SymbolIndex & SymbolIndex::instance()
 {
-    static MultiVersion<SymbolIndex> instance(std::unique_ptr<SymbolIndex>(new SymbolIndex));
+    /// To avoid recursive initialization of SymbolIndex we need to block debug
+    /// checks in MemoryTracker.
+    ///
+    /// Those debug checks capture the stacktrace for collecting big
+    /// allocation (> 16MiB), while SymbolIndex will do
+    /// ~25MiB, and so if exception will be thrown before SymbolIndex
+    /// initialized (this is the case for client/local, and no, we do not want
+    /// to initialize it explicitly, since this will increase startup time for
+    /// the client) and later during SymbolIndex initialization it will try to
+    /// initialize it one more time, and in debug build you will get pretty
+    /// nice error:
+    ///
+    ///   __cxa_guard_acquire detected recursive initialization: do you have a function-local static variable whose initialization depends on that function
+    ///
+    [[maybe_unused]] MemoryTrackerDebugBlockerInThread blocker;
+    static SymbolIndex instance;
     return instance;
 }
 
-MultiVersion<SymbolIndex>::Version SymbolIndex::instance()
-{
-    return instanceImpl().get();
 }
-
-void SymbolIndex::reload()
-{
-    instanceImpl().set(std::unique_ptr<SymbolIndex>(new SymbolIndex));
-    /// Also drop stacktrace cache.
-    StackTrace::dropCache();
-}
-
-}
-
-#endif
