@@ -1,5 +1,4 @@
 #include <iostream>
-#include <iomanip>
 
 #include <Interpreters/AggregationCommon.h>
 
@@ -28,29 +27,13 @@ int main(int, char **)
             std::cerr << x.getValue() << std::endl;
 
         DB::WriteBufferFromOwnString wb;
-        cont.writeText(wb);
+        cont.write(wb);
 
         std::cerr << "dump: " << wb.str() << std::endl;
     }
 
     {
-        using Cont = SmallMap<int, std::string, 16>;
-        Cont cont;
-
-        cont.insert(Cont::value_type(1, "Hello, world!"));
-        cont[1] = "Goodbye.";
-
-        for (auto x : cont)
-            std::cerr << x.getKey() << " -> " << x.getMapped() << std::endl;
-
-        DB::WriteBufferFromOwnString wb;
-        cont.writeText(wb);
-
-        std::cerr << "dump: " << wb.str() << std::endl;
-    }
-
-    {
-        using Cont = SmallSet<DB::UInt128, 16>;
+        using Cont = SmallSet<UInt128, 16>;
         Cont cont;
 
         DB::WriteBufferFromOwnString wb;
