@@ -1,10 +1,10 @@
 #include <Common/getHashOfLoadedBinary.h>
 
-#if defined(__linux__)
+#if defined(OS_LINUX)
 
 #include <link.h>
 #include <array>
-#include <Common/hex.h>
+#include <base/hex.h>
 
 
 static int callback(dl_phdr_info * info, size_t, void * data)
@@ -37,8 +37,7 @@ SipHash getHashOfLoadedBinary()
 std::string getHashOfLoadedBinaryHex()
 {
     SipHash hash = getHashOfLoadedBinary();
-    std::array<UInt64, 2> checksum;
-    hash.get128(checksum);
+    const auto checksum = hash.get128();
     return getHexUIntUppercase(checksum);
 }
 
