@@ -30,17 +30,23 @@ struct ExternalResultDescription
         vtDateTime,
         vtUUID,
         vtDateTime64,
+        vtTime,
+        vtTime64,
         vtDecimal32,
         vtDecimal64,
         vtDecimal128,
         vtDecimal256,
         vtArray,
-        vtFixedString
+        vtFixedString,
+        vtPoint,
     };
 
     Block sample_block;
     std::vector<std::pair<ValueType, bool /* is_nullable */>> types;
 
+    ExternalResultDescription() = default;
+    explicit ExternalResultDescription(const Block & sample_block_);
+
     void init(const Block & sample_block_);
 };
 
