@@ -16,7 +16,8 @@ int main(int, char **)
 
     String s;
     readStringUntilEOF(s, in);
-    out << StringRefHash()(s) << '\n';
+    out << StringRefHash()(StringRef{s}) << '\n';
+    out.finalize();
 
     return 0;
 }
