@@ -11,6 +11,18 @@ public:
     DataTypePointName() : DataTypeCustomFixedName("Point") {}
 };
 
+class DataTypeLineStringName : public DataTypeCustomFixedName
+{
+public:
+    DataTypeLineStringName() : DataTypeCustomFixedName("LineString") {}
+};
+
+class DataTypeMultiLineStringName : public DataTypeCustomFixedName
+{
+public:
+    DataTypeMultiLineStringName() : DataTypeCustomFixedName("MultiLineString") {}
+};
+
 class DataTypeRingName : public DataTypeCustomFixedName
 {
 public:
