@@ -1,7 +1,7 @@
 #pragma once
 
+#include <Core/Field.h>
 #include <DataTypes/DataTypeNumberBase.h>
-#include <Common/DateLUT.h>
 
 namespace DB
 {
@@ -11,12 +11,10 @@ public:
     static constexpr auto family_name = "Date32";
 
     TypeIndex getTypeId() const override { return TypeIndex::Date32; }
+    TypeIndex getColumnType() const override { return TypeIndex::Int32; }
     const char * getFamilyName() const override { return family_name; }
 
-    Field getDefault() const override
-    {
-        return -static_cast<Int64>(DateLUT::instance().getDayNumOffsetEpoch());
-    }
+    Field getDefault() const override;
 
     bool canBeUsedAsVersion() const override { return true; }
     bool canBeInsideNullable() const override { return true; }
