@@ -19,7 +19,7 @@ public:
     bool isParametric() const override { return true; }
 
     /// Some types could be still unknown.
-    DataTypeFunction(const DataTypes & argument_types_ = DataTypes(), const DataTypePtr & return_type_ = nullptr)
+    explicit DataTypeFunction(const DataTypes & argument_types_ = DataTypes(), const DataTypePtr & return_type_ = nullptr)
             : argument_types(argument_types_), return_type(return_type_) {}
 
     std::string doGetName() const override;
@@ -37,6 +37,10 @@ public:
     }
 
     bool equals(const IDataType & rhs) const override;
+
+    bool supportsSparseSerialization() const override { return false; }
+
+    void updateHashImpl(SipHash & hash) const override;
 };
 
 }
