@@ -1,5 +1,6 @@
 #pragma once
 
+#include <DataTypes/DataTypeCustom.h>
 #include <DataTypes/IDataType.h>
 
 
@@ -19,6 +20,8 @@ public:
     }
 
     String getName() const override;
+    const DataTypes & getElements() const { return elems; }
+    const Names & getNames() const { return names; }
 };
 
 DataTypePtr createNested(const DataTypes & types, const Names & names);
