@@ -1,19 +1,17 @@
-#include "CacheDictionary.h"
+#include <Dictionaries/CacheDictionary.h>
 
 #include <memory>
 #include <base/chrono_io.h>
 
-#include <Core/Defines.h>
 #include <Common/CurrentMetrics.h>
-#include <Common/HashTable/Hash.h>
 #include <Common/HashTable/HashSet.h>
 #include <Common/ProfileEvents.h>
 #include <Common/ProfilingScopedRWLock.h>
 
-#include <Dictionaries//DictionarySource.h>
+#include <Dictionaries/DictionarySource.h>
+#include <Dictionaries/DictionaryPipelineExecutor.h>
 #include <Dictionaries/HierarchyDictionariesUtils.h>
 
-#include <Processors/Executors/PullingPipelineExecutor.h>
 #include <QueryPipeline/QueryPipelineBuilder.h>
 
 namespace ProfileEvents
@@ -50,8 +48,7 @@ CacheDictionary<dictionary_key_type>::CacheDictionary(
     DictionarySourcePtr source_ptr_,
     CacheDictionaryStoragePtr cache_storage_ptr_,
     CacheDictionaryUpdateQueueConfiguration update_queue_configuration_,
-    DictionaryLifetime dict_lifetime_,
-    bool allow_read_expired_keys_)
+    CacheDictionaryConfiguration configuration_)
     : IDictionary(dict_id_)
     , dict_struct(dict_struct_)
     , source_ptr{std::move(source_ptr_)}
@@ -63,9 +60,8 @@ CacheDictionary<dictionary_key_type>::CacheDictionary(
         {
             update(unit_to_update);
         })
-    , dict_lifetime(dict_lifetime_)
-    , log(&Poco::Logger::get("ExternalDictionaries"))
-    , allow_read_expired_keys(allow_read_expired_keys_)
+    , configuration(configuration_)
+    , log(getLogger("ExternalDictionaries"))
     , rnd_engine(randomSeed())
 {
     if (!source_ptr->supportsSelectiveLoad())
@@ -120,25 +116,36 @@ DictionarySourcePtr CacheDictionary<dictionary_key_type>::getSource() const
 
 template <DictionaryKeyType dictionary_key_type>
 ColumnPtr CacheDictionary<dictionary_key_type>::getColumn(
-    const std::string & attribute_name,
-    const DataTypePtr & result_type,
-    const Columns & key_columns,
-    const DataTypes & key_types,
-    const ColumnPtr & default_values_column) const
+        const std::string & attribute_name,
+        const DataTypePtr & attribute_type,
+        const Columns & key_columns,
+        const DataTypes & key_types,
+        DefaultOrFilter default_or_filter) const
 {
-    return getColumns({attribute_name}, {result_type}, key_columns, key_types, {default_values_column}).front();
+    bool is_short_circuit = std::holds_alternative<RefFilter>(default_or_filter);
+    assert(is_short_circuit || std::holds_alternative<RefDefault>(default_or_filter));
+
+    if (is_short_circuit)
+    {
+        IColumn::Filter & default_mask = std::get<RefFilter>(default_or_filter).get();
+        return getColumns({attribute_name}, {attribute_type}, key_columns, key_types, default_mask).front();
+    }
+
+    const ColumnPtr & default_values_column = std::get<RefDefault>(default_or_filter).get();
+    const Columns & columns = Columns({default_values_column});
+    return getColumns({attribute_name}, {attribute_type}, key_columns, key_types, columns).front();
 }
 
 template <DictionaryKeyType dictionary_key_type>
 Columns CacheDictionary<dictionary_key_type>::getColumns(
     const Strings & attribute_names,
-    const DataTypes & result_types,
+    const DataTypes & attribute_types,
     const Columns & key_columns,
     const DataTypes & key_types,
-    const Columns & default_values_columns) const
+     DefaultsOrFilter defaults_or_filter) const
 {
     /**
-    * Flow of getColumsImpl
+    * Flow of getColumnsImpl
     * 1. Get fetch result from storage
     * 2. If all keys are found in storage and not expired
     *   2.1. If storage returns fetched columns in order of keys then result is returned to client.
@@ -151,6 +158,9 @@ Columns CacheDictionary<dictionary_key_type>::getColumns(
     * use default value.
     */
 
+    bool is_short_circuit = std::holds_alternative<RefFilter>(defaults_or_filter);
+    assert(is_short_circuit || std::holds_alternative<RefDefaults>(defaults_or_filter));
+
     if (dictionary_key_type == DictionaryKeyType::Complex)
         dict_struct.validateKeyTypes(key_types);
 
@@ -158,13 +168,18 @@ Columns CacheDictionary<dictionary_key_type>::getColumns(
     DictionaryKeysExtractor<dictionary_key_type> extractor(key_columns, arena_holder.getComplexKeyArena());
     auto keys = extractor.extractAllKeys();
 
-    DictionaryStorageFetchRequest request(dict_struct, attribute_names, result_types, default_values_columns);
+    DictionaryStorageFetchRequest request(dict_struct, attribute_names, attribute_types,
+        is_short_circuit ? nullptr : &std::get<RefDefaults>(defaults_or_filter).get() /*default_values_columns*/);
 
     FetchResult result_of_fetch_from_storage;
 
+    IColumn::Filter * default_mask = nullptr;
+    if (is_short_circuit)
+        default_mask= &std::get<RefFilter>(defaults_or_filter).get();
+
     {
         const ProfilingScopedReadRWLock read_lock{rw_lock, ProfileEvents::DictCacheLockWriteNs};
-        result_of_fetch_from_storage = cache_storage_ptr->fetchColumnsForKeys(keys, request);
+        result_of_fetch_from_storage = cache_storage_ptr->fetchColumnsForKeys(keys, request, default_mask);
     }
 
     size_t found_keys_size = result_of_fetch_from_storage.found_keys_size;
@@ -180,9 +195,11 @@ Columns CacheDictionary<dictionary_key_type>::getColumns(
     found_count.fetch_add(found_keys_size, std::memory_order_relaxed);
 
     MutableColumns & fetched_columns_from_storage = result_of_fetch_from_storage.fetched_columns;
-    const PaddedPODArray<KeyState> & key_index_to_state_from_storage = result_of_fetch_from_storage.key_index_to_state;
+    const PaddedPODArray<KeyState> & key_index_to_state_from_storage =
+        result_of_fetch_from_storage.key_index_to_state;
 
-    bool source_returns_fetched_columns_in_order_of_keys = cache_storage_ptr->returnsFetchedColumnsInOrderOfRequestedKeys();
+    bool source_returns_fetched_columns_in_order_of_keys =
+        cache_storage_ptr->returnsFetchedColumnsInOrderOfRequestedKeys();
 
     if (not_found_keys_size == 0 && expired_keys_size == 0)
     {
@@ -190,61 +207,52 @@ Columns CacheDictionary<dictionary_key_type>::getColumns(
 
         if (source_returns_fetched_columns_in_order_of_keys)
             return request.filterRequestedColumns(fetched_columns_from_storage);
-        else
-        {
-            /// Reorder result from storage to requested keys indexes
-            MutableColumns aggregated_columns = aggregateColumnsInOrderOfKeys(
-                keys,
-                request,
-                fetched_columns_from_storage,
-                key_index_to_state_from_storage);
 
-            return request.filterRequestedColumns(aggregated_columns);
-        }
+        /// Reorder result from storage to requested keys indexes
+        MutableColumns aggregated_columns
+            = aggregateColumnsInOrderOfKeys(keys, request, fetched_columns_from_storage, key_index_to_state_from_storage, default_mask);
+
+        return request.filterRequestedColumns(aggregated_columns);
     }
 
     size_t keys_to_update_size = not_found_keys_size + expired_keys_size;
-    auto update_unit = std::make_shared<CacheDictionaryUpdateUnit<dictionary_key_type>>(key_columns, key_index_to_state_from_storage, request, keys_to_update_size);
+    auto update_unit = std::make_shared<CacheDictionaryUpdateUnit<dictionary_key_type>>(
+        key_columns, key_index_to_state_from_storage, request, keys_to_update_size);
 
     HashMap<KeyType, size_t> requested_keys_to_fetched_columns_during_update_index;
     MutableColumns fetched_columns_during_update = request.makeAttributesResultColumns();
 
-    if (not_found_keys_size == 0 && expired_keys_size > 0 && allow_read_expired_keys)
+    if (not_found_keys_size == 0 && expired_keys_size > 0 && configuration.allow_read_expired_keys)
     {
         /// Start async update only if allow read expired keys and all keys are found
         update_queue.tryPushToUpdateQueueOrThrow(update_unit);
 
         if (source_returns_fetched_columns_in_order_of_keys)
             return request.filterRequestedColumns(fetched_columns_from_storage);
-        else
-        {
-            /// Reorder result from storage to requested keys indexes
-            MutableColumns aggregated_columns = aggregateColumnsInOrderOfKeys(
-                keys,
-                request,
-                fetched_columns_from_storage,
-                key_index_to_state_from_storage);
 
-            return request.filterRequestedColumns(aggregated_columns);
-        }
-    }
-    else
-    {
-        /// Start sync update
-        update_queue.tryPushToUpdateQueueOrThrow(update_unit);
-        update_queue.waitForCurrentUpdateFinish(update_unit);
+        /// Reorder result from storage to requested keys indexes
+        MutableColumns aggregated_columns
+            = aggregateColumnsInOrderOfKeys(keys, request, fetched_columns_from_storage, key_index_to_state_from_storage, default_mask);
 
-        requested_keys_to_fetched_columns_during_update_index = std::move(update_unit->requested_keys_to_fetched_columns_during_update_index);
-        fetched_columns_during_update = std::move(update_unit->fetched_columns_during_update);
+        return request.filterRequestedColumns(aggregated_columns);
     }
 
+    /// Start sync update
+    update_queue.tryPushToUpdateQueueOrThrow(update_unit);
+    update_queue.waitForCurrentUpdateFinish(update_unit);
+
+    requested_keys_to_fetched_columns_during_update_index = std::move(update_unit->requested_keys_to_fetched_columns_during_update_index);
+    fetched_columns_during_update = std::move(update_unit->fetched_columns_during_update);
+
+
     MutableColumns aggregated_columns = aggregateColumns(
         keys,
         request,
         fetched_columns_from_storage,
         key_index_to_state_from_storage,
         fetched_columns_during_update,
-        requested_keys_to_fetched_columns_during_update_index);
+        requested_keys_to_fetched_columns_during_update_index,
+        default_mask);
 
     return request.filterRequestedColumns(aggregated_columns);
 }
@@ -284,7 +292,7 @@ ColumnUInt8::Ptr CacheDictionary<dictionary_key_type>::hasKeys(const Columns & k
         /// Write lock on storage
         const ProfilingScopedWriteRWLock write_lock{rw_lock, ProfileEvents::DictCacheLockWriteNs};
 
-        result_of_fetch_from_storage = cache_storage_ptr->fetchColumnsForKeys(keys, request);
+        result_of_fetch_from_storage = cache_storage_ptr->fetchColumnsForKeys(keys, request, /*default_mask*/ nullptr);
     }
 
     size_t found_keys_size = result_of_fetch_from_storage.found_keys_size;
@@ -314,7 +322,7 @@ ColumnUInt8::Ptr CacheDictionary<dictionary_key_type>::hasKeys(const Columns & k
 
         allow_expired_keys_during_aggregation = true;
     }
-    else if (not_found_keys_size == 0 && expired_keys_size > 0 && allow_read_expired_keys)
+    else if (not_found_keys_size == 0 && expired_keys_size > 0 && configuration.allow_read_expired_keys)
     {
         /// Start async update only if allow read expired keys and all keys are found
         update_queue.tryPushToUpdateQueueOrThrow(update_unit);
@@ -371,8 +379,7 @@ ColumnPtr CacheDictionary<dictionary_key_type>::getHierarchy(
         found_count.fetch_add(keys_found, std::memory_order_relaxed);
         return result;
     }
-    else
-        return nullptr;
+    return nullptr;
 }
 
 template <DictionaryKeyType dictionary_key_type>
@@ -389,8 +396,7 @@ ColumnUInt8::Ptr CacheDictionary<dictionary_key_type>::isInHierarchy(
         found_count.fetch_add(keys_found, std::memory_order_relaxed);
         return result;
     }
-    else
-        return nullptr;
+    return nullptr;
 }
 
 template <DictionaryKeyType dictionary_key_type>
@@ -398,7 +404,8 @@ MutableColumns CacheDictionary<dictionary_key_type>::aggregateColumnsInOrderOfKe
     const PaddedPODArray<KeyType> & keys,
     const DictionaryStorageFetchRequest & request,
     const MutableColumns & fetched_columns,
-    const PaddedPODArray<KeyState> & key_index_to_state)
+    const PaddedPODArray<KeyState> & key_index_to_state,
+    IColumn::Filter * default_mask) const
 {
     MutableColumns aggregated_columns = request.makeAttributesResultColumns();
 
@@ -409,6 +416,9 @@ MutableColumns CacheDictionary<dictionary_key_type>::aggregateColumnsInOrderOfKe
         if (!request.shouldFillResultColumnWithIndex(fetch_request_index))
             continue;
 
+        if (default_mask)
+            default_mask->resize(keys.size());
+
         const auto & aggregated_column = aggregated_columns[fetch_request_index];
         const auto & fetched_column = fetched_columns[fetch_request_index];
 
@@ -419,7 +429,21 @@ MutableColumns CacheDictionary<dictionary_key_type>::aggregateColumnsInOrderOfKe
             if (state.isNotFound())
                 continue;
 
-            aggregated_column->insertFrom(*fetched_column, state.getFetchedColumnIndex());
+            if (default_mask)
+            {
+                if (state.isDefault())
+                {
+                    (*default_mask)[key_index] = 1;
+                    aggregated_column->insertDefault();
+                }
+                else
+                {
+                    (*default_mask)[key_index] = 0;
+                    aggregated_column->insertFrom(*fetched_column, state.getFetchedColumnIndex());
+                }
+            }
+            else
+                aggregated_column->insertFrom(*fetched_column, state.getFetchedColumnIndex());
         }
     }
 
@@ -433,7 +457,8 @@ MutableColumns CacheDictionary<dictionary_key_type>::aggregateColumns(
         const MutableColumns & fetched_columns_from_storage,
         const PaddedPODArray<KeyState> & key_index_to_fetched_columns_from_storage_result,
         const MutableColumns & fetched_columns_during_update,
-        const HashMap<KeyType, size_t> & found_keys_to_fetched_columns_during_update_index)
+        const HashMap<KeyType, size_t> & found_keys_to_fetched_columns_during_update_index,
+        IColumn::Filter * default_mask) const
 {
     /**
     * Aggregation of columns fetched from storage and from source during update.
@@ -452,7 +477,9 @@ MutableColumns CacheDictionary<dictionary_key_type>::aggregateColumns(
         const auto & aggregated_column = aggregated_columns[fetch_request_index];
         const auto & fetched_column_from_storage = fetched_columns_from_storage[fetch_request_index];
         const auto & fetched_column_during_update = fetched_columns_during_update[fetch_request_index];
-        const auto & default_value_provider = request.defaultValueProviderAtIndex(fetch_request_index);
+
+        if (default_mask)
+            default_mask->resize(keys.size());
 
         for (size_t key_index = 0; key_index < keys.size(); ++key_index)
         {
@@ -462,7 +489,25 @@ MutableColumns CacheDictionary<dictionary_key_type>::aggregateColumns(
             if (key_state_from_storage.isFound())
             {
                 /// Check and insert value if key was fetched from cache
-                aggregated_column->insertFrom(*fetched_column_from_storage, key_state_from_storage.getFetchedColumnIndex());
+
+                if (default_mask)
+                {
+                    if (key_state_from_storage.isDefault())
+                    {
+                        (*default_mask)[key_index] = 1;
+                        aggregated_column->insertDefault();
+                    }
+                    else
+                    {
+                        (*default_mask)[key_index] = 0;
+                        aggregated_column->insertFrom(*fetched_column_from_storage,
+                            key_state_from_storage.getFetchedColumnIndex());
+                    }
+                }
+                else
+                    aggregated_column->insertFrom(*fetched_column_from_storage,
+                        key_state_from_storage.getFetchedColumnIndex());
+
                 continue;
             }
 
@@ -471,11 +516,24 @@ MutableColumns CacheDictionary<dictionary_key_type>::aggregateColumns(
             if (find_iterator_in_fetch_during_update)
             {
                 aggregated_column->insertFrom(*fetched_column_during_update, find_iterator_in_fetch_during_update->getMapped());
+
+                if (default_mask)
+                    (*default_mask)[key_index] = 0;
+
                 continue;
             }
 
-            /// Insert default value
-            aggregated_column->insert(default_value_provider.getDefaultValue(key_index));
+            if (default_mask)
+            {
+                aggregated_column->insertDefault(); /// Any default is ok
+                (*default_mask)[key_index] = 1;
+            }
+            else
+            {
+                /// Insert default value
+                const auto & default_value_provider = request.defaultValueProviderAtIndex(fetch_request_index);
+                aggregated_column->insert(default_value_provider.getDefaultValue(key_index));
+            }
         }
     }
 
@@ -494,7 +552,7 @@ Pipe CacheDictionary<dictionary_key_type>::read(const Names & column_names, size
         {
             auto keys = cache_storage_ptr->getCachedSimpleKeys();
             auto keys_column = getColumnFromPODArray(std::move(keys));
-            key_columns = {ColumnWithTypeAndName(std::move(keys_column), std::make_shared<DataTypeUInt64>(), dict_struct.id->name)};
+            key_columns = {ColumnWithTypeAndName(keys_column, std::make_shared<DataTypeUInt64>(), dict_struct.id->name)};
         }
         else
         {
@@ -504,7 +562,7 @@ Pipe CacheDictionary<dictionary_key_type>::read(const Names & column_names, size
     }
 
     std::shared_ptr<const IDictionary> dictionary = shared_from_this();
-    auto coordinator = DictionarySourceCoordinator::create(dictionary, column_names, std::move(key_columns), max_block_size);
+    auto coordinator = std::make_shared<DictionarySourceCoordinator>(dictionary, column_names, std::move(key_columns), max_block_size);
     auto result = coordinator->read(num_streams);
 
     return result;
@@ -549,16 +607,17 @@ void CacheDictionary<dictionary_key_type>::update(CacheDictionaryUpdateUnitPtr<d
 
     for (size_t i = 0; i < key_index_to_state_from_storage.size(); ++i)
     {
-        if (key_index_to_state_from_storage[i].isExpired()
-            || key_index_to_state_from_storage[i].isNotFound())
+        if (key_index_to_state_from_storage[i].isExpired() || key_index_to_state_from_storage[i].isNotFound())
         {
-            if constexpr (dictionary_key_type == DictionaryKeyType::Simple)
-                requested_keys_vector.emplace_back(requested_keys[i]);
-            else
-                requested_complex_key_rows.emplace_back(i);
-
             auto requested_key = requested_keys[i];
-            not_found_keys.insert(requested_key);
+            auto [_, inserted] = not_found_keys.insert(requested_key);
+            if (inserted)
+            {
+                if constexpr (dictionary_key_type == DictionaryKeyType::Simple)
+                    requested_keys_vector.emplace_back(requested_keys[i]);
+                else
+                    requested_complex_key_rows.emplace_back(i);
+            }
         }
     }
 
@@ -588,14 +647,15 @@ void CacheDictionary<dictionary_key_type>::update(CacheDictionaryUpdateUnitPtr<d
 
             Columns fetched_columns_during_update = fetch_request.makeAttributesResultColumnsNonMutable();
 
-            PullingPipelineExecutor executor(pipeline);
+            DictionaryPipelineExecutor executor(pipeline, configuration.use_async_executor);
+            pipeline.setConcurrencyControl(false);
             Block block;
             while (executor.pull(block))
             {
                 Columns key_columns;
                 key_columns.reserve(skip_keys_size_offset);
 
-                convertToFullIfSparse(block);
+                removeSpecialColumnRepresentations(block);
                 auto block_columns = block.getColumns();
 
                 /// Split into keys columns and attribute columns
@@ -685,9 +745,11 @@ void CacheDictionary<dictionary_key_type>::update(CacheDictionaryUpdateUnitPtr<d
     {
         /// Won't request source for keys
         throw DB::Exception(ErrorCodes::CACHE_DICTIONARY_UPDATE_FAIL,
-            "Query contains keys that are not present in cache or expired. Could not update cache dictionary {} now, because nearest update is scheduled at {}. Try again later.",
-            getDictionaryID().getNameForLogs(),
-            to_string(backoff_end_time.load()));
+                            "Query contains keys that are not present in cache or expired. "
+                            "Could not update cache dictionary {} now, because nearest update is scheduled at {}. "
+                            "Try again later.",
+                            getDictionaryID().getNameForLogs(),
+                            to_string(backoff_end_time.load()));
     }
 }
 
