@@ -5,34 +5,77 @@
 #include <Common/HashTable/HashMap.h>
 #include <Common/HashTable/HashSet.h>
 
-#include <Columns/IColumn.h>
-#include <Columns/ColumnVector.h>
-#include <Columns/ColumnArray.h>
+#include <Columns/IColumn_fwd.h>
 
 #include <Dictionaries/IDictionary.h>
 
 namespace DB
 {
 
+class DictionaryHierarchicalParentToChildIndex;
+using DictionaryHierarchyParentToChildIndexPtr = std::shared_ptr<DictionaryHierarchicalParentToChildIndex>;
+
+class DictionaryHierarchicalParentToChildIndex
+{
+public:
+    struct KeysRange
+    {
+        UInt32 start_index;
+        UInt32 end_index;
+    };
+
+    /// By default we use initial_bytes=4096 in PodArray.
+    /// It might lead to really high memory consumption when arrays are almost empty but there are a lot of them.
+    using Array = PODArray<UInt64, 8 * sizeof(UInt64), Allocator<false>, PADDING_FOR_SIMD - 1, PADDING_FOR_SIMD>;
+    using ParentToChildIndex = HashMap<UInt64, Array>;
+
+    explicit DictionaryHierarchicalParentToChildIndex(const ParentToChildIndex & parent_to_children_map_)
+    {
+        size_t parent_to_children_map_size = parent_to_children_map_.size();
+
+        keys.reserve(parent_to_children_map_size);
+        parent_to_children_keys_range.reserve(parent_to_children_map_size);
+
+        for (const auto & [parent, children] : parent_to_children_map_)
+        {
+            size_t keys_size = keys.size();
+            UInt32 start_index = static_cast<UInt32>(keys_size);
+            UInt32 end_index = start_index + static_cast<UInt32>(children.size());
+
+            keys.insert(children.begin(), children.end());
+
+            parent_to_children_keys_range[parent] = KeysRange{start_index, end_index};
+        }
+    }
+
+    size_t getSizeInBytes() const
+    {
+        return parent_to_children_keys_range.getBufferSizeInBytes() + (keys.size() * sizeof(UInt64));
+    }
+
+    /// Map parent key to range of children from keys array
+    HashMap<UInt64, KeysRange> parent_to_children_keys_range;
+
+    /// Array of keys in hierarchy
+    PaddedPODArray<UInt64> keys;
+};
+
 namespace detail
 {
-    template <typename KeyType>
     struct ElementsAndOffsets
     {
-        PaddedPODArray<KeyType> elements;
+        PaddedPODArray<UInt64> elements;
         PaddedPODArray<IColumn::Offset> offsets;
     };
 
-    template <typename T>
     struct IsKeyValidFuncInterface
     {
-        bool operator()(T key [[maybe_unused]]) { return false; }
+        bool operator()(UInt64 key [[maybe_unused]]) { return false; }
     };
 
-    template <typename T>
     struct GetParentKeyFuncInterface
     {
-        std::optional<T> operator()(T key [[maybe_unused]]) { return {}; }
+        std::optional<UInt64> operator()(UInt64 key [[maybe_unused]]) { return {}; }
     };
 
     /** Calculate hierarchy for keys iterating the hierarchy from child to parent using get_parent_key_func provided by client.
@@ -54,16 +97,15 @@ namespace detail
       * Elements: [1, 2, 1, 3, 1, 4, 2, 1]
       * Offsets: [1, 3, 5, 8, 8]
       */
-    template <typename KeyType, typename IsKeyValidFunc, typename GetParentKeyFunc>
-    ElementsAndOffsets<KeyType> getHierarchy(
-        const PaddedPODArray<KeyType> & keys,
-        const KeyType & hierarchy_null_value,
+    template <typename IsKeyValidFunc, typename GetParentKeyFunc>
+    ElementsAndOffsets getHierarchy(
+        const PaddedPODArray<UInt64> & keys,
         IsKeyValidFunc && is_key_valid_func,
         GetParentKeyFunc && get_parent_key_func)
     {
         size_t hierarchy_keys_size = keys.size();
 
-        PaddedPODArray<KeyType> elements;
+        PaddedPODArray<UInt64> elements;
         elements.reserve(hierarchy_keys_size);
 
         PaddedPODArray<IColumn::Offset> offsets;
@@ -75,7 +117,7 @@ namespace detail
             size_t array_element_offset;
         };
 
-        HashMap<KeyType, OffsetInArray> already_processes_keys_to_offset;
+        HashMap<UInt64, OffsetInArray> already_processes_keys_to_offset;
         already_processes_keys_to_offset.reserve(hierarchy_keys_size);
 
         for (size_t i = 0; i < hierarchy_keys_size; ++i)
@@ -83,7 +125,7 @@ namespace detail
             auto hierarchy_key = keys[i];
             size_t current_hierarchy_depth = 0;
 
-            bool is_key_valid = std::forward<IsKeyValidFunc>(is_key_valid_func)(hierarchy_key);
+            bool is_key_valid = is_key_valid_func(hierarchy_key);
 
             if (!is_key_valid)
             {
@@ -108,7 +150,7 @@ namespace detail
 
                     size_t array_element_offset = index.array_element_offset;
 
-                    size_t previous_offset_size = offset > 0 ? offsets[offset - 1] : 0;
+                    size_t previous_offset_size = offsets[offset - 1];
                     size_t start_index = previous_offset_size + array_element_offset;
                     size_t end_index = offsets[offset];
 
@@ -116,14 +158,14 @@ namespace detail
                     break;
                 }
 
-                if (hierarchy_key == hierarchy_null_value || current_hierarchy_depth >= DBMS_HIERARCHICAL_DICTIONARY_MAX_DEPTH)
+                if (current_hierarchy_depth >= DBMS_HIERARCHICAL_DICTIONARY_MAX_DEPTH)
                     break;
 
                 already_processes_keys_to_offset[hierarchy_key] = {offsets.size(), current_hierarchy_depth};
                 elements.emplace_back(hierarchy_key);
                 ++current_hierarchy_depth;
 
-                std::optional<KeyType> parent_key = std::forward<GetParentKeyFunc>(get_parent_key_func)(hierarchy_key);
+                std::optional<UInt64> parent_key = get_parent_key_func(hierarchy_key);
 
                 if (!parent_key.has_value())
                     break;
@@ -134,23 +176,22 @@ namespace detail
             offsets.emplace_back(elements.size());
         }
 
-        ElementsAndOffsets<KeyType> result = {std::move(elements), std::move(offsets)};
+        ElementsAndOffsets result = {std::move(elements), std::move(offsets)};
 
         return result;
     }
 
     /** Returns array with UInt8 represent if key from in_keys array is in hierarchy of key from keys column.
       * If value in result array is 1 that means key from in_keys array is in hierarchy of key from
-      * keys array with same index, 0 therwise.
+      * keys array with same index, 0 otherwise.
       * For getting hierarchy implementation uses getKeysHierarchy function.
       *
       * Not: keys size must be equal to in_keys_size.
       */
-    template <typename KeyType, typename IsKeyValidFunc, typename GetParentKeyFunc>
+    template <typename IsKeyValidFunc, typename GetParentKeyFunc>
     PaddedPODArray<UInt8> getIsInHierarchy(
-        const PaddedPODArray<KeyType> & keys,
-        const PaddedPODArray<KeyType> & in_keys,
-        const KeyType & hierarchy_null_value,
+        const PaddedPODArray<UInt64> & keys,
+        const PaddedPODArray<UInt64> & in_keys,
         IsKeyValidFunc && is_key_valid_func,
         GetParentKeyFunc && get_parent_func)
     {
@@ -159,9 +200,8 @@ namespace detail
         PaddedPODArray<UInt8> result;
         result.resize_fill(keys.size());
 
-        detail::ElementsAndOffsets<KeyType> hierarchy = detail::getHierarchy(
+        detail::ElementsAndOffsets hierarchy = detail::getHierarchy(
             keys,
-            hierarchy_null_value,
             std::forward<IsKeyValidFunc>(is_key_valid_func),
             std::forward<GetParentKeyFunc>(get_parent_func));
 
@@ -173,7 +213,7 @@ namespace detail
             size_t i_elements_start = i > 0 ? offsets[i - 1] : 0;
             size_t i_elements_end = offsets[i];
 
-            auto & key_to_find = in_keys[i];
+            const auto & key_to_find = in_keys[i];
 
             const auto * begin = elements.begin() + i_elements_start;
             const auto * end = elements.begin() + i_elements_end;
@@ -216,19 +256,22 @@ namespace detail
       * Result: [1], [2, 3], [4], [], [];
       * Offsets: [1, 3, 4, 4, 4];
       */
-    template <typename KeyType, typename Strategy>
-    ElementsAndOffsets<KeyType> getDescendants(
-        const PaddedPODArray<KeyType> & keys,
-        const HashMap<KeyType, PaddedPODArray<KeyType>> & parent_to_child,
+    template <typename Strategy>
+    ElementsAndOffsets getDescendants(
+        const PaddedPODArray<UInt64> & keys,
+        const DictionaryHierarchicalParentToChildIndex & parent_to_child_index,
         Strategy strategy,
         size_t & valid_keys)
     {
+        const auto & parent_to_children_keys_range = parent_to_child_index.parent_to_children_keys_range;
+        const auto & children_keys = parent_to_child_index.keys;
+
         /// If strategy is GetAllDescendantsStrategy we try to cache and later reuse previously calculated descendants.
         /// If strategy is GetDescendantsAtSpecificLevelStrategy we does not use cache strategy.
         size_t keys_size = keys.size();
         valid_keys = 0;
 
-        PaddedPODArray<KeyType> descendants;
+        PaddedPODArray<UInt64> descendants;
         descendants.reserve(keys_size);
 
         PaddedPODArray<IColumn::Offset> descendants_offsets;
@@ -241,18 +284,18 @@ namespace detail
         };
 
         static constexpr Int64 key_range_requires_update = -1;
-        HashMap<KeyType, Range> already_processed_keys_to_range [[maybe_unused]];
+        HashMap<UInt64, Range> already_processed_keys_to_range [[maybe_unused]];
 
         if constexpr (std::is_same_v<Strategy, GetAllDescendantsStrategy>)
             already_processed_keys_to_range.reserve(keys_size);
 
         struct KeyAndDepth
         {
-            KeyType key;
+            UInt64 key;
             Int64 depth;
         };
 
-        HashSet<KeyType> already_processed_keys_during_loop;
+        HashSet<UInt64> already_processed_keys_during_loop;
         already_processed_keys_during_loop.reserve(keys_size);
 
         PaddedPODArray<KeyAndDepth> next_keys_to_process_stack;
@@ -262,9 +305,9 @@ namespace detail
 
         for (size_t i = 0; i < keys_size; ++i)
         {
-            const KeyType & requested_key = keys[i];
+            const UInt64 & requested_key = keys[i];
 
-            if (parent_to_child.find(requested_key) == nullptr)
+            if (parent_to_children_keys_range.find(requested_key) == nullptr)
             {
                 descendants_offsets.emplace_back(descendants.size());
                 continue;
@@ -282,7 +325,7 @@ namespace detail
             {
                 KeyAndDepth key_to_process = next_keys_to_process_stack.back();
 
-                KeyType key = key_to_process.key;
+                UInt64 key = key_to_process.key;
                 Int64 depth = key_to_process.depth;
                 next_keys_to_process_stack.pop_back();
 
@@ -329,7 +372,7 @@ namespace detail
                     }
                 }
 
-                const auto * it = parent_to_child.find(key);
+                const auto * it = parent_to_children_keys_range.find(key);
 
                 if (!it || depth >= DBMS_HIERARCHICAL_DICTIONARY_MAX_DEPTH)
                     continue;
@@ -352,15 +395,26 @@ namespace detail
 
                 ++depth;
 
-                const auto & children = it->getMapped();
+                DictionaryHierarchicalParentToChildIndex::KeysRange children_range = it->getMapped();
 
-                for (auto child_key : children)
+                for (; children_range.start_index < children_range.end_index; ++children_range.start_index)
                 {
+                    auto child_key = children_keys[children_range.start_index];
+
                     /// In case of GetAllDescendantsStrategy we add any descendant to result array
                     /// If strategy is GetDescendantsAtSpecificLevelStrategy we require depth == level
-                    if (std::is_same_v<Strategy, GetAllDescendantsStrategy> || depth == level)
+                    if constexpr (std::is_same_v<Strategy, GetAllDescendantsStrategy>)
                         descendants.emplace_back(child_key);
 
+                    if constexpr (std::is_same_v<Strategy, GetDescendantsAtSpecificLevelStrategy>)
+                    {
+                        if (depth == level)
+                        {
+                            descendants.emplace_back(child_key);
+                            continue;
+                        }
+                    }
+
                     next_keys_to_process_stack.emplace_back(KeyAndDepth{child_key, depth});
                 }
             }
@@ -370,37 +424,23 @@ namespace detail
             descendants_offsets.emplace_back(descendants.size());
         }
 
-        ElementsAndOffsets<KeyType> result = {std::move(descendants), std::move(descendants_offsets)};
+        ElementsAndOffsets result = {std::move(descendants), std::move(descendants_offsets)};
         return result;
     }
 
     /// Converts ElementAndOffsets structure into ArrayColumn
-    template<typename KeyType>
-    ColumnPtr convertElementsAndOffsetsIntoArray(ElementsAndOffsets<KeyType> && elements_and_offsets)
-    {
-        auto elements_column = ColumnVector<KeyType>::create();
-        elements_column->getData() = std::move(elements_and_offsets.elements);
-
-        auto offsets_column = ColumnVector<IColumn::Offset>::create();
-        offsets_column->getData() = std::move(elements_and_offsets.offsets);
-
-        auto column_array = ColumnArray::create(std::move(elements_column), std::move(offsets_column));
-
-        return column_array;
-    }
+    ColumnPtr convertElementsAndOffsetsIntoArray(ElementsAndOffsets && elements_and_offsets);
 }
 
 /// Returns hierarchy array column for keys
 template <typename KeyType, typename IsKeyValidFunc, typename GetParentKeyFunc>
 ColumnPtr getKeysHierarchyArray(
     const PaddedPODArray<KeyType> & keys,
-    const KeyType & hierarchy_null_value,
     IsKeyValidFunc && is_key_valid_func,
     GetParentKeyFunc && get_parent_func)
 {
     auto elements_and_offsets = detail::getHierarchy(
         keys,
-        hierarchy_null_value,
         std::forward<IsKeyValidFunc>(is_key_valid_func),
         std::forward<GetParentKeyFunc>(get_parent_func));
 
@@ -412,14 +452,12 @@ template <typename KeyType, typename IsKeyValidFunc, typename GetParentKeyFunc>
 ColumnUInt8::Ptr getKeysIsInHierarchyColumn(
     const PaddedPODArray<KeyType> & hierarchy_keys,
     const PaddedPODArray<KeyType> & hierarchy_in_keys,
-    const KeyType & hierarchy_null_value,
     IsKeyValidFunc && is_key_valid_func,
     GetParentKeyFunc && get_parent_func)
 {
     auto is_in_hierarchy_data = detail::getIsInHierarchy(
         hierarchy_keys,
         hierarchy_in_keys,
-        hierarchy_null_value,
         std::forward<IsKeyValidFunc>(is_key_valid_func),
         std::forward<GetParentKeyFunc>(get_parent_func));
 
@@ -432,26 +470,11 @@ ColumnUInt8::Ptr getKeysIsInHierarchyColumn(
 /// Returns descendants array column for keys
 ///
 /// @param valid_keys - number of keys that are valid in parent_to_child map
-template <typename KeyType>
 ColumnPtr getKeysDescendantsArray(
-    const PaddedPODArray<KeyType> & requested_keys,
-    const HashMap<KeyType, PaddedPODArray<KeyType>> & parent_to_child,
+    const PaddedPODArray<UInt64> & requested_keys,
+    const DictionaryHierarchicalParentToChildIndex & parent_to_child_index,
     size_t level,
-    size_t & valid_keys)
-{
-    if (level == 0)
-    {
-        detail::GetAllDescendantsStrategy strategy { .level = level };
-        auto elements_and_offsets = detail::getDescendants(requested_keys, parent_to_child, strategy, valid_keys);
-        return detail::convertElementsAndOffsetsIntoArray(std::move(elements_and_offsets));
-    }
-    else
-    {
-        detail::GetDescendantsAtSpecificLevelStrategy strategy { .level = level };
-        auto elements_and_offsets = detail::getDescendants(requested_keys, parent_to_child, strategy, valid_keys);
-        return detail::convertElementsAndOffsetsIntoArray(std::move(elements_and_offsets));
-    }
-}
+    size_t & valid_keys);
 
 /** Default getHierarchy implementation for dictionaries that does not have structure with child to parent representation.
   * Implementation will build such structure with getColumn calls, and then getHierarchy for such structure.
