@@ -3,34 +3,22 @@
 #include <Core/Block.h>
 
 #include <Core/ExternalResultDescription.h>
-#include <Processors/Sources/SourceWithProgress.h>
+#include <Processors/ISource.h>
 #include <Poco/Redis/Array.h>
 #include <Poco/Redis/Type.h>
-#include "RedisDictionarySource.h"
-
-namespace Poco
-{
-    namespace Redis
-    {
-        class Client;
-    }
-}
+#include <Storages/RedisCommon.h>
 
 
 namespace DB
 {
-    class RedisSource final : public SourceWithProgress
+    class RedisSource final : public ISource
     {
     public:
-        using RedisArray = Poco::Redis::Array;
-        using RedisBulkString = Poco::Redis::BulkString;
-        using ConnectionPtr = RedisDictionarySource::ConnectionPtr;
-
         RedisSource(
-            ConnectionPtr connection_,
-            const Poco::Redis::Array & keys_,
+            RedisConnectionPtr connection_,
+            const RedisArray & keys_,
             const RedisStorageType & storage_type_,
-            const Block & sample_block,
+            SharedHeader sample_block,
             size_t max_block_size);
 
         ~RedisSource() override;
@@ -40,7 +28,7 @@ namespace DB
     private:
         Chunk generate() override;
 
-        ConnectionPtr connection;
+        RedisConnectionPtr connection;
         Poco::Redis::Array keys;
         RedisStorageType storage_type;
         const size_t max_block_size;
