@@ -27,7 +27,9 @@ public:
         const Poco::Util::AbstractConfiguration & config,
         const String & config_prefix,
         ContextPtr context,
-        const DisksMap & map)>;
+        const DisksMap & map,
+        bool attach,
+        bool custom_disk)>;
 
     static DiskFactory & instance();
 
@@ -38,7 +40,12 @@ public:
         const Poco::Util::AbstractConfiguration & config,
         const String & config_prefix,
         ContextPtr context,
-        const DisksMap & map) const;
+        const DisksMap & map,
+        bool attach = false,
+        bool custom_disk = false,
+        const std::unordered_set<String> & skip_types = {}) const;
+
+    void clearRegistry();
 
 private:
     using DiskTypeRegistry = std::unordered_map<String, Creator>;
