@@ -7,16 +7,17 @@
 #include <DataTypes/DataTypesNumber.h>
 #include <DataTypes/DataTypesDecimal.h>
 #include <DataTypes/DataTypeDateTime64.h>
+#include <Core/callOnTypeIndex.h>
 #include <Columns/ColumnVector.h>
+#include <Common/intExp10.h>
 #include <Interpreters/castColumn.h>
-#include "IFunction.h"
+#include <Functions/IFunction.h>
 #include <Common/intExp.h>
 #include <Common/assert_cast.h>
 #include <Core/Defines.h>
 #include <cmath>
 #include <type_traits>
 #include <array>
-#include <base/bit_cast.h>
 #include <base/sort.h>
 #include <algorithm>
 
@@ -32,7 +33,6 @@ namespace DB
 
 namespace ErrorCodes
 {
-    extern const int NUMBER_OF_ARGUMENTS_DOESNT_MATCH;
     extern const int ILLEGAL_TYPE_OF_ARGUMENT;
     extern const int ARGUMENT_OUT_OF_BOUND;
     extern const int ILLEGAL_COLUMN;
@@ -41,34 +41,30 @@ namespace ErrorCodes
 }
 
 
-/** Rounding Functions:
-    * round(x, N) - rounding to nearest (N = 0 by default). Use banker's rounding for floating point numbers.
-    * roundBankers(x, N) - rounding to nearest (N = 0 by default). Use banker's rounding for all numbers.
-    * floor(x, N) is the largest number <= x (N = 0 by default).
-    * ceil(x, N) is the smallest number >= x (N = 0 by default).
-    * trunc(x, N) - is the largest by absolute value number that is not greater than x by absolute value (N = 0 by default).
-    *
-    * The value of the parameter N (scale):
-    * - N > 0: round to the number with N decimal places after the decimal point
-    * - N < 0: round to an integer with N zero characters
-    * - N = 0: round to an integer
-    *
-    * Type of the result is the type of argument.
-    * For integer arguments, when passing negative scale, overflow can occur.
-    * In that case, the behavior is implementation specific.
-    */
+/// Rounding Functions:
+/// - round(x, N) - rounding to nearest (N = 0 by default). Use banker's rounding for floating point numbers.
+/// - roundBankers(x, N) - rounding to nearest (N = 0 by default). Use banker's rounding for all numbers.
+/// - floor(x, N) is the largest number <= x (N = 0 by default).
+/// - ceil(x, N) is the smallest number >= x (N = 0 by default).
+/// - trunc(x, N) - is the largest by absolute value number that is not greater than x by absolute value (N = 0 by default).
 
+/// The value of the parameter N (scale):
+/// - N > 0: round to the number with N decimal places after the decimal point
+/// - N < 0: round to an integer with N zero characters
+/// - N = 0: round to an integer
 
-/** This parameter controls the behavior of the rounding functions.
-  */
-enum class ScaleMode
+/// Type of the result is the type of argument.
+/// For integer arguments, when passing negative scale, overflow can occur. In that case, the behavior is undefined.
+
+/// Controls the behavior of the rounding functions.
+enum class ScaleMode : uint8_t
 {
     Positive,   // round to a number with N decimal places after the decimal point
     Negative,   // round to an integer with N zero characters
     Zero,       // round to an integer
 };
 
-enum class RoundingMode
+enum class RoundingMode : uint8_t
 {
 #ifdef __SSE4_1__
     Round   = _MM_FROUND_TO_NEAREST_INT | _MM_FROUND_NO_EXC,
@@ -76,25 +72,30 @@ enum class RoundingMode
     Ceil    = _MM_FROUND_TO_POS_INF | _MM_FROUND_NO_EXC,
     Trunc   = _MM_FROUND_TO_ZERO | _MM_FROUND_NO_EXC,
 #else
-    Round   = 8,    /// Values are correspond to above just in case.
+    Round   = 8,    /// Values correspond to above values, just in case.
     Floor   = 9,
     Ceil    = 10,
     Trunc   = 11,
 #endif
 };
 
-enum class TieBreakingMode
+enum class TieBreakingMode : uint8_t
 {
-    Auto, // use banker's rounding for floating point numbers, round up otherwise
-    Bankers, // use banker's rounding
+    Auto,    /// banker's rounding for floating point numbers, round up otherwise
+    Bankers, /// banker's rounding
+};
+
+enum class Vectorize : uint8_t
+{
+    No,
+    Yes
 };
 
 /// For N, no more than the number of digits in the largest type.
 using Scale = Int16;
 
 
-/** Rounding functions for integer values.
-  */
+/// Rounding functions for integer values.
 template <typename T, RoundingMode rounding_mode, ScaleMode scale_mode, TieBreakingMode tie_breaking_mode>
 struct IntegerRoundingComputation
 {
@@ -151,7 +152,7 @@ struct IntegerRoundingComputation
             }
         }
 
-        __builtin_unreachable();
+        std::unreachable();
     }
 
     static ALWAYS_INLINE T compute(T x, T scale)
@@ -165,10 +166,11 @@ struct IntegerRoundingComputation
                 return computeImpl(x, scale);
         }
 
-        __builtin_unreachable();
+        std::unreachable();
     }
 
     static ALWAYS_INLINE void compute(const T * __restrict in, size_t scale, T * __restrict out)
+    requires std::integral<T>
     {
         if constexpr (sizeof(T) <= sizeof(scale) && scale_mode == ScaleMode::Negative)
         {
@@ -178,19 +180,26 @@ struct IntegerRoundingComputation
                 return;
             }
         }
-        *out = compute(*in, scale);
+        *out = compute(*in, static_cast<T>(scale));
     }
 
+    static ALWAYS_INLINE void compute(const T * __restrict in, T scale, T * __restrict out)
+    requires(!std::integral<T>)
+    {
+        *out = compute(*in, scale);
+    }
 };
 
 
+template <typename T, Vectorize vectorize>
+class FloatRoundingComputationBase;
+
 #ifdef __SSE4_1__
 
-template <typename T>
-class BaseFloatRoundingComputation;
+/// Vectorized implementation for x86.
 
 template <>
-class BaseFloatRoundingComputation<Float32>
+class FloatRoundingComputationBase<Float32, Vectorize::Yes>
 {
 public:
     using ScalarType = Float32;
@@ -211,7 +220,7 @@ public:
 };
 
 template <>
-class BaseFloatRoundingComputation<Float64>
+class FloatRoundingComputationBase<Float64, Vectorize::Yes>
 {
 public:
     using ScalarType = Float64;
@@ -231,9 +240,9 @@ public:
     }
 };
 
-#else
+#endif
 
-/// Implementation for ARM. Not vectorized.
+/// Sequential implementation for ARM. Also used for scalar arguments.
 
 inline float roundWithMode(float x, RoundingMode mode)
 {
@@ -245,7 +254,7 @@ inline float roundWithMode(float x, RoundingMode mode)
         case RoundingMode::Trunc: return truncf(x);
     }
 
-    __builtin_unreachable();
+    std::unreachable();
 }
 
 inline double roundWithMode(double x, RoundingMode mode)
@@ -258,11 +267,24 @@ inline double roundWithMode(double x, RoundingMode mode)
         case RoundingMode::Trunc: return trunc(x);
     }
 
-    __builtin_unreachable();
+    std::unreachable();
+}
+
+inline BFloat16 roundWithMode(BFloat16 x, RoundingMode mode)
+{
+    switch (mode)
+    {
+        case RoundingMode::Round: return BFloat16(nearbyintf(Float32(x)));
+        case RoundingMode::Floor: return BFloat16(floorf(Float32(x)));
+        case RoundingMode::Ceil: return BFloat16(ceilf(Float32(x)));
+        case RoundingMode::Trunc: return BFloat16(truncf(Float32(x)));
+    }
+
+    std::unreachable();
 }
 
 template <typename T>
-class BaseFloatRoundingComputation
+class FloatRoundingComputationBase<T, Vectorize::No>
 {
 public:
     using ScalarType = T;
@@ -278,22 +300,25 @@ public:
 
     static VectorType prepare(size_t scale)
     {
-        return load1(scale);
+        return load1(ScalarType(scale));
     }
 };
 
-#endif
+template <>
+class FloatRoundingComputationBase<BFloat16, Vectorize::Yes> : public FloatRoundingComputationBase<BFloat16, Vectorize::No>
+{
+};
 
 
 /** Implementation of low-level round-off functions for floating-point values.
   */
-template <typename T, RoundingMode rounding_mode, ScaleMode scale_mode>
-class FloatRoundingComputation : public BaseFloatRoundingComputation<T>
+template <typename T, RoundingMode rounding_mode, ScaleMode scale_mode, Vectorize vectorize>
+class FloatRoundingComputation : public FloatRoundingComputationBase<T, vectorize>
 {
-    using Base = BaseFloatRoundingComputation<T>;
+    using Base = FloatRoundingComputationBase<T, vectorize>;
 
 public:
-    static inline void compute(const T * __restrict in, const typename Base::VectorType & scale, T * __restrict out)
+    static void compute(const T * __restrict in, const typename Base::VectorType & scale, T * __restrict out)
     {
         auto val = Base::load(in);
 
@@ -322,15 +347,22 @@ struct FloatRoundingImpl
 private:
     static_assert(!is_decimal<T>);
 
-    using Op = FloatRoundingComputation<T, rounding_mode, scale_mode>;
-    using Data = std::array<T, Op::data_count>;
+    template <Vectorize vectorize =
+#ifdef __SSE4_1__
+        Vectorize::Yes
+#else
+        Vectorize::No
+#endif
+    >
+    using Op = FloatRoundingComputation<T, rounding_mode, scale_mode, vectorize>;
+    using Data = std::array<T, Op<>::data_count>;
     using ColumnType = ColumnVector<T>;
     using Container = typename ColumnType::Container;
 
 public:
     static NO_INLINE void apply(const Container & in, size_t scale, Container & out)
     {
-        auto mm_scale = Op::prepare(scale);
+        auto mm_scale = Op<>::prepare(scale);
 
         const size_t data_count = std::tuple_size<Data>();
 
@@ -342,7 +374,7 @@ public:
 
         while (p_in < limit)
         {
-            Op::compute(p_in, mm_scale, p_out);
+            Op<>::compute(p_in, mm_scale, p_out);
             p_in += data_count;
             p_out += data_count;
         }
@@ -355,10 +387,17 @@ public:
             size_t tail_size_bytes = (end_in - p_in) * sizeof(*p_in);
 
             memcpy(&tmp_src, p_in, tail_size_bytes);
-            Op::compute(reinterpret_cast<T *>(&tmp_src), mm_scale, reinterpret_cast<T *>(&tmp_dst));
+            Op<>::compute(reinterpret_cast<T *>(&tmp_src), mm_scale, reinterpret_cast<T *>(&tmp_dst));
             memcpy(p_out, &tmp_dst, tail_size_bytes);
         }
     }
+
+    static void applyOne(T in, size_t scale, T& out)
+    {
+        using ScalarOp = Op<Vectorize::No>;
+        auto s = ScalarOp::prepare(scale);
+        ScalarOp::compute(&in, s, &out);
+    }
 };
 
 template <typename T, RoundingMode rounding_mode, ScaleMode scale_mode, TieBreakingMode tie_breaking_mode>
@@ -411,10 +450,14 @@ public:
             case 1000000000000000000ULL: return applyImpl<1000000000000000000ULL>(in, out);
             case 10000000000000000000ULL: return applyImpl<10000000000000000000ULL>(in, out);
             default:
-                throw Exception("Unexpected 'scale' parameter passed to function",
-                    ErrorCodes::BAD_ARGUMENTS);
+                throw Exception(ErrorCodes::BAD_ARGUMENTS, "Unexpected 'scale' parameter passed to function");
         }
     }
+
+    static void applyOne(T in, size_t scale, T& out)
+    {
+        Op::compute(&in, scale, &out);
+    }
 };
 
 
@@ -432,7 +475,7 @@ public:
         scale_arg = in_scale - scale_arg;
         if (scale_arg > 0)
         {
-            size_t scale = intExp10(scale_arg);
+            auto scale = intExp10OfSize<NativeType>(scale_arg);
 
             const NativeType * __restrict p_in = reinterpret_cast<const NativeType *>(in.data());
             const NativeType * end_in = reinterpret_cast<const NativeType *>(in.data()) + in.size();
@@ -450,72 +493,185 @@ public:
             memcpy(out.data(), in.data(), in.size() * sizeof(T));
         }
     }
+
+    static void applyOne(NativeType in, UInt32 in_scale, NativeType& out, Scale scale_arg)
+    {
+        scale_arg = in_scale - scale_arg;
+        if (scale_arg > 0)
+        {
+            auto scale = intExp10OfSize<NativeType>(scale_arg);
+            Op::compute(&in, scale, &out);
+        }
+        else
+        {
+            memcpy(&out, &in, sizeof(T));
+        }
+    }
 };
 
+/// Select the appropriate processing algorithm depending on the scale.
+inline void validateScale(Int64 scale64)
+{
+    if (scale64 > std::numeric_limits<Scale>::max() || scale64 < std::numeric_limits<Scale>::min())
+        throw Exception(ErrorCodes::ARGUMENT_OUT_OF_BOUND, "Scale argument for rounding function is too large");
+}
 
-/** Select the appropriate processing algorithm depending on the scale.
-  */
+inline Scale getScaleArg(const ColumnConst* scale_col)
+{
+    const auto & scale_field = scale_col->getField();
+
+    Int64 scale64 = scale_field.safeGet<Int64>();
+    validateScale(scale64);
+
+    return scale64;
+}
+
+/// Generic dispatcher
 template <typename T, RoundingMode rounding_mode, TieBreakingMode tie_breaking_mode>
 struct Dispatcher
 {
     template <ScaleMode scale_mode>
-    using FunctionRoundingImpl = std::conditional_t<std::is_floating_point_v<T>,
+    using FunctionRoundingImpl = std::conditional_t<is_floating_point<T>,
         FloatRoundingImpl<T, rounding_mode, scale_mode>,
         IntegerRoundingImpl<T, rounding_mode, scale_mode, tie_breaking_mode>>;
 
-    static ColumnPtr apply(const IColumn * col_general, Scale scale_arg)
+    template <typename ScaleType>
+    static ColumnPtr apply(const IColumn * value_col, const IColumn * scale_col = nullptr)
     {
-        const auto * const col = checkAndGetColumn<ColumnVector<T>>(col_general);
-        auto col_res = ColumnVector<T>::create();
-
-        typename ColumnVector<T>::Container & vec_res = col_res->getData();
-        vec_res.resize(col->getData().size());
-
-        if (!vec_res.empty())
+        // Non-const value argument:
+        const auto * value_col_typed = checkAndGetColumn<ColumnVector<T>>(value_col);
+        if (value_col_typed)
         {
-            if (scale_arg == 0)
-            {
-                size_t scale = 1;
-                FunctionRoundingImpl<ScaleMode::Zero>::apply(col->getData(), scale, vec_res);
-            }
-            else if (scale_arg > 0)
-            {
-                size_t scale = intExp10(scale_arg);
-                FunctionRoundingImpl<ScaleMode::Positive>::apply(col->getData(), scale, vec_res);
-            }
-            else
-            {
-                size_t scale = intExp10(-scale_arg);
-                FunctionRoundingImpl<ScaleMode::Negative>::apply(col->getData(), scale, vec_res);
-            }
-        }
+            auto col_res = ColumnVector<T>::create();
 
-        return col_res;
+            typename ColumnVector<T>::Container & vec_res = col_res->getData();
+            vec_res.resize(value_col_typed->getData().size());
+
+            if (!vec_res.empty())
+            {
+                // Const scale argument:
+                if (scale_col == nullptr || isColumnConst(*scale_col))
+                {
+                    auto scale_arg = (scale_col == nullptr) ? 0 : getScaleArg(checkAndGetColumnConst<ColumnVector<ScaleType>>(scale_col));
+                    if (scale_arg == 0)
+                    {
+                        size_t scale = 1;
+                        FunctionRoundingImpl<ScaleMode::Zero>::apply(value_col_typed->getData(), scale, vec_res);
+                    }
+                    else if (scale_arg > 0)
+                    {
+                        size_t scale = intExp10(scale_arg);
+                        FunctionRoundingImpl<ScaleMode::Positive>::apply(value_col_typed->getData(), scale, vec_res);
+                    }
+                    else
+                    {
+                        size_t scale = intExp10(-scale_arg);
+                        FunctionRoundingImpl<ScaleMode::Negative>::apply(value_col_typed->getData(), scale, vec_res);
+                    }
+                }
+                /// Non-const scale argument:
+                else if (const auto * scale_col_typed = checkAndGetColumn<ColumnVector<ScaleType>>(scale_col))
+                {
+                    const auto & value_data = value_col_typed->getData();
+                    const auto & scale_data = scale_col_typed->getData();
+                    const size_t rows = value_data.size();
+
+                    for (size_t i = 0; i < rows; ++i)
+                    {
+                        Int64 scale64 = scale_data[i];
+                        validateScale(scale64);
+                        Scale raw_scale = scale64;
+
+                        if (raw_scale == 0)
+                        {
+                            size_t scale = 1;
+                            FunctionRoundingImpl<ScaleMode::Zero>::applyOne(value_data[i], scale, vec_res[i]);
+                        }
+                        else if (raw_scale > 0)
+                        {
+                            size_t scale = intExp10(raw_scale);
+                            FunctionRoundingImpl<ScaleMode::Positive>::applyOne(value_data[i], scale, vec_res[i]);
+                        }
+                        else
+                        {
+                            size_t scale = intExp10(-raw_scale);
+                            FunctionRoundingImpl<ScaleMode::Negative>::applyOne(value_data[i], scale, vec_res[i]);
+                        }
+                    }
+                }
+            }
+            return col_res;
+        }
+        // Const value argument:
+        const auto * value_col_typed_const = checkAndGetColumnConst<ColumnVector<T>>(value_col);
+        if (value_col_typed_const)
+        {
+            auto value_col_full = value_col_typed_const->convertToFullColumn();
+            return apply<ScaleType>(value_col_full.get(), scale_col);
+        }
+        return nullptr;
     }
 };
 
+/// Dispatcher for Decimal inputs
 template <is_decimal T, RoundingMode rounding_mode, TieBreakingMode tie_breaking_mode>
 struct Dispatcher<T, rounding_mode, tie_breaking_mode>
 {
 public:
-    static ColumnPtr apply(const IColumn * col_general, Scale scale_arg)
+    template <typename ScaleType>
+    static ColumnPtr apply(const IColumn * value_col, const IColumn * scale_col = nullptr)
     {
-        const auto * const col = checkAndGetColumn<ColumnDecimal<T>>(col_general);
-        const typename ColumnDecimal<T>::Container & vec_src = col->getData();
+        // Non-const value argument:
+        const auto * value_col_typed = checkAndGetColumn<ColumnDecimal<T>>(value_col);
+        if (value_col_typed)
+        {
+            const typename ColumnDecimal<T>::Container & vec_src = value_col_typed->getData();
 
-        auto col_res = ColumnDecimal<T>::create(vec_src.size(), col->getScale());
-        auto & vec_res = col_res->getData();
+            auto col_res = ColumnDecimal<T>::create(vec_src.size(), value_col_typed->getScale());
+            auto & vec_res = col_res->getData();
+            vec_res.resize(vec_src.size());
 
-        if (!vec_res.empty())
-            DecimalRoundingImpl<T, rounding_mode, tie_breaking_mode>::apply(col->getData(), col->getScale(), vec_res, scale_arg);
+            if (!vec_res.empty())
+            {
+                /// Const scale argument:
+                if (scale_col == nullptr || isColumnConst(*scale_col))
+                {
+                    auto scale_arg = scale_col == nullptr ? 0 : getScaleArg(checkAndGetColumnConst<ColumnVector<ScaleType>>(scale_col));
+                    DecimalRoundingImpl<T, rounding_mode, tie_breaking_mode>::apply(vec_src, value_col_typed->getScale(), vec_res, scale_arg);
+                }
+                /// Non-const scale argument:
+                else if (const auto * scale_col_typed = checkAndGetColumn<ColumnVector<ScaleType>>(scale_col))
+                {
+                    const auto & scale = scale_col_typed->getData();
+                    const size_t rows = vec_src.size();
 
-        return col_res;
+                    for (size_t i = 0; i < rows; ++i)
+                    {
+                        Int64 scale64 = scale[i];
+                        validateScale(scale64);
+                        Scale raw_scale = scale64;
+
+                        DecimalRoundingImpl<T, rounding_mode, tie_breaking_mode>::applyOne(value_col_typed->getElement(i), value_col_typed->getScale(),
+                            reinterpret_cast<typename ColumnDecimal<T>::NativeT&>(col_res->getElement(i)), raw_scale);
+                    }
+                }
+            }
+
+            return col_res;
+        }
+        // Const value argument:
+        const auto * value_col_typed_const = checkAndGetColumnConst<ColumnDecimal<T>>(value_col);
+        if (value_col_typed_const)
+        {
+            auto value_col_full = value_col_typed_const->convertToFullColumn();
+            return apply<ScaleType>(value_col_full.get(), scale_col);
+        }
+        return nullptr;
     }
 };
 
-/** A template for functions that round the value of an input parameter of type
-  * (U)Int8/16/32/64, Float32/64 or Decimal32/64/128, and accept an additional optional parameter (default is 0).
-  */
+/// Functions that round the value of an input parameter of type (U)Int8/16/32/64, Float32/64 or Decimal32/64/128.
+/// Accept an additional optional parameter of type (U)Int8/16/32/64 (0 by default).
 template <typename Name, RoundingMode rounding_mode, TieBreakingMode tie_breaking_mode>
 class FunctionRounding : public IFunction
 {
@@ -523,75 +679,55 @@ public:
     static constexpr auto name = Name::name;
     static FunctionPtr create(ContextPtr) { return std::make_shared<FunctionRounding>(); }
 
-    String getName() const override
-    {
-        return name;
-    }
-
+    String getName() const override { return name; }
     bool isVariadic() const override { return true; }
     size_t getNumberOfArguments() const override { return 0; }
     bool isSuitableForShortCircuitArgumentsExecution(const DataTypesWithConstInfo & /*arguments*/) const override { return false; }
-
-    /// Get result types by argument types. If the function does not apply to these arguments, throw an exception.
-    DataTypePtr getReturnTypeImpl(const DataTypes & arguments) const override
-    {
-        if ((arguments.empty()) || (arguments.size() > 2))
-            throw Exception("Number of arguments for function " + getName() + " doesn't match: passed "
-                + toString(arguments.size()) + ", should be 1 or 2.",
-                ErrorCodes::NUMBER_OF_ARGUMENTS_DOESNT_MATCH);
-
-        for (const auto & type : arguments)
-            if (!isNumber(type))
-                throw Exception("Illegal type " + arguments[0]->getName() + " of argument of function " + getName(),
-                    ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT);
-
-        return arguments[0];
-    }
-
-    static Scale getScaleArg(const ColumnsWithTypeAndName & arguments)
-    {
-        if (arguments.size() == 2)
-        {
-            const IColumn & scale_column = *arguments[1].column;
-            if (!isColumnConst(scale_column))
-                throw Exception("Scale argument for rounding functions must be constant", ErrorCodes::ILLEGAL_COLUMN);
-
-            Field scale_field = assert_cast<const ColumnConst &>(scale_column).getField();
-            if (scale_field.getType() != Field::Types::UInt64
-                && scale_field.getType() != Field::Types::Int64)
-                throw Exception("Scale argument for rounding functions must have integer type", ErrorCodes::ILLEGAL_COLUMN);
-
-            Int64 scale64 = scale_field.get<Int64>();
-            if (scale64 > std::numeric_limits<Scale>::max()
-                || scale64 < std::numeric_limits<Scale>::min())
-                throw Exception("Scale argument for rounding function is too large", ErrorCodes::ARGUMENT_OUT_OF_BOUND);
-
-            return scale64;
-        }
-        return 0;
-    }
-
     bool useDefaultImplementationForConstants() const override { return true; }
-    ColumnNumbers getArgumentsThatAreAlwaysConstant() const override { return {1}; }
+
+    DataTypePtr getReturnTypeImpl(const ColumnsWithTypeAndName & arguments) const override
+    {
+        FunctionArgumentDescriptors mandatory_args{
+            {"x", static_cast<FunctionArgumentDescriptor::TypeValidator>(&isNumber), nullptr, "A number to round"},
+        };
+        FunctionArgumentDescriptors optional_args{
+            {"N", static_cast<FunctionArgumentDescriptor::TypeValidator>(&isNativeInteger), nullptr, "The number of decimal places to round to"},
+        };
+        validateFunctionArguments(*this, arguments, mandatory_args, optional_args);
+
+        return arguments[0].type;
+    }
 
     ColumnPtr executeImpl(const ColumnsWithTypeAndName & arguments, const DataTypePtr &, size_t /*input_rows_count*/) const override
     {
-        const ColumnWithTypeAndName & column = arguments[0];
-        Scale scale_arg = getScaleArg(arguments);
+        const ColumnWithTypeAndName & value_arg = arguments[0];
 
         ColumnPtr res;
-        auto call = [&](const auto & types) -> bool
+        auto call_data = [&](const auto & types) -> bool
         {
             using Types = std::decay_t<decltype(types)>;
-            using DataType = typename Types::LeftType;
+            using DataType = typename Types::RightType;
 
-            if constexpr (IsDataTypeNumber<DataType> || IsDataTypeDecimal<DataType>)
+            if (arguments.size() > 1)
             {
-                using FieldType = typename DataType::FieldType;
-                res = Dispatcher<FieldType, rounding_mode, tie_breaking_mode>::apply(column.column.get(), scale_arg);
+                const ColumnWithTypeAndName & scale_column = arguments[1];
+
+                auto call_scale = [&](const auto & scaleTypes) -> bool
+                {
+                    using ScaleTypes = std::decay_t<decltype(scaleTypes)>;
+                    using ScaleType = typename ScaleTypes::RightType;
+
+                    res = Dispatcher<DataType, rounding_mode, tie_breaking_mode>::template apply<ScaleType>(value_arg.column.get(), scale_column.column.get());
+                    return true;
+                };
+
+                TypeIndex right_index = scale_column.type->getTypeId();
+                if (!callOnBasicType<void, true, false, false, false>(right_index, call_scale))
+                    throw Exception(ErrorCodes::ILLEGAL_COLUMN, "Scale argument for rounding functions must have integer type");
                 return true;
             }
-            return false;
+            res = Dispatcher<DataType, rounding_mode, tie_breaking_mode>::template apply<int>(value_arg.column.get());
+            return true;
         };
 
 #if !defined(__SSE4_1__)
@@ -600,14 +736,12 @@ public:
 
         if constexpr (rounding_mode == RoundingMode::Round)
             if (0 != fesetround(FE_TONEAREST))
-                throw Exception("Cannot set floating point rounding mode", ErrorCodes::CANNOT_SET_ROUNDING_MODE);
+                throw Exception(ErrorCodes::CANNOT_SET_ROUNDING_MODE, "Cannot set floating point rounding mode");
 #endif
 
-        if (!callOnIndexAndDataType<void>(column.type->getTypeId(), call))
-        {
-            throw Exception("Illegal column " + column.name + " of argument of function " + getName(),
-                    ErrorCodes::ILLEGAL_COLUMN);
-        }
+        TypeIndex left_index = value_arg.type->getTypeId();
+        if (!callOnBasicType<void, true, true, true, false>(left_index, call_data))
+            throw Exception(ErrorCodes::ILLEGAL_COLUMN, "Illegal column {} of argument of function {}", value_arg.name, getName());
 
         return res;
     }
@@ -624,9 +758,8 @@ public:
 };
 
 
-/** Rounds down to a number within explicitly specified array.
-  * If the value is less than the minimal bound - returns the minimal bound.
-  */
+/// Rounds down to a number within explicitly specified array.
+/// If the value is less than the minimal bound - returns the minimal bound.
 class FunctionRoundDown : public IFunction
 {
 public:
@@ -634,7 +767,6 @@ public:
     static FunctionPtr create(ContextPtr) { return std::make_shared<FunctionRoundDown>(); }
 
     String getName() const override { return name; }
-
     bool isVariadic() const override { return false; }
     size_t getNumberOfArguments() const override { return 2; }
     bool useDefaultImplementationForConstants() const override { return true; }
@@ -646,24 +778,24 @@ public:
         const DataTypePtr & type_x = arguments[0];
 
         if (!isNumber(type_x))
-            throw Exception{"Unsupported type " + type_x->getName()
-                            + " of first argument of function " + getName()
-                            + ", must be numeric type.", ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT};
+            throw Exception(ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT,
+                            "Unsupported type {} of first argument of function {}, must be numeric type.",
+                            type_x->getName(), getName());
 
         const DataTypeArray * type_arr = checkAndGetDataType<DataTypeArray>(arguments[1].get());
 
         if (!type_arr)
-            throw Exception{"Second argument of function " + getName()
-                            + ", must be array of boundaries to round to.", ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT};
+            throw Exception(ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT,
+                            "Second argument of function {}, must be array of boundaries to round to.", getName());
 
         const auto type_arr_nested = type_arr->getNestedType();
 
         if (!isNumber(type_arr_nested))
         {
-            throw Exception{"Elements of array of second argument of function " + getName()
-                            + " must be numeric type.", ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT};
+            throw Exception(ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT,
+                            "Elements of array of second argument of function {} must be numeric type.", getName());
         }
-        return getLeastSupertype({type_x, type_arr_nested});
+        return getLeastSupertype(DataTypes{type_x, type_arr_nested});
     }
 
     ColumnPtr executeImpl(const ColumnsWithTypeAndName & arguments, const DataTypePtr & result_type, size_t) const override
@@ -688,7 +820,7 @@ public:
         auto boundaries = typeid_cast<const ColumnConst &>(*array_column).getValue<Array>();
         size_t num_boundaries = boundaries.size();
         if (!num_boundaries)
-            throw Exception("Empty array is illegal for boundaries in " + getName() + " function", ErrorCodes::BAD_ARGUMENTS);
+            throw Exception(ErrorCodes::BAD_ARGUMENTS, "Empty array is illegal for boundaries in {} function", getName());
 
         if (!executeNum<UInt8>(in, out, boundaries)
             && !executeNum<UInt16>(in, out, boundaries)
@@ -702,9 +834,10 @@ public:
             && !executeNum<Float64>(in, out, boundaries)
             && !executeDecimal<Decimal32>(in, out, boundaries)
             && !executeDecimal<Decimal64>(in, out, boundaries)
-            && !executeDecimal<Decimal128>(in, out, boundaries))
+            && !executeDecimal<Decimal128>(in, out, boundaries)
+            && !executeDecimal<Decimal256>(in, out, boundaries))
         {
-            throw Exception{"Illegal column " + in->getName() + " of first argument of function " + getName(), ErrorCodes::ILLEGAL_COLUMN};
+            throw Exception(ErrorCodes::ILLEGAL_COLUMN, "Illegal column {} of first argument of function {}", in->getName(), getName());
         }
 
         return column_result;
@@ -741,7 +874,7 @@ private:
         using ValueType = typename Container::value_type;
         std::vector<ValueType> boundary_values(boundaries.size());
         for (size_t i = 0; i < boundaries.size(); ++i)
-            boundary_values[i] = boundaries[i].get<ValueType>();
+            boundary_values[i] = static_cast<ValueType>(boundaries[i].safeGet<ValueType>());
 
         ::sort(boundary_values.begin(), boundary_values.end());
         boundary_values.erase(std::unique(boundary_values.begin(), boundary_values.end()), boundary_values.end());
