@@ -4,7 +4,6 @@
 
 namespace DB::GatherUtils
 {
-#pragma GCC visibility push(hidden)
 
 template <typename T>
 struct NumericArraySource;
@@ -33,5 +32,4 @@ protected:
     ~ArraySourceVisitorImpl() = default;
 };
 
-#pragma GCC visibility pop
 }
