@@ -10,7 +10,7 @@ template <typename JSONParser>
 class VisitorJSONPathStar : public IVisitor<JSONParser>
 {
 public:
-    VisitorJSONPathStar(ASTPtr)
+    explicit VisitorJSONPathStar(ASTPtr)
     {
         current_index = 0;
     }
@@ -19,7 +19,6 @@ public:
 
     VisitorStatus apply(typename JSONParser::Element & element) const override
     {
-        typename JSONParser::Element result;
         typename JSONParser::Array array = element.getArray();
         element = array[current_index];
         return VisitorStatus::Ok;
