@@ -3,8 +3,8 @@
 [ ! -f public_suffix_list.dat ] && wget -nv -O public_suffix_list.dat https://publicsuffix.org/list/public_suffix_list.dat
 
 echo '%language=C++
-%define lookup-function-name is_valid
-%define class-name tldLookupHash
+%define lookup-function-name isValid
+%define class-name TopLevelDomainLookupHash
 %readonly-tables
 %includes
 %compare-strncmp
@@ -13,8 +13,10 @@ echo '%language=C++
 #pragma GCC diagnostic ignored "-Wimplicit-fallthrough"
 #pragma GCC diagnostic ignored "-Wzero-as-null-pointer-constant"
 #pragma GCC diagnostic ignored "-Wunused-macros"
+// NOLINTBEGIN(modernize-macro-to-enum)
 %}
 # List generated using https://publicsuffix.org/list/public_suffix_list.dat
 %%' > tldLookup.gperf
 grep -v "//" public_suffix_list.dat | grep . | grep "\." | grep -ve "\..*\..*" >> tldLookup.gperf
 echo "%%" >> tldLookup.gperf
+echo '// NOLINTEND(modernize-macro-to-enum)' >> tldLookup.gperf
