@@ -1,4 +1,4 @@
-#include <Functions/IFunction.h>
+#include <Functions/array/emptyArrayToSingle.h>
 #include <Functions/FunctionFactory.h>
 #include <Functions/FunctionHelpers.h>
 #include <DataTypes/DataTypeArray.h>
@@ -20,36 +20,6 @@ namespace ErrorCodes
     extern const int ILLEGAL_TYPE_OF_ARGUMENT;
 }
 
-
-/** emptyArrayToSingle(arr) - replace empty arrays with arrays of one element with a default value.
-  */
-class FunctionEmptyArrayToSingle : public IFunction
-{
-public:
-    static constexpr auto name = "emptyArrayToSingle";
-    static FunctionPtr create(ContextPtr) { return std::make_shared<FunctionEmptyArrayToSingle>(); }
-
-    String getName() const override { return name; }
-
-    size_t getNumberOfArguments() const override { return 1; }
-    bool useDefaultImplementationForConstants() const override { return true; }
-    bool useDefaultImplementationForLowCardinalityColumns() const override { return false; }
-    bool isSuitableForShortCircuitArgumentsExecution(const DataTypesWithConstInfo & /*arguments*/) const override { return false; }
-
-    DataTypePtr getReturnTypeImpl(const DataTypes & arguments) const override
-    {
-        const DataTypeArray * array_type = checkAndGetDataType<DataTypeArray>(arguments[0].get());
-        if (!array_type)
-            throw Exception("Argument for function " + getName() + " must be array.",
-                ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT);
-
-        return arguments[0];
-    }
-
-    ColumnPtr executeImpl(const ColumnsWithTypeAndName & arguments, const DataTypePtr & result_type, size_t input_rows_count) const override;
-};
-
-
 namespace
 {
     namespace FunctionEmptyArrayToSingleImpl
@@ -66,11 +36,9 @@ namespace
                         input_rows_count,
                         Array{nested_type->getDefault()});
                 }
-                else
-                    return arguments[0].column;
+                return arguments[0].column;
             }
-            else
-                return nullptr;
+            return nullptr;
         }
 
         template <typename T, bool nullable>
@@ -127,8 +95,7 @@ namespace
 
                 return true;
             }
-            else
-                return false;
+            return false;
         }
 
 
@@ -146,7 +113,7 @@ namespace
 
                 auto * concrete_res_data = typeid_cast<ColumnFixedString *>(&res_data_col);
                 if (!concrete_res_data)
-                    throw Exception{"Internal error", ErrorCodes::LOGICAL_ERROR};
+                    throw Exception(ErrorCodes::LOGICAL_ERROR, "Internal error");
 
                 ColumnFixedString::Chars & res_data = concrete_res_data->getChars();
                 size_t size = src_offsets.size();
@@ -194,8 +161,7 @@ namespace
 
                 return true;
             }
-            else
-                return false;
+            return false;
         }
 
 
@@ -212,14 +178,14 @@ namespace
 
                 auto * concrete_res_string_offsets = typeid_cast<ColumnString *>(&res_data_col);
                 if (!concrete_res_string_offsets)
-                    throw Exception{"Internal error", ErrorCodes::LOGICAL_ERROR};
+                    throw Exception(ErrorCodes::LOGICAL_ERROR, "Internal error");
                 ColumnString::Offsets & res_string_offsets = concrete_res_string_offsets->getOffsets();
 
                 const ColumnString::Chars & src_data_vec = src_data_concrete->getChars();
 
                 auto * concrete_res_data = typeid_cast<ColumnString *>(&res_data_col);
                 if (!concrete_res_data)
-                    throw Exception{"Internal error", ErrorCodes::LOGICAL_ERROR};
+                    throw Exception(ErrorCodes::LOGICAL_ERROR, "Internal error");
                 ColumnString::Chars & res_data = concrete_res_data->getChars();
 
                 size_t size = src_array_offsets.size();
@@ -270,12 +236,9 @@ namespace
                     }
                     else
                     {
-                        res_data.push_back(0);  /// An empty string, including zero at the end.
-
                         if (nullable)
                             res_null_map->push_back(1);
 
-                        ++res_string_prev_offset;
                         res_string_offsets.push_back(res_string_prev_offset);
 
                         ++res_array_prev_offset;
@@ -290,8 +253,7 @@ namespace
 
                 return true;
             }
-            else
-                return false;
+            return false;
         }
 
 
@@ -367,6 +329,14 @@ namespace
     }
 }
 
+DataTypePtr FunctionEmptyArrayToSingle::getReturnTypeImpl(const DataTypes & arguments) const
+{
+    const DataTypeArray * array_type = checkAndGetDataType<DataTypeArray>(arguments[0].get());
+    if (!array_type)
+        throw Exception(ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT, "Argument for function {} must be array.", getName());
+
+    return arguments[0];
+}
 
 ColumnPtr FunctionEmptyArrayToSingle::executeImpl(const ColumnsWithTypeAndName & arguments, const DataTypePtr & result_type, size_t input_rows_count) const
 {
@@ -375,8 +345,8 @@ ColumnPtr FunctionEmptyArrayToSingle::executeImpl(const ColumnsWithTypeAndName &
 
     const ColumnArray * array = checkAndGetColumn<ColumnArray>(arguments[0].column.get());
     if (!array)
-        throw Exception("Illegal column " + arguments[0].column->getName() + " of first argument of function " + getName(),
-            ErrorCodes::ILLEGAL_COLUMN);
+        throw Exception(ErrorCodes::ILLEGAL_COLUMN, "Illegal column {} of first argument of function {}",
+            arguments[0].column->getName(), getName());
 
     MutableColumnPtr res_ptr = array->cloneEmpty();
     ColumnArray & res = assert_cast<ColumnArray &>(*res_ptr);
@@ -392,7 +362,7 @@ ColumnPtr FunctionEmptyArrayToSingle::executeImpl(const ColumnsWithTypeAndName &
     const IColumn * inner_col;
     IColumn * inner_res_col;
 
-    const auto * nullable_col = checkAndGetColumn<ColumnNullable>(src_data);
+    const auto * nullable_col = checkAndGetColumn<ColumnNullable>(&src_data);
     if (nullable_col)
     {
         inner_col = &nullable_col->getNestedColumn();
@@ -417,9 +387,36 @@ ColumnPtr FunctionEmptyArrayToSingle::executeImpl(const ColumnsWithTypeAndName &
 }
 
 
-void registerFunctionEmptyArrayToSingle(FunctionFactory & factory)
+REGISTER_FUNCTION(EmptyArrayToSingle)
 {
-    factory.registerFunction<FunctionEmptyArrayToSingle>();
+    FunctionDocumentation::Description description = R"(
+Accepts an empty array and returns a one-element array that is equal to the default value.
+    )";
+    FunctionDocumentation::Syntax syntax = "emptyArrayToSingle(arr)";
+    FunctionDocumentation::Arguments arguments = {{"arr", "An empty array.", {"Array(T)"}}};
+    FunctionDocumentation::ReturnedValue returned_value = {"An array with a single value of the Array's default type.", {"Array(T)"}};
+    FunctionDocumentation::Examples examples = {{"Basic example", R"(
+CREATE TABLE test (
+  a Array(Int32),
+  b Array(String),
+  c Array(DateTime)
+)
+ENGINE = MergeTree
+ORDER BY tuple();
+
+INSERT INTO test VALUES ([], [], []);
+
+SELECT emptyArrayToSingle(a), emptyArrayToSingle(b), emptyArrayToSingle(c) FROM test;
+)", R"(
+┌─emptyArrayToSingle(a)─┬─emptyArrayToSingle(b)─┬─emptyArrayToSingle(c)───┐
+│ [0]                   │ ['']                  │ ['1970-01-01 01:00:00'] │
+└───────────────────────┴───────────────────────┴─────────────────────────┘
+    )"}};
+    FunctionDocumentation::IntroducedIn introduced_in = {1, 1};
+    FunctionDocumentation::Category category = FunctionDocumentation::Category::Array;
+    FunctionDocumentation documentation = {description, syntax, arguments, returned_value, examples, introduced_in, category};
+
+    factory.registerFunction<FunctionEmptyArrayToSingle>(documentation);
 }
 
 }
