@@ -1,6 +1,6 @@
-#include "FunctionsStringSearch.h"
-#include "FunctionFactory.h"
-#include "CountSubstringsImpl.h"
+#include <Functions/FunctionsStringSearch.h>
+#include <Functions/FunctionFactory.h>
+#include <Functions/CountSubstringsImpl.h>
 
 
 namespace DB
@@ -17,7 +17,7 @@ using FunctionCountSubstringsCaseInsensitive = FunctionsStringSearch<CountSubstr
 
 }
 
-void registerFunctionCountSubstringsCaseInsensitive(FunctionFactory & factory)
+REGISTER_FUNCTION(CountSubstringsCaseInsensitive)
 {
     factory.registerFunction<FunctionCountSubstringsCaseInsensitive>();
 }
