@@ -1,7 +1,7 @@
-#include "FunctionsMultiStringSearch.h"
-#include "FunctionFactory.h"
-#include "MultiSearchFirstIndexImpl.h"
-#include "PositionImpl.h"
+#include <Functions/FunctionsMultiStringSearch.h>
+#include <Functions/FunctionFactory.h>
+#include <Functions/MultiSearchFirstIndexImpl.h>
+#include <Functions/PositionImpl.h>
 
 
 namespace DB
@@ -19,7 +19,7 @@ using FunctionMultiSearchFirstIndexCaseInsensitiveUTF8
 
 }
 
-void registerFunctionMultiSearchFirstIndexCaseInsensitiveUTF8(FunctionFactory & factory)
+REGISTER_FUNCTION(MultiSearchFirstIndexCaseInsensitiveUTF8)
 {
     factory.registerFunction<FunctionMultiSearchFirstIndexCaseInsensitiveUTF8>();
 }
