@@ -1,16 +1,16 @@
 #pragma once
 
 #include <IO/ReadBuffer.h>
-#include <IO/BufferWithOwnMemory.h>
+#include <IO/CompressedReadBufferWrapper.h>
 
 
 namespace DB
 {
 
-class Bzip2ReadBuffer : public BufferWithOwnMemory<ReadBuffer>
+class Bzip2ReadBuffer : public CompressedReadBufferWrapper
 {
 public:
-    Bzip2ReadBuffer(
+    explicit Bzip2ReadBuffer(
             std::unique_ptr<ReadBuffer> in_,
             size_t buf_size = DBMS_DEFAULT_BUFFER_SIZE,
             char * existing_memory = nullptr,
@@ -21,8 +21,6 @@ public:
 private:
     bool nextImpl() override;
 
-    std::unique_ptr<ReadBuffer> in;
-
     class Bzip2StateWrapper;
     std::unique_ptr<Bzip2StateWrapper> bz;
 
