@@ -16,21 +16,22 @@ namespace DB
 class MemoryWriteBuffer : public WriteBuffer, public IReadableWriteBuffer, boost::noncopyable, private Allocator<false>
 {
 public:
-
     /// Use max_total_size_ = 0 for unlimited storage
-    MemoryWriteBuffer(
+    explicit MemoryWriteBuffer(
         size_t max_total_size_ = 0,
         size_t initial_chunk_size_ = DBMS_DEFAULT_BUFFER_SIZE,
         double growth_rate_ = 2.0,
         size_t max_chunk_size_ = 128 * DBMS_DEFAULT_BUFFER_SIZE);
 
-    void nextImpl() override;
-
     ~MemoryWriteBuffer() override;
 
 protected:
 
-    std::shared_ptr<ReadBuffer> getReadBufferImpl() override;
+    void nextImpl() override;
+
+    void finalizeImpl() override { /* no op */ }
+
+    std::unique_ptr<ReadBuffer> getReadBufferImpl() override;
 
     const size_t max_total_size;
     const size_t initial_chunk_size;
