@@ -21,9 +21,8 @@ std::string ReadBufferFromFileDecorator::getFileName() const
 {
     if (!file_name.empty())
         return file_name;
-    if (ReadBufferFromFileBase * buffer = dynamic_cast<ReadBufferFromFileBase *>(impl.get()))
-        return buffer->getFileName();
-    return std::string();
+
+    return getFileNameFromReadBuffer(*impl);
 }
 
 
@@ -53,4 +52,9 @@ bool ReadBufferFromFileDecorator::nextImpl()
     return result;
 }
 
+std::optional<size_t> ReadBufferFromFileDecorator::tryGetFileSize()
+{
+    return tryGetFileSizeFromReadBuffer(*impl);
+}
+
 }
