@@ -12,18 +12,27 @@ class WriteBuffer;
 
 /// WriteBuffer that decorates data and delegates it to underlying buffer.
 /// It's used for writing compressed and encrypted data
-template <class Base>
+/// This class can own or not own underlying buffer - constructor will differentiate
+/// std::unique_ptr<WriteBuffer> for owning and WriteBuffer* for not owning.
+template <typename Base>
 class WriteBufferDecorator : public Base
 {
 public:
     template <class ... BaseArgs>
     explicit WriteBufferDecorator(std::unique_ptr<WriteBuffer> out_, BaseArgs && ... args)
-        : Base(std::forward<BaseArgs>(args)...), out(std::move(out_))
+        : Base(std::forward<BaseArgs>(args)...), owning_holder(std::move(out_)), out(owning_holder.get())
+    {
+    }
+
+    template <class ... BaseArgs>
+    explicit WriteBufferDecorator(WriteBuffer * out_, BaseArgs && ... args)
+        : Base(std::forward<BaseArgs>(args)...), out(out_)
     {
     }
 
     void finalizeImpl() override
     {
+        Base::finalizeImpl();
         try
         {
             finalizeBefore();
@@ -38,7 +47,13 @@ public:
         }
     }
 
-    WriteBuffer * getNestedBuffer() { return out.get(); }
+    void cancelImpl() noexcept override
+    {
+        Base::cancelImpl();
+        out->cancel();
+    }
+
+    WriteBuffer * getNestedBuffer() { return out; }
 
 protected:
     /// Do some finalization before finalization of underlying buffer.
@@ -47,7 +62,8 @@ protected:
     /// Do some finalization after finalization of underlying buffer.
     virtual void finalizeAfter() {}
 
-    std::unique_ptr<WriteBuffer> out;
+    std::unique_ptr<WriteBuffer> owning_holder;
+    WriteBuffer * out;
 };
 
 using WriteBufferWithOwnMemoryDecorator = WriteBufferDecorator<BufferWithOwnMemory<WriteBuffer>>;
