@@ -1,6 +1,6 @@
 #pragma once
 
-#include <iostream>
+#include <iosfwd>
 
 #include <IO/WriteBuffer.h>
 #include <IO/BufferWithOwnMemory.h>
@@ -12,16 +12,14 @@ namespace DB
 class WriteBufferFromOStream : public BufferWithOwnMemory<WriteBuffer>
 {
 public:
-    WriteBufferFromOStream(
+    explicit WriteBufferFromOStream(
         std::ostream & ostr_,
         size_t size = DBMS_DEFAULT_BUFFER_SIZE,
         char * existing_memory = nullptr,
         size_t alignment = 0);
 
-    ~WriteBufferFromOStream() override;
-
 protected:
-    WriteBufferFromOStream(size_t size = DBMS_DEFAULT_BUFFER_SIZE, char * existing_memory = nullptr, size_t alignment = 0);
+    explicit WriteBufferFromOStream(size_t size = DBMS_DEFAULT_BUFFER_SIZE, char * existing_memory = nullptr, size_t alignment = 0);
 
     void nextImpl() override;
 
