@@ -1,6 +1,6 @@
 #pragma once
 
-#include <IO/BufferWithOwnMemory.h>
+#include <IO/CompressedReadBufferWrapper.h>
 #include <IO/CompressionMethod.h>
 #include <IO/ReadBuffer.h>
 
@@ -13,21 +13,21 @@ namespace ErrorCodes
 {
 }
 
-class ZstdInflatingReadBuffer : public BufferWithOwnMemory<ReadBuffer>
+class ZstdInflatingReadBuffer : public CompressedReadBufferWrapper
 {
 public:
-    ZstdInflatingReadBuffer(
+    explicit ZstdInflatingReadBuffer(
         std::unique_ptr<ReadBuffer> in_,
         size_t buf_size = DBMS_DEFAULT_BUFFER_SIZE,
         char * existing_memory = nullptr,
-        size_t alignment = 0);
+        size_t alignment = 0,
+        int zstd_window_log_max = 0);
 
     ~ZstdInflatingReadBuffer() override;
 
 private:
     bool nextImpl() override;
 
-    std::unique_ptr<ReadBuffer> in;
     ZSTD_DCtx * dctx;
     ZSTD_inBuffer input;
     ZSTD_outBuffer output;
