@@ -2,6 +2,8 @@
 #include <IO/WriteBufferFromFileDescriptor.h>
 #include <IO/WriteBufferFromString.h>
 
+#include <iostream>
+
 
 int main(int, char **)
 {
@@ -17,6 +19,7 @@ int main(int, char **)
             << LocalDate(time(nullptr)) << '\n'
             << 1234567890123456789UL << '\n'
             << DB::flush;
+        buf.finalize();
     }
 
     {
@@ -35,6 +38,7 @@ int main(int, char **)
         DB::WriteBufferFromFileDescriptor buf(STDOUT_FILENO);
         size_t x = 11;
         buf << "Column " << x << ", \n";
+        buf.finalize();
     }
 
     return 0;
