@@ -2,21 +2,19 @@
 
 #include <iostream>
 
-#include <base/types.h>
-#include <IO/ReadHelpers.h>
+#include <IO/ReadBuffer.h>
 #include <IO/ReadBufferFromString.h>
+#include <IO/ReadHelpers.h>
+#include <base/types.h>
 
-
-int main(int, char **)
+int readAndPrint(DB::ReadBuffer & in)
 {
     try
     {
-        std::string s = "-123456 123.456 вася пе\\tтя\t'\\'xyz\\\\'";
-        DB::ReadBufferFromString in(s);
-
-        DB::Int64 a;
-        DB::Float64 b;
-        DB::String c, d;
+        Int64 a;
+        Float64 b;
+        String c;
+        String d;
 
         DB::readIntText(a, in);
         in.ignore();
@@ -31,12 +29,32 @@ int main(int, char **)
 
         std::cout << a << ' ' << b << ' ' << c << '\t' << '\'' << d << '\'' << std::endl;
         std::cout << in.count() << std::endl;
+        return 0;
     }
     catch (const DB::Exception & e)
     {
         std::cerr << e.what() << ", " << e.displayText() << std::endl;
         return 1;
     }
+}
+
+int main(int, char **)
+{
+    {
+        std::string s = "-123456 123.456 вася pe\\ttya\t'\\'xyz\\\\'";
+        DB::ReadBufferFromString in(s);
+        if (readAndPrint(in))
+            std::cout << "readAndPrint from ReadBufferFromString failed" << std::endl;
+    }
+
+
+    std::shared_ptr<DB::ReadBufferFromOwnString> in;
+    {
+        std::string s = "-123456 123.456 вася pe\\ttya\t'\\'xyz\\\\'";
+        in = std::make_shared<DB::ReadBufferFromOwnString>(s);
+    }
+    if (readAndPrint(*in))
+        std::cout << "readAndPrint from ReadBufferFromOwnString failed" << std::endl;
 
     return 0;
 }
