@@ -17,7 +17,12 @@ public:
 
     BlockIO execute() override;
 
-    static void updateUserFromQuery(User & user, const ASTCreateUserQuery & query);
+    static void updateUserFromQuery(
+        User & user,
+        const ASTCreateUserQuery & query,
+        bool allow_no_password,
+        bool allow_plaintext_password,
+        std::size_t max_number_of_authentication_methods);
 
 private:
     ASTPtr query_ptr;
