@@ -3,6 +3,8 @@
 #include <Interpreters/SystemLog.h>
 #include <Core/NamesAndTypes.h>
 #include <Core/NamesAndAliases.h>
+#include <Core/Field.h>
+#include <Storages/ColumnsDescription.h>
 
 
 /// Call this function on crash.
@@ -26,7 +28,7 @@ struct CrashLogElement
     Array trace_full;
 
     static std::string name() { return "CrashLog"; }
-    static NamesAndTypesList getNamesAndTypes();
+    static ColumnsDescription getColumnsDescription();
     static NamesAndAliases getNamesAndAliases() { return {}; }
     void appendToBlock(MutableColumns & columns) const;
 };
@@ -41,8 +43,13 @@ class CrashLog : public SystemLog<CrashLogElement>
 public:
     static void initialize(std::shared_ptr<CrashLog> crash_log_)
     {
-        crash_log = std::move(crash_log_);
+        crash_log = crash_log_;
     }
+
+    static consteval size_t getDefaultMaxSize() { return 1024; }
+    static consteval size_t getDefaultReservedSize() { return 1024; }
+    static consteval size_t getDefaultFlushIntervalMilliseconds() { return 1000; }
+    static consteval size_t shouldNotifyFlushOnCrash() { return true; }
 };
 
 }
