@@ -1,7 +1,7 @@
 #pragma once
 
 #include <Interpreters/InDepthNodeVisitor.h>
-#include <Parsers/IAST.h>
+#include <Parsers/IAST_fwd.h>
 
 
 namespace DB
