@@ -1,11 +1,10 @@
-#include "config_core.h"
+#include "config.h"
 
 #if USE_EMBEDDED_COMPILER
 
-#include <optional>
 #include <stack>
 
-#include <base/logger_useful.h>
+#include <Common/logger_useful.h>
 #include <base/sort.h>
 #include <Columns/ColumnConst.h>
 #include <Columns/ColumnNullable.h>
@@ -14,7 +13,6 @@
 #include <Common/assert_cast.h>
 #include <DataTypes/DataTypeNullable.h>
 #include <DataTypes/DataTypesNumber.h>
-#include <Functions/FunctionsComparison.h>
 #include <DataTypes/Native.h>
 #include <Functions/IFunctionAdaptors.h>
 
@@ -38,10 +36,9 @@ static CHJIT & getJITInstance()
     return jit;
 }
 
-static Poco::Logger * getLogger()
+static LoggerPtr getLogger()
 {
-    static Poco::Logger & logger = Poco::Logger::get("ExpressionJIT");
-    return &logger;
+    return ::getLogger("ExpressionJIT");
 }
 
 class CompiledFunctionHolder : public CompiledExpressionCacheEntry
@@ -113,14 +110,14 @@ public:
                 const auto & null_map_column = nullable_column->getNullMapColumn();
 
                 auto nested_column_raw_data = nested_column.getRawData();
-                __msan_unpoison(nested_column_raw_data.data, nested_column_raw_data.size);
+                __msan_unpoison(nested_column_raw_data.data(), nested_column_raw_data.size());
 
                 auto null_map_column_raw_data = null_map_column.getRawData();
-                __msan_unpoison(null_map_column_raw_data.data, null_map_column_raw_data.size);
+                __msan_unpoison(null_map_column_raw_data.data(), null_map_column_raw_data.size());
             }
             else
             {
-                __msan_unpoison(result_column->getRawData().data, result_column->getRawData().size);
+                __msan_unpoison(result_column->getRawData().data(), result_column->getRawData().size());
             }
 
             #endif
@@ -160,9 +157,9 @@ public:
 
     bool isCompilable() const override { return true; }
 
-    llvm::Value * compile(llvm::IRBuilderBase & builder, Values values) const override
+    llvm::Value * compile(llvm::IRBuilderBase & builder, const ValuesWithType & arguments) const override
     {
-        return dag.compile(builder, values);
+        return dag.compile(builder, arguments).value;
     }
 
     bool isSuitableForShortCircuitArgumentsExecution(const DataTypesWithConstInfo & arguments) const override
@@ -263,11 +260,11 @@ public:
         return result;
     }
 
-    static void applyFunction(IFunctionBase & function, Field & value)
+    static void applyFunction(const IFunctionBase & function, Field & value)
     {
         const auto & type = function.getArgumentTypes().at(0);
         ColumnsWithTypeAndName args{{type->createColumnConst(1, value), type, "x" }};
-        auto col = function.execute(args, function.getResultType(), 1);
+        auto col = function.execute(args, function.getResultType(), 1, /* dry_run = */ false);
         col->get(0, value);
     }
 
@@ -338,7 +335,7 @@ static bool isCompilableFunction(const ActionsDAG::Node & node, const std::unord
     if (node.type != ActionsDAG::ActionType::FUNCTION)
         return false;
 
-    auto & function = *node.function_base;
+    const auto & function = *node.function_base;
 
     IFunction::ShortCircuitSettings settings;
     if (function.isShortCircuit(settings, node.children.size()))
@@ -500,7 +497,7 @@ void ActionsDAG::compileFunctions(size_t min_count_to_compile_expression, const
 
             while (current_frame.next_child_to_visit < current_node->children.size())
             {
-                const auto & child = node.children[current_frame.next_child_to_visit];
+                const auto & child = current_node->children[current_frame.next_child_to_visit];
 
                 if (visited_nodes.contains(child))
                 {
@@ -551,10 +548,10 @@ void ActionsDAG::compileFunctions(size_t min_count_to_compile_expression, const
             node_to_data[child].all_parents_compilable &= node_is_valid_for_compilation;
     }
 
-    for (const auto & node : index)
+    for (const auto & output_node : outputs)
     {
-        /// Force result nodes to compile
-        node_to_data[node].all_parents_compilable = false;
+        /// Force output nodes to compile
+        node_to_data[output_node].all_parents_compilable = false;
     }
 
     std::vector<Node *> nodes_to_compile;
