@@ -1,6 +1,5 @@
 #pragma once
 
-#include <AggregateFunctions/AggregateFunctionQuantile.h>
 #include <Interpreters/InDepthNodeVisitor.h>
 #include <Parsers/IAST_fwd.h>
 
@@ -26,6 +25,8 @@ public:
 
     void visit(ASTFunction & function, ASTPtr & ast);
 
+    static String toFusedNameOrSelf(const String & func_name);
+
     static String getFusedName(const String & func_name);
 
     static bool needChild(const ASTPtr & node, const ASTPtr &);
