@@ -3,6 +3,7 @@
 #include <QueryPipeline/BlockIO.h>
 #include <Interpreters/Context_fwd.h>
 #include <Parsers/IAST_fwd.h>
+#include <Storages//IStorage_fwd.h>
 
 namespace DB
 {
@@ -33,6 +34,13 @@ public:
 
     virtual void extendQueryLogElemImpl(QueryLogElement &, const ASTPtr &, ContextPtr) const {}
 
+    /// Returns true if transactions maybe supported for this type of query.
+    /// If Interpreter returns true, than it is responsible to check that specific query with specific Storage is supported.
+    virtual bool supportsTransactions() const { return false; }
+
+    /// Helper function for some Interpreters.
+    static void checkStorageSupportsTransactionsIfNeeded(const StoragePtr & storage, ContextPtr context, bool is_readonly_query = false);
+
     virtual ~IInterpreter() = default;
 };
 
