@@ -1,13 +1,14 @@
 #pragma once
 
-#include <Interpreters/SystemLog.h>
+#include <Interpreters/PeriodicLog.h>
 #include <Common/ProfileEvents.h>
 #include <Common/CurrentMetrics.h>
+#include <Common/ThreadPool_fwd.h>
 #include <Core/NamesAndTypes.h>
 #include <Core/NamesAndAliases.h>
+#include <Storages/ColumnsDescription.h>
 
 #include <vector>
-#include <atomic>
 #include <ctime>
 
 
@@ -21,37 +22,27 @@ struct MetricLogElement
 {
     time_t event_time{};
     Decimal64 event_time_microseconds{};
-    UInt64 milliseconds{};
 
     std::vector<ProfileEvents::Count> profile_events;
     std::vector<CurrentMetrics::Metric> current_metrics;
 
     static std::string name() { return "MetricLog"; }
-    static NamesAndTypesList getNamesAndTypes();
+    static ColumnsDescription getColumnsDescription();
     static NamesAndAliases getNamesAndAliases() { return {}; }
     void appendToBlock(MutableColumns & columns) const;
 };
 
-
-class MetricLog : public SystemLog<MetricLogElement>
+class MetricLog : public PeriodicLog<MetricLogElement>
 {
-    using SystemLog<MetricLogElement>::SystemLog;
+    using PeriodicLog<MetricLogElement>::PeriodicLog;
 
-public:
-    void shutdown() override;
-
-    /// Launches a background thread to collect metrics with interval
-    void startCollectMetric(size_t collect_interval_milliseconds_);
-
-    /// Stop background thread. Call before shutdown.
-    void stopCollectMetric();
+protected:
+    void stepFunction(TimePoint current_time) override;
 
 private:
-    void metricThreadFunction();
-
-    ThreadFromGlobalPool metric_flush_thread;
-    size_t collect_interval_milliseconds;
-    std::atomic<bool> is_shutdown_metric_thread{false};
+    /// stepFunction and flushBufferToLog may be executed concurrently, hence the mutex
+    std::vector<ProfileEvents::Count> previous_profile_events TSA_GUARDED_BY(previous_profile_events_mutex) = std::vector<ProfileEvents::Count>(ProfileEvents::end());
+    mutable std::mutex previous_profile_events_mutex;
 };
 
 }
