@@ -1,6 +1,7 @@
 #pragma once
 
 #include <Core/Names.h>
+#include <Core/Types.h>
 #include <Parsers/IAST_fwd.h>
 
 namespace DB
@@ -19,9 +20,14 @@ public:
 
     void visit(ASTPtr & ast);
 
+    size_t getNumberOfReplacedParameters() const { return num_replaced_parameters; }
+
 private:
     const NameToNameMap & query_parameters;
+    size_t num_replaced_parameters = 0;
+
     const String & getParamValue(const String & name);
+    void resolveParameterizedAlias(ASTPtr & ast);
     void visitIdentifier(ASTPtr & ast);
     void visitQueryParameter(ASTPtr & ast);
     void visitChildren(ASTPtr & ast);
