@@ -1,11 +1,10 @@
 #pragma once
 
+#include <Core/Names.h>
 #include <Interpreters/Context_fwd.h>
+#include <Common/COW.h>
 
 #include <memory>
-#include <string>
-#include <unordered_map>
-
 
 namespace DB
 {
@@ -14,13 +13,19 @@ class Block;
 class NamesAndTypesList;
 class ColumnsDescription;
 
+class IColumn;
+using ColumnPtr = COW<IColumn>::Ptr;
+using Columns = std::vector<ColumnPtr>;
+
+struct StorageInMemoryMetadata;
+using StorageMetadataPtr = std::shared_ptr<const StorageInMemoryMetadata>;
+
 class ActionsDAG;
-using ActionsDAGPtr = std::shared_ptr<ActionsDAG>;
 
 /// Create actions which adds missing defaults to block according to required_columns using columns description
 /// or substitute NULL into DEFAULT value in case of INSERT SELECT query (null_as_default) if according setting is 1.
 /// Return nullptr if no actions required.
-ActionsDAGPtr evaluateMissingDefaults(
+std::optional<ActionsDAG> evaluateMissingDefaults(
     const Block & header,
     const NamesAndTypesList & required_columns,
     const ColumnsDescription & columns,
@@ -31,4 +36,12 @@ ActionsDAGPtr evaluateMissingDefaults(
 /// Tries to convert columns in block to required_columns
 void performRequiredConversions(Block & block, const NamesAndTypesList & required_columns, ContextPtr context);
 
+void fillMissingColumns(
+    Columns & res_columns,
+    size_t num_rows,
+    const NamesAndTypesList & requested_columns,
+    const NamesAndTypesList & available_columns,
+    const NameSet & partially_read_columns,
+    StorageMetadataPtr metadata_snapshot);
+
 }
