@@ -8,6 +8,8 @@ namespace DB
 
 struct StorageInMemoryMetadata;
 using StorageMetadataPtr = std::shared_ptr<const StorageInMemoryMetadata>;
+class ColumnsDescription;
+struct StorageID;
 
 /// Process column transformers (e.g. * EXCEPT(a)), asterisks and qualified columns.
 ASTPtr processColumnTransformers(
@@ -16,4 +18,10 @@ ASTPtr processColumnTransformers(
         const StorageMetadataPtr & metadata_snapshot,
         ASTPtr query_columns);
 
+ASTPtr processColumnTransformers(
+        const String & current_database,
+        const StorageID & table_id,
+        const ColumnsDescription & columns,
+        ASTPtr query_columns);
+
 }
