@@ -31,7 +31,18 @@ public:
     static String toString(BigNum num);
     static String toString(Rational ratio);
 
-    void formatImpl(const FormatSettings & settings, FormatState &, FormatStateStacked) const override;
+protected:
+    void formatImpl(WriteBuffer & ostr, const FormatSettings &, FormatState &, FormatStateStacked) const override;
 };
 
+inline bool operator==(const ASTSampleRatio::Rational & lhs, const ASTSampleRatio::Rational & rhs)
+{
+    return lhs.numerator == rhs.numerator && lhs.denominator == rhs.denominator;
+}
+
+inline bool operator!=(const ASTSampleRatio::Rational & lhs, const ASTSampleRatio::Rational & rhs)
+{
+    return !(lhs == rhs);
+}
+
 }
