@@ -17,7 +17,7 @@ struct ASTWindowDefinition : public IAST
     ASTPtr order_by;
 
     bool frame_is_default = true;
-    WindowFrame::FrameType frame_type = WindowFrame::FrameType::Range;
+    WindowFrame::FrameType frame_type = WindowFrame::FrameType::RANGE;
     WindowFrame::BoundaryType frame_begin_type = WindowFrame::BoundaryType::Unbounded;
     ASTPtr frame_begin_offset;
     bool frame_begin_preceding = true;
@@ -29,9 +29,19 @@ struct ASTWindowDefinition : public IAST
 
     String getID(char delimiter) const override;
 
-    void formatImpl(const FormatSettings & settings, FormatState & state, FormatStateStacked frame) const override;
-
     std::string getDefaultWindowName() const;
+
+    void forEachPointerToChild(std::function<void(void**)> f) override
+    {
+        f(reinterpret_cast<void **>(&partition_by));
+        f(reinterpret_cast<void **>(&order_by));
+        f(reinterpret_cast<void **>(&frame_begin_offset));
+        f(reinterpret_cast<void **>(&frame_end_offset));
+    }
+
+
+protected:
+    void formatImpl(WriteBuffer & ostr, const FormatSettings & settings, FormatState & state, FormatStateStacked frame) const override;
 };
 
 struct ASTWindowListElement : public IAST
@@ -41,12 +51,12 @@ struct ASTWindowListElement : public IAST
     // ASTWindowDefinition
     ASTPtr definition;
 
-
     ASTPtr clone() const override;
 
     String getID(char delimiter) const override;
 
-    void formatImpl(const FormatSettings & settings, FormatState & state, FormatStateStacked frame) const override;
+protected:
+    void formatImpl(WriteBuffer & ostr, const FormatSettings & settings, FormatState & state, FormatStateStacked frame) const override;
 };
 
 }
