@@ -10,12 +10,12 @@ namespace DB
 {
 bool ParserShowGrantsQuery::parseImpl(Pos & pos, ASTPtr & node, Expected & expected)
 {
-    if (!ParserKeyword{"SHOW GRANTS"}.ignore(pos, expected))
+    if (!ParserKeyword{Keyword::SHOW_GRANTS}.ignore(pos, expected))
         return false;
 
     std::shared_ptr<ASTRolesOrUsersSet> for_roles;
 
-    if (ParserKeyword{"FOR"}.ignore(pos, expected))
+    if (ParserKeyword{Keyword::FOR}.ignore(pos, expected))
     {
         ASTPtr for_roles_ast;
         ParserRolesOrUsersSet for_roles_p;
@@ -31,8 +31,30 @@ bool ParserShowGrantsQuery::parseImpl(Pos & pos, ASTPtr & node, Expected & expec
         for_roles->current_user = true;
     }
 
+    bool with_implicit = false;
+    bool final = false;
+
+    while (true)
+    {
+        if (!with_implicit && ParserKeyword{Keyword::WITH_IMPLICIT}.ignore(pos, expected))
+        {
+            with_implicit = true;
+            continue;
+        }
+
+        if (!final && ParserKeyword{Keyword::FINAL}.ignore(pos, expected))
+        {
+            final = true;
+            continue;
+        }
+
+        break;
+    }
+
     auto query = std::make_shared<ASTShowGrantsQuery>();
     query->for_roles = std::move(for_roles);
+    query->with_implicit = with_implicit;
+    query->final = final;
     node = query;
 
     return true;
