@@ -1,6 +1,5 @@
 #pragma once
 
-#include <Parsers/IAST.h>
 #include <Interpreters/InDepthNodeVisitor.h>
 
 namespace DB
@@ -10,10 +9,13 @@ class ASTSelectQuery;
 struct SettingChange;
 class SettingsChanges;
 
+class IAST;
+using ASTPtr = std::shared_ptr<IAST>;
+
 /// Pushdown SETTINGS clause to the INSERT from the SELECT query:
 /// (since SETTINGS after SELECT will be parsed by the SELECT parser.)
 ///
-/// NOTE: INSERT ... SELECT ... FORMAT Null SETTINGS max_insert_threads=10 works even w/o push down,
+/// NOTE: INSERT ... SELECT ... FORMAT Null SETTINGS max_insert_threads=10 works even without push down,
 /// since ParserInsertQuery does not use ParserQueryWithOutput.
 class InsertQuerySettingsPushDownMatcher
 {
