@@ -12,16 +12,24 @@ struct ASTTableJoin;
   */
 class ParserTablesInSelectQuery : public IParserBase
 {
+public:
+    explicit ParserTablesInSelectQuery(bool allow_alias_without_as_keyword_ = true)
+        : allow_alias_without_as_keyword(allow_alias_without_as_keyword_) {}
+
 protected:
     const char * getName() const override { return "table, table function, subquery or list of joined tables"; }
     bool parseImpl(Pos & pos, ASTPtr & node, Expected & expected) override;
+
+private:
+    bool allow_alias_without_as_keyword;
 };
 
 
 class ParserTablesInSelectQueryElement : public IParserBase
 {
 public:
-    ParserTablesInSelectQueryElement(bool is_first_) : is_first(is_first_) {}
+    explicit ParserTablesInSelectQueryElement(bool is_first_, bool allow_alias_without_as_keyword_ = true)
+        : is_first(is_first_), allow_alias_without_as_keyword(allow_alias_without_as_keyword_) {}
 
 protected:
     const char * getName() const override { return "table, table function, subquery or list of joined tables"; }
@@ -29,16 +37,22 @@ protected:
 
 private:
     bool is_first;
-
-    static void parseJoinStrictness(Pos & pos, ASTTableJoin & table_join);
+    bool allow_alias_without_as_keyword;
 };
 
 
 class ParserTableExpression : public IParserBase
 {
+public:
+    explicit ParserTableExpression(bool allow_alias_without_as_keyword_ = true)
+        : allow_alias_without_as_keyword(allow_alias_without_as_keyword_) {}
+
 protected:
     const char * getName() const override { return "table or subquery or table function"; }
     bool parseImpl(Pos & pos, ASTPtr & node, Expected & expected) override;
+
+private:
+    bool allow_alias_without_as_keyword;
 };
 
 
