@@ -1,17 +1,22 @@
 #pragma once
+
+#include <Core/Block_fwd.h>
 #include <Processors/IProcessor.h>
 
+#include <vector>
+
 namespace DB
 {
+class Block;
 
 /// Processor with N inputs and N outputs. Only moves data from i-th input to i-th output as is.
 /// Some ports are delayed. Delayed ports are processed after other outputs are all finished.
 /// Data between ports is not mixed. It is important because this processor can be used before MergingSortedTransform.
 /// Delayed ports are appeared after joins, when some non-matched data need to be processed at the end.
-class DelayedPortsProcessor : public IProcessor
+class DelayedPortsProcessor final : public IProcessor
 {
 public:
-    DelayedPortsProcessor(const Block & header, size_t num_ports, const PortNumbers & delayed_ports, bool assert_main_ports_empty = false);
+    DelayedPortsProcessor(SharedHeader header, size_t num_ports, const PortNumbers & delayed_ports, bool assert_main_ports_empty = false);
 
     String getName() const override { return "DelayedPorts"; }
 
@@ -29,14 +34,16 @@ private:
 
     std::vector<PortsPair> port_pairs;
     const size_t num_delayed_ports;
-    size_t num_finished_pairs = 0;
+    size_t num_finished_inputs = 0;
     size_t num_finished_outputs = 0;
+    size_t num_finished_main_inputs = 0;
 
     std::vector<size_t> output_to_pair;
     bool are_inputs_initialized = false;
 
     bool processPair(PortsPair & pair);
     void finishPair(PortsPair & pair);
+    bool shouldSkipDelayed() const;
 };
 
 }
