@@ -6,6 +6,8 @@
 namespace DB
 {
 
+class Block;
+
 /** Has one input and arbitrary non zero number of outputs.
   * All of them have the same structure.
   *
@@ -15,13 +17,10 @@ namespace DB
   * Doesn't do any heavy calculations.
   * Preserves an order of data.
   */
-class ForkProcessor : public IProcessor
+class ForkProcessor final : public IProcessor
 {
 public:
-    ForkProcessor(const Block & header, size_t num_outputs)
-        : IProcessor(InputPorts{header}, OutputPorts(num_outputs, header))
-    {
-    }
+    ForkProcessor(const Block & header, size_t num_outputs);
 
     String getName() const override { return "Fork"; }
 
