@@ -1,21 +1,23 @@
 #pragma once
 
-#include "config_formats.h"
+#include "config.h"
 #if USE_CAPNP
 
-#include <Processors/Formats/IRowOutputFormat.h>
-#include <Formats/FormatSchemaInfo.h>
-#include <Formats/CapnProtoUtils.h>
-#include <capnp/schema.h>
-#include <capnp/dynamic.h>
-#include <kj/io.h>
+#    include <Formats/CapnProtoSchema.h>
+#    include <Formats/CapnProtoSerializer.h>
+#    include <Formats/FormatSchemaInfo.h>
+#    include <Processors/Formats/IRowOutputFormat.h>
+#    include <capnp/dynamic.h>
+#    include <capnp/schema.h>
+#    include <kj/io.h>
 
 namespace DB
 {
+
 class CapnProtoOutputStream : public kj::OutputStream
 {
 public:
-    CapnProtoOutputStream(WriteBuffer & out_);
+    explicit CapnProtoOutputStream(WriteBuffer & out_);
 
     void write(const void * buffer, size_t size) override;
 
@@ -28,10 +30,9 @@ class CapnProtoRowOutputFormat final : public IRowOutputFormat
 public:
     CapnProtoRowOutputFormat(
         WriteBuffer & out_,
-        const Block & header_,
-        const RowOutputFormatParams & params_,
-        const FormatSchemaInfo & info,
-        const FormatSettings & format_settings_);
+        SharedHeader header_,
+        const CapnProtoSchemaInfo & info,
+        const FormatSettings & format_settings);
 
     String getName() const override { return "CapnProtoRowOutputFormat"; }
 
@@ -44,8 +45,9 @@ private:
     DataTypes column_types;
     capnp::StructSchema schema;
     std::unique_ptr<CapnProtoOutputStream> output_stream;
-    const FormatSettings format_settings;
     CapnProtoSchemaParser schema_parser;
+    std::unique_ptr<CapnProtoSerializer> serializer;
+
 };
 
 }
