@@ -2,7 +2,6 @@
 
 #include <Processors/Formats/IRowInputFormat.h>
 #include <Processors/Formats/ISchemaReader.h>
-#include <Formats/FormatFactory.h>
 #include <DataTypes/DataTypeString.h>
 
 namespace DB
@@ -17,7 +16,7 @@ class ReadBuffer;
 class LineAsStringRowInputFormat final : public IRowInputFormat
 {
 public:
-    LineAsStringRowInputFormat(const Block & header_, ReadBuffer & in_, Params params_);
+    LineAsStringRowInputFormat(SharedHeader header_, ReadBuffer & in_, Params params_);
 
     String getName() const override { return "LineAsStringRowInputFormat"; }
     void resetParser() override;
@@ -26,6 +25,9 @@ private:
     bool readRow(MutableColumns & columns, RowReadExtension & ext) override;
 
     void readLineObject(IColumn & column);
+
+    size_t countRows(size_t max_block_size) override;
+    bool supportsCountRows() const override { return true; }
 };
 
 class LinaAsStringSchemaReader : public IExternalSchemaReader
