@@ -1,5 +1,6 @@
 #pragma once
 
+#include <Core/NamesAndTypes.h>
 #include <Formats/FormatSettings.h>
 #include <Processors/Formats/Impl/TabSeparatedRowOutputFormat.h>
 
@@ -14,7 +15,7 @@ namespace DB
 class TSKVRowOutputFormat final : public TabSeparatedRowOutputFormat
 {
 public:
-    TSKVRowOutputFormat(WriteBuffer & out_, const Block & header, const RowOutputFormatParams & params_, const FormatSettings & format_settings);
+    TSKVRowOutputFormat(WriteBuffer & out_, SharedHeader header, const FormatSettings & format_settings);
 
     String getName() const override { return "TSKVRowOutputFormat"; }
 
@@ -22,6 +23,10 @@ private:
     void writeField(const IColumn & column, const ISerialization & serialization, size_t row_num) override;
     void writeRowEndDelimiter() override;
 
+    /// Disable totals and extremes, because they are enabled in TSV.
+    bool supportTotals() const override { return false; }
+    bool supportExtremes() const override { return false; }
+
     NamesAndTypes fields;
     size_t field_number = 0;
 };
