@@ -1,11 +1,18 @@
+#include <IO/NullWriteBuffer.h>
 #include <Processors/Formats/LazyOutputFormat.h>
+#include <Processors/Port.h>
 #include <Processors/Transforms/AggregatingTransform.h>
 
 
 namespace DB
 {
 
-WriteBuffer LazyOutputFormat::out(nullptr, 0);
+NullWriteBuffer LazyOutputFormat::out;
+
+LazyOutputFormat::LazyOutputFormat(SharedHeader header)
+    : IOutputFormat(header, out), queue(2)
+{
+}
 
 Chunk LazyOutputFormat::getChunk(UInt64 milliseconds)
 {
@@ -45,4 +52,9 @@ void LazyOutputFormat::setRowsBeforeLimit(size_t rows_before_limit)
     info.setRowsBeforeLimit(rows_before_limit);
 }
 
+void LazyOutputFormat::setRowsBeforeAggregation(size_t rows_before_aggregation)
+{
+    info.setRowsBeforeAggregation(rows_before_aggregation);
+}
+
 }
