@@ -1,5 +1,6 @@
+#include <IO/NullWriteBuffer.h>
 #include <Processors/Formats/PullingOutputFormat.h>
-#include <IO/WriteBuffer.h>
+#include <Processors/Port.h>
 
 namespace DB
 {
@@ -9,13 +10,17 @@ namespace ErrorCodes
     extern const int LOGICAL_ERROR;
 }
 
-WriteBuffer PullingOutputFormat::out(nullptr, 0);
+NullWriteBuffer PullingOutputFormat::out;
+
+PullingOutputFormat::PullingOutputFormat(SharedHeader header, std::atomic_bool & consume_data_flag_)
+    : IOutputFormat(header, out)
+    , has_data_flag(consume_data_flag_)
+{}
 
 void PullingOutputFormat::consume(Chunk chunk)
 {
     if (data)
-        throw Exception("PullingOutputFormat cannot consume chunk because it already has data",
-                        ErrorCodes::LOGICAL_ERROR);
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "PullingOutputFormat cannot consume chunk because it already has data");
 
     if (chunk)
         info.update(chunk.getNumRows(), chunk.allocatedBytes());
@@ -38,5 +43,8 @@ void PullingOutputFormat::setRowsBeforeLimit(size_t rows_before_limit)
 {
     info.setRowsBeforeLimit(rows_before_limit);
 }
-
+void PullingOutputFormat::setRowsBeforeAggregation(size_t rows_before_aggregation)
+{
+    info.setRowsBeforeAggregation(rows_before_aggregation);
+}
 }
