@@ -1,5 +1,7 @@
 #pragma once
 
+#include <Core/Block_fwd.h>
+#include <Processors/Chunk.h>
 #include <Processors/IProcessor.h>
 
 
@@ -20,7 +22,7 @@ protected:
     virtual void onFinish() {}
 
 public:
-    explicit ISink(Block header);
+    explicit ISink(SharedHeader header);
 
     Status prepare() override;
     void work() override;
