@@ -12,14 +12,18 @@ class MergingSortedTransform final : public IMergingTransform<MergingSortedAlgor
 {
 public:
     MergingSortedTransform(
-        const Block & header,
+        SharedHeader header,
         size_t num_inputs,
-        SortDescription description,
-        size_t max_block_size,
+        const SortDescription & description,
+        size_t max_block_size_rows,
+        size_t max_block_size_bytes,
+        SortingQueueStrategy sorting_queue_strategy,
         UInt64 limit_ = 0,
+        bool always_read_till_end_ = false,
         WriteBuffer * out_row_sources_buf_ = nullptr,
-        bool quiet_ = false,
+        const std::optional<String> & filter_column_name_ = std::nullopt,
         bool use_average_block_sizes = false,
+        bool apply_virtual_row_conversions = true,
         bool have_all_inputs_ = true);
 
     String getName() const override { return "MergingSortedTransform"; }
@@ -27,9 +31,6 @@ public:
 protected:
     void onNewInput() override;
     void onFinish() override;
-
-private:
-    bool quiet = false;
 };
 
 }
