@@ -1,4 +1,6 @@
 #include <Processors/QueryPlan/ExtremesStep.h>
+#include <Processors/QueryPlan/Serialization.h>
+#include <Processors/QueryPlan/QueryPlanStepRegistry.h>
 #include <QueryPipeline/QueryPipelineBuilder.h>
 
 namespace DB
@@ -9,7 +11,6 @@ static ITransformingStep::Traits getTraits()
     return ITransformingStep::Traits
     {
         {
-            .preserves_distinct_columns = true,
             .returns_single_stream = false,
             .preserves_number_of_streams = true,
             .preserves_sorting = true,
@@ -20,8 +21,8 @@ static ITransformingStep::Traits getTraits()
     };
 }
 
-ExtremesStep::ExtremesStep(const DataStream & input_stream_)
-    : ITransformingStep(input_stream_, input_stream_.header, getTraits())
+ExtremesStep::ExtremesStep(const SharedHeader & input_header)
+    : ITransformingStep(input_header, input_header, getTraits())
 {
 }
 
@@ -30,4 +31,19 @@ void ExtremesStep::transformPipeline(QueryPipelineBuilder & pipeline, const Buil
     pipeline.addExtremesTransform();
 }
 
+void ExtremesStep::serialize(Serialization & ctx) const
+{
+    (void)ctx;
+}
+
+std::unique_ptr<IQueryPlanStep> ExtremesStep::deserialize(Deserialization & ctx)
+{
+    return std::make_unique<ExtremesStep>(ctx.input_headers.front());
+}
+
+void registerExtremesStep(QueryPlanStepRegistry & registry)
+{
+    registry.registerStep("Extremes", ExtremesStep::deserialize);
+}
+
 }
