@@ -1,35 +1,43 @@
 #pragma once
+
+#include <Interpreters/Context_fwd.h>
 #include <Processors/QueryPlan/ISourceStep.h>
 #include <QueryPipeline/Pipe.h>
+#include <Storages/SelectQueryInfo.h>
 
 namespace DB
 {
 
+class IStorage;
+using StoragePtr = std::shared_ptr<IStorage>;
+
 /// Create source from prepared pipe.
 class ReadFromPreparedSource : public ISourceStep
 {
 public:
-    explicit ReadFromPreparedSource(Pipe pipe_, ContextPtr context_ = nullptr);
+    explicit ReadFromPreparedSource(Pipe pipe_);
 
     String getName() const override { return "ReadFromPreparedSource"; }
-
     void initializePipeline(QueryPipelineBuilder & pipeline, const BuildQueryPipelineSettings &) override;
 
-private:
+protected:
     Pipe pipe;
-    ContextPtr context;
 };
 
-class ReadFromStorageStep : public ReadFromPreparedSource
+class ReadFromStorageStep final : public ReadFromPreparedSource
 {
 public:
-    ReadFromStorageStep(Pipe pipe_, String storage_name)
-        : ReadFromPreparedSource(std::move(pipe_))
-    {
-        setStepDescription(storage_name);
-    }
+    ReadFromStorageStep(Pipe pipe_, StoragePtr storage_, ContextPtr context_, const SelectQueryInfo & query_info_);
 
     String getName() const override { return "ReadFromStorage"; }
+
+    const StoragePtr & getStorage() const { return storage; }
+
+private:
+    StoragePtr storage;
+
+    ContextPtr context;
+    SelectQueryInfo query_info;
 };
 
 }
