@@ -6,17 +6,15 @@
 namespace DB
 {
 
-class ActionsDAG;
-using ActionsDAGPtr = std::shared_ptr<ActionsDAG>;
-
 class WindowTransform;
 
 class WindowStep : public ITransformingStep
 {
 public:
-    explicit WindowStep(const DataStream & input_stream_,
+    explicit WindowStep(const SharedHeader & input_header_,
             const WindowDescription & window_description_,
-            const std::vector<WindowFunctionDescription> & window_functions_);
+            const std::vector<WindowFunctionDescription> & window_functions_,
+            bool streams_fan_out_);
 
     String getName() const override { return "Window"; }
 
@@ -25,10 +23,14 @@ public:
     void describeActions(JSONBuilder::JSONMap & map) const override;
     void describeActions(FormatSettings & settings) const override;
 
+    const WindowDescription & getWindowDescription() const;
+
 private:
+    void updateOutputHeader() override;
+
     WindowDescription window_description;
     std::vector<WindowFunctionDescription> window_functions;
-    Block input_header;
+    bool streams_fan_out;
 };
 
 }
