@@ -1,17 +1,25 @@
 #include <Processors/Sources/ShellCommandSource.h>
 
-#include <sys/poll.h>
+#include <poll.h>
 
+#include <Common/CurrentThread.h>
 #include <Common/Stopwatch.h>
+#include <Common/logger_useful.h>
 
 #include <IO/WriteHelpers.h>
 #include <IO/ReadHelpers.h>
 
-#include <QueryPipeline/Pipe.h>
-#include <Processors/ISimpleTransform.h>
-#include <Processors/Formats/IOutputFormat.h>
-#include <Processors/Executors/CompletedPipelineExecutor.h>
 #include <Interpreters/Context.h>
+#include <Processors/Executors/CompletedPipelineExecutor.h>
+#include <Processors/Formats/IOutputFormat.h>
+#include <Processors/ISimpleTransform.h>
+#include <QueryPipeline/Pipe.h>
+#include <Core/Field.h>
+
+#include <boost/circular_buffer.hpp>
+#include <fmt/ranges.h>
+
+#include <ranges>
 
 
 namespace DB
@@ -21,10 +29,10 @@ namespace ErrorCodes
 {
     extern const int UNSUPPORTED_METHOD;
     extern const int TIMEOUT_EXCEEDED;
-    extern const int CANNOT_FCNTL;
     extern const int CANNOT_READ_FROM_FILE_DESCRIPTOR;
-    extern const int CANNOT_POLL;
     extern const int CANNOT_WRITE_TO_FILE_DESCRIPTOR;
+    extern const int CANNOT_FCNTL;
+    extern const int CANNOT_POLL;
 }
 
 static bool tryMakeFdNonBlocking(int fd)
@@ -42,7 +50,7 @@ static void makeFdNonBlocking(int fd)
 {
     bool result = tryMakeFdNonBlocking(fd);
     if (!result)
-        throwFromErrno("Cannot set non-blocking mode of pipe", ErrorCodes::CANNOT_FCNTL);
+        throw ErrnoException(ErrorCodes::CANNOT_FCNTL, "Cannot set non-blocking mode of pipe");
 }
 
 static bool tryMakeFdBlocking(int fd)
@@ -61,7 +69,50 @@ static void makeFdBlocking(int fd)
 {
     bool result = tryMakeFdBlocking(fd);
     if (!result)
-        throwFromErrno("Cannot set blocking mode of pipe", ErrorCodes::CANNOT_FCNTL);
+        throw ErrnoException(ErrorCodes::CANNOT_FCNTL, "Cannot set blocking mode of pipe");
+}
+
+static int pollWithTimeout(pollfd * pfds, size_t num, size_t timeout_milliseconds)
+{
+    auto logger = getLogger("TimeoutReadBufferFromFileDescriptor");
+    auto describe_fd = [](const auto & pollfd) { return fmt::format("(fd={}, flags={})", pollfd.fd, fcntl(pollfd.fd, F_GETFL)); };
+
+    int res;
+
+    while (true)
+    {
+        Stopwatch watch;
+
+        LOG_TEST(logger, "Polling descriptors: {}", fmt::join(std::span(pfds, pfds + num) | std::views::transform(describe_fd), ", "));
+
+        res = poll(pfds, static_cast<nfds_t>(num), static_cast<int>(timeout_milliseconds));
+
+        if (res < 0)
+        {
+            if (errno != EINTR)
+                throw ErrnoException(ErrorCodes::CANNOT_POLL, "Cannot poll");
+
+            const auto elapsed = watch.elapsedMilliseconds();
+            if (timeout_milliseconds <= elapsed)
+            {
+                LOG_TEST(logger, "Timeout exceeded: elapsed={}, timeout={}", elapsed, timeout_milliseconds);
+                break;
+            }
+            timeout_milliseconds -= elapsed;
+        }
+        else
+        {
+            break;
+        }
+    }
+
+    LOG_TEST(
+        logger,
+        "Poll for descriptors: {} returned {}",
+        fmt::join(std::span(pfds, pfds + num) | std::views::transform(describe_fd), ", "),
+        res);
+
+    return res;
 }
 
 static bool pollFd(int fd, size_t timeout_milliseconds, int events)
@@ -71,46 +122,34 @@ static bool pollFd(int fd, size_t timeout_milliseconds, int events)
     pfd.events = events;
     pfd.revents = 0;
 
-    Stopwatch watch;
-
-    int res;
-
-    while (true)
-    {
-        res = poll(&pfd, 1, timeout_milliseconds);
-
-        if (res < 0)
-        {
-            if (errno == EINTR)
-            {
-                watch.stop();
-                timeout_milliseconds -= watch.elapsedMilliseconds();
-                watch.start();
-
-                continue;
-            }
-            else
-            {
-                throwFromErrno("Cannot poll", ErrorCodes::CANNOT_POLL);
-            }
-        }
-        else
-        {
-            break;
-        }
-    }
-
-    return res > 0;
+    return pollWithTimeout(&pfd, 1, timeout_milliseconds) > 0;
 }
 
 class TimeoutReadBufferFromFileDescriptor : public BufferWithOwnMemory<ReadBuffer>
 {
 public:
-    explicit TimeoutReadBufferFromFileDescriptor(int fd_, size_t timeout_milliseconds_)
-        : fd(fd_)
+    explicit TimeoutReadBufferFromFileDescriptor(
+        int stdout_fd_,
+        int stderr_fd_,
+        size_t timeout_milliseconds_,
+        ExternalCommandStderrReaction stderr_reaction_)
+        : stdout_fd(stdout_fd_)
+        , stderr_fd(stderr_fd_)
         , timeout_milliseconds(timeout_milliseconds_)
+        , stderr_reaction(stderr_reaction_)
     {
-        makeFdNonBlocking(fd);
+        makeFdNonBlocking(stdout_fd);
+        makeFdNonBlocking(stderr_fd);
+
+        pfds[0].fd = stdout_fd;
+        pfds[0].events = POLLIN;
+        pfds[1].fd = stderr_fd;
+        pfds[1].events = POLLIN;
+
+        if (stderr_reaction == ExternalCommandStderrReaction::NONE)
+            num_pfds = 1;
+        else
+            num_pfds = 2;
     }
 
     bool nextImpl() override
@@ -119,19 +158,53 @@ public:
 
         while (!bytes_read)
         {
-            if (!pollFd(fd, timeout_milliseconds, POLLIN))
+            pfds[0].revents = 0;
+            pfds[1].revents = 0;
+            int num_events = pollWithTimeout(pfds, num_pfds, timeout_milliseconds);
+            if (num_events <= 0)
                 throw Exception(ErrorCodes::TIMEOUT_EXCEEDED, "Pipe read timeout exceeded {} milliseconds", timeout_milliseconds);
 
-            ssize_t res = ::read(fd, internal_buffer.begin(), internal_buffer.size());
+            bool has_stdout = pfds[0].revents > 0;
+            bool has_stderr = pfds[1].revents > 0;
 
-            if (-1 == res && errno != EINTR)
-                throwFromErrno("Cannot read from pipe ", ErrorCodes::CANNOT_READ_FROM_FILE_DESCRIPTOR);
+            if (has_stderr)
+            {
+                if (stderr_read_buf == nullptr)
+                    stderr_read_buf.reset(new char[BUFFER_SIZE]);
+                ssize_t res = ::read(stderr_fd, stderr_read_buf.get(), BUFFER_SIZE);
+                if (res > 0)
+                {
+                    std::string_view str(stderr_read_buf.get(), res);
+                    if (stderr_reaction == ExternalCommandStderrReaction::THROW)
+                        throw Exception(ErrorCodes::UNSUPPORTED_METHOD, "Executable generates stderr: {}", str);
+                    if (stderr_reaction == ExternalCommandStderrReaction::LOG)
+                        LOG_WARNING(getLogger("TimeoutReadBufferFromFileDescriptor"), "Executable generates stderr: {}", str);
+                    else if (stderr_reaction == ExternalCommandStderrReaction::LOG_FIRST)
+                    {
+                        res = std::min(ssize_t(stderr_result_buf.reserve()), res);
+                        if (res > 0)
+                            stderr_result_buf.insert(stderr_result_buf.end(), str.begin(), str.begin() + res);
+                    }
+                    else if (stderr_reaction == ExternalCommandStderrReaction::LOG_LAST)
+                    {
+                        stderr_result_buf.insert(stderr_result_buf.end(), str.begin(), str.begin() + res);
+                    }
+                }
+            }
 
-            if (res == 0)
-                break;
+            if (has_stdout)
+            {
+                ssize_t res = ::read(stdout_fd, internal_buffer.begin(), internal_buffer.size());
 
-            if (res > 0)
-                bytes_read += res;
+                if (-1 == res && errno != EINTR)
+                    throw ErrnoException(ErrorCodes::CANNOT_READ_FROM_FILE_DESCRIPTOR, "Cannot read from pipe");
+
+                if (res == 0)
+                    break;
+
+                if (res > 0)
+                    bytes_read += res;
+            }
         }
 
         if (bytes_read > 0)
@@ -147,27 +220,42 @@ public:
         return true;
     }
 
-    void reset() const
-    {
-        makeFdBlocking(fd);
-    }
-
     ~TimeoutReadBufferFromFileDescriptor() override
     {
-        tryMakeFdBlocking(fd);
+        tryMakeFdBlocking(stdout_fd);
+        tryMakeFdBlocking(stderr_fd);
+
+        if (!stderr_result_buf.empty())
+        {
+            String stderr_result;
+            stderr_result.reserve(stderr_result_buf.size());
+            stderr_result.append(stderr_result_buf.begin(), stderr_result_buf.end());
+            LOG_WARNING(
+                getLogger("ShellCommandSource"),
+                "Executable generates stderr at the {}: {}",
+                stderr_reaction == ExternalCommandStderrReaction::LOG_FIRST ? "beginning" : "end",
+                stderr_result);
+        }
     }
 
 private:
-    int fd;
+    int stdout_fd;
+    int stderr_fd;
     size_t timeout_milliseconds;
+    ExternalCommandStderrReaction stderr_reaction;
+
+    static constexpr size_t BUFFER_SIZE = 4_KiB;
+    pollfd pfds[2];
+    size_t num_pfds;
+    std::unique_ptr<char[]> stderr_read_buf;
+    boost::circular_buffer_space_optimized<char> stderr_result_buf{BUFFER_SIZE};
 };
 
 class TimeoutWriteBufferFromFileDescriptor : public BufferWithOwnMemory<WriteBuffer>
 {
 public:
     explicit TimeoutWriteBufferFromFileDescriptor(int fd_, size_t timeout_milliseconds_)
-        : fd(fd_)
-        , timeout_milliseconds(timeout_milliseconds_)
+        : fd(fd_), timeout_milliseconds(timeout_milliseconds_)
     {
         makeFdNonBlocking(fd);
     }
@@ -187,7 +275,7 @@ public:
             ssize_t res = ::write(fd, working_buffer.begin() + bytes_written, offset() - bytes_written);
 
             if ((-1 == res || 0 == res) && errno != EINTR)
-                throwFromErrno("Cannot write into pipe ", ErrorCodes::CANNOT_WRITE_TO_FILE_DESCRIPTOR);
+                throw ErrnoException(ErrorCodes::CANNOT_WRITE_TO_FILE_DESCRIPTOR, "Cannot write into pipe");
 
             if (res > 0)
                 bytes_written += res;
@@ -244,7 +332,7 @@ namespace
     *
     * If process_pool is passed in constructor then after source is destroyed process is returned to pool.
     */
-    class ShellCommandSource final : public SourceWithProgress
+    class ShellCommandSource final : public ISource
     {
     public:
 
@@ -254,63 +342,97 @@ namespace
             ContextPtr context_,
             const std::string & format_,
             size_t command_read_timeout_milliseconds,
-            const Block & sample_block_,
+            ExternalCommandStderrReaction stderr_reaction,
+            bool check_exit_code_,
+            SharedHeader sample_block_,
             std::unique_ptr<ShellCommand> && command_,
             std::vector<SendDataTask> && send_data_tasks = {},
             const ShellCommandSourceConfiguration & configuration_ = {},
             std::unique_ptr<ShellCommandHolder> && command_holder_ = nullptr,
             std::shared_ptr<ProcessPool> process_pool_ = nullptr)
-            : SourceWithProgress(sample_block_)
+            : ISource(sample_block_)
             , context(context_)
             , format(format_)
             , sample_block(sample_block_)
             , command(std::move(command_))
             , configuration(configuration_)
-            , timeout_command_out(command->out.getFD(), command_read_timeout_milliseconds)
+            , timeout_command_out(command->out.getFD(), command->err.getFD(), command_read_timeout_milliseconds, stderr_reaction)
             , command_holder(std::move(command_holder_))
             , process_pool(process_pool_)
+            , check_exit_code(check_exit_code_)
         {
-            for (auto && send_data_task : send_data_tasks)
-            {
-                send_data_threads.emplace_back([task = std::move(send_data_task), this]()
-                {
-                    try
-                    {
-                        task();
-                    }
-                    catch (...)
-                    {
-                        std::lock_guard<std::mutex> lock(send_data_lock);
-                        exception_during_send_data = std::current_exception();
-                    }
-                });
-            }
-
-            size_t max_block_size = configuration.max_block_size;
-
+            auto context_for_reading = Context::createCopy(context);
+            /// Currently parallel parsing input format cannot read exactly max_block_size rows from input,
+            /// so it will be blocked on ReadBufferFromFileDescriptor because this file descriptor represent pipe that does not have eof.
             if (configuration.read_fixed_number_of_rows)
-            {
-                /** Currently parallel parsing input format cannot read exactly max_block_size rows from input,
-                  * so it will be blocked on ReadBufferFromFileDescriptor because this file descriptor represent pipe that does not have eof.
-                  */
-                auto context_for_reading = Context::createCopy(context);
                 context_for_reading->setSetting("input_format_parallel_parsing", false);
-                context = context_for_reading;
+            /// Here header auto detection can only cause troubles, since if it
+            /// will find "header" the number of input and output rows will not
+            /// match.
+            context_for_reading->setSetting("input_format_csv_detect_header", false);
+            context_for_reading->setSetting("input_format_tsv_detect_header", false);
+            context_for_reading->setSetting("input_format_custom_detect_header", false);
+            context = context_for_reading;
 
-                if (configuration.read_number_of_rows_from_process_output)
+            auto thread_group = CurrentThread::getGroup();
+
+            try
+            {
+                for (auto && send_data_task : send_data_tasks)
                 {
-                    /// Initialize executor in generate
-                    return;
+                    send_data_threads.emplace_back([thread_group, task = std::move(send_data_task), this]() mutable
+                    {
+                        ThreadGroupSwitcher switcher(thread_group, "SendToShellCmd");
+
+                        try
+                        {
+                            task();
+                        }
+                        catch (...)
+                        {
+                            std::lock_guard lock(send_data_lock);
+                            exception_during_send_data = std::current_exception();
+                        }
+
+                        // In case of exception, the task should be reset in thread
+                        // worker function or else it breaks d'tor invariants such
+                        // as in ~WriteBuffer.
+                        //
+                        // For completed execution, the task reset allows to account
+                        // memory deallocation in sending data thread group.
+                        task = {};
+                    });
+                }
+                size_t max_block_size = configuration.max_block_size;
+
+                if (configuration.read_fixed_number_of_rows)
+                {
+                    if (configuration.read_number_of_rows_from_process_output)
+                    {
+                        /// Initialize executor in generate
+                        return;
+                    }
+
+                    max_block_size = configuration.number_of_rows_to_read;
                 }
 
-                max_block_size = configuration.number_of_rows_to_read;
+                pipeline = QueryPipeline(Pipe(context->getInputFormat(format, timeout_command_out, *sample_block, max_block_size)));
+                executor = std::make_unique<PullingPipelineExecutor>(pipeline);
+            }
+            catch (...)
+            {
+                cleanup();
+                throw;
             }
-
-            pipeline = QueryPipeline(Pipe(context->getInputFormat(format, timeout_command_out, sample_block, max_block_size)));
-            executor = std::make_unique<PullingPipelineExecutor>(pipeline);
         }
 
         ~ShellCommandSource() override
+        {
+            cleanup();
+        }
+
+    protected:
+        void cleanup()
         {
             for (auto & thread : send_data_threads)
                 if (thread.joinable())
@@ -330,8 +452,6 @@ namespace
             }
         }
 
-    protected:
-
         Chunk generate() override
         {
             rethrowExceptionDuringSendDataIfNeeded();
@@ -349,7 +469,7 @@ namespace
                         readChar(dummy, timeout_command_out);
 
                         size_t max_block_size = configuration.number_of_rows_to_read;
-                        pipeline = QueryPipeline(Pipe(context->getInputFormat(format, timeout_command_out, sample_block, max_block_size)));
+                        pipeline = QueryPipeline(Pipe(context->getInputFormat(format, timeout_command_out, *sample_block, max_block_size)));
                         executor = std::make_unique<PullingPipelineExecutor>(pipeline);
                     }
 
@@ -373,7 +493,7 @@ namespace
 
         Status prepare() override
         {
-            auto status = SourceWithProgress::prepare();
+            auto status = ISource::prepare();
 
             if (status == Status::Finished)
             {
@@ -381,6 +501,21 @@ namespace
                     if (thread.joinable())
                         thread.join();
 
+                if (check_exit_code)
+                {
+                    if (process_pool)
+                    {
+                        bool valid_command
+                            = configuration.read_fixed_number_of_rows && current_read_rows >= configuration.number_of_rows_to_read;
+
+                        // We can only wait for pooled commands when they are invalid.
+                        if (!valid_command)
+                            command->wait();
+                    }
+                    else
+                        command->wait();
+                }
+
                 rethrowExceptionDuringSendDataIfNeeded();
             }
 
@@ -393,7 +528,7 @@ namespace
 
         void rethrowExceptionDuringSendDataIfNeeded()
         {
-            std::lock_guard<std::mutex> lock(send_data_lock);
+            std::lock_guard lock(send_data_lock);
             if (exception_during_send_data)
             {
                 command_is_invalid = true;
@@ -403,7 +538,7 @@ namespace
 
         ContextPtr context;
         std::string format;
-        Block sample_block;
+        SharedHeader sample_block;
 
         std::unique_ptr<ShellCommand> command;
         ShellCommandSourceConfiguration configuration;
@@ -415,6 +550,8 @@ namespace
         ShellCommandHolderPtr command_holder;
         std::shared_ptr<ProcessPool> process_pool;
 
+        bool check_exit_code = false;
+
         QueryPipeline pipeline;
         std::unique_ptr<PullingPipelineExecutor> executor;
 
@@ -429,7 +566,7 @@ namespace
     class SendingChunkHeaderTransform final : public ISimpleTransform
     {
     public:
-        SendingChunkHeaderTransform(const Block & header, std::shared_ptr<TimeoutWriteBufferFromFileDescriptor> buffer_)
+        SendingChunkHeaderTransform(SharedHeader header, WriteBuffer & buffer_)
             : ISimpleTransform(header, header, false)
             , buffer(buffer_)
         {
@@ -441,12 +578,12 @@ namespace
 
         void transform(Chunk & chunk) override
         {
-            writeText(chunk.getNumRows(), *buffer);
-            writeChar('\n', *buffer);
+            writeText(chunk.getNumRows(), buffer);
+            writeChar('\n', buffer);
         }
 
     private:
-        std::shared_ptr<TimeoutWriteBufferFromFileDescriptor> buffer;
+        WriteBuffer & buffer;
     };
 
 }
@@ -474,7 +611,7 @@ Pipe ShellCommandSourceCoordinator::createPipe(
     std::unique_ptr<ShellCommand> process;
     std::unique_ptr<ShellCommandHolder> process_holder;
 
-    auto destructor_strategy = ShellCommand::DestructorStrategy{true /*terminate_in_destructor*/, configuration.command_termination_timeout_seconds};
+    auto destructor_strategy = ShellCommand::DestructorStrategy{true /*terminate_in_destructor*/, SIGTERM, configuration.command_termination_timeout_seconds};
     command_config.terminate_in_destructor_strategy = destructor_strategy;
 
     bool is_executable_pool = (process_pool != nullptr);
@@ -490,8 +627,7 @@ Pipe ShellCommandSourceCoordinator::createPipe(
                 {
                     if (execute_direct)
                         return ShellCommand::executeDirect(command_config);
-                    else
-                        return ShellCommand::execute(command_config);
+                    return ShellCommand::execute(command_config);
                 };
 
                 return std::make_unique<ShellCommandHolder>(std::move(func));
@@ -527,7 +663,7 @@ Pipe ShellCommandSourceCoordinator::createPipe(
         }
         else
         {
-            auto descriptor = i + 2;
+            int descriptor = static_cast<int>(i) + 2;
             auto it = process->write_fds.find(descriptor);
             if (it == process->write_fds.end())
                 throw Exception(ErrorCodes::UNSUPPORTED_METHOD, "Process does not contain descriptor to write {}", descriptor);
@@ -536,19 +672,24 @@ Pipe ShellCommandSourceCoordinator::createPipe(
         }
 
         int write_buffer_fd = write_buffer->getFD();
-        auto timeout_write_buffer = std::make_shared<TimeoutWriteBufferFromFileDescriptor>(write_buffer_fd, configuration.command_write_timeout_milliseconds);
+        auto timeout_write_buffer
+            = std::make_shared<TimeoutWriteBufferFromFileDescriptor>(write_buffer_fd, configuration.command_write_timeout_milliseconds);
 
         input_pipes[i].resize(1);
 
+        auto out = context->getOutputFormat(configuration.format, *timeout_write_buffer, materializeBlock(input_pipes[i].getHeader()));
+        out->setAutoFlush();
+
         if (configuration.send_chunk_header)
         {
-            auto transform = std::make_shared<SendingChunkHeaderTransform>(input_pipes[i].getHeader(), timeout_write_buffer);
+            /// We cannot use timeout_write_buffer directly since the output format may wrap the buffer, so we need to use a wrapper
+            auto transform = std::make_shared<SendingChunkHeaderTransform>(input_pipes[i].getSharedHeader(), *out->getWriteBufferPtr());
             input_pipes[i].addTransform(std::move(transform));
         }
 
+        auto num_streams = input_pipes[i].maxParallelStreams();
         auto pipeline = std::make_shared<QueryPipeline>(std::move(input_pipes[i]));
-        auto out = context->getOutputFormat(configuration.format, *timeout_write_buffer, materializeBlock(pipeline->getHeader()));
-        out->setAutoFlush();
+        pipeline->setNumThreads(num_streams);
         pipeline->complete(std::move(out));
 
         ShellCommandSource::SendDataTask task = [pipeline, timeout_write_buffer, write_buffer, is_executable_pool]()
@@ -556,11 +697,11 @@ Pipe ShellCommandSourceCoordinator::createPipe(
             CompletedPipelineExecutor executor(*pipeline);
             executor.execute();
 
+            timeout_write_buffer->finalize();
+            timeout_write_buffer->reset();
+
             if (!is_executable_pool)
             {
-                timeout_write_buffer->next();
-                timeout_write_buffer->reset();
-
                 write_buffer->close();
             }
         };
@@ -572,15 +713,16 @@ Pipe ShellCommandSourceCoordinator::createPipe(
         context,
         configuration.format,
         configuration.command_read_timeout_milliseconds,
-        std::move(sample_block),
+        configuration.stderr_reaction,
+        configuration.check_exit_code,
+        std::make_shared<const Block>(std::move(sample_block)),
         std::move(process),
         std::move(tasks),
         source_configuration,
         std::move(process_holder),
         process_pool);
-    auto pipe = Pipe(std::move(source));
 
-    return pipe;
+    return Pipe(std::move(source));
 }
 
 }
