@@ -5,7 +5,7 @@
 namespace DB
 {
 
-enum class TTLUpdateField
+enum class TTLUpdateField : uint8_t
 {
     COLUMNS_TTL,
     TABLE_TTL,
@@ -20,9 +20,10 @@ class TTLUpdateInfoAlgorithm : public ITTLAlgorithm
 {
 public:
     TTLUpdateInfoAlgorithm(
+        const TTLExpressions & ttl_expressions_,
         const TTLDescription & description_,
-        const TTLUpdateField ttl_update_field_,
-        const String ttl_update_key_,
+        TTLUpdateField ttl_update_field_,
+        String ttl_update_key_,
         const TTLInfo & old_ttl_info_,
         time_t current_time_, bool force_
     );
