@@ -1,32 +1,28 @@
 #pragma once
 #include <Processors/ISimpleTransform.h>
 #include <Core/SortDescription.h>
-#include <Columns/IColumn.h>
+#include <Columns/IColumn_fwd.h>
 
 namespace DB
 {
-using SortDescriptionsWithPositions = std::vector<SortColumnDescription>;
-
 /// Streams checks that flow of blocks is sorted in the sort_description order
 /// Othrewise throws exception in readImpl function.
 class CheckSortedTransform : public ISimpleTransform
 {
 public:
-    CheckSortedTransform(
-        const Block & header_,
-        const SortDescription & sort_description_);
+    CheckSortedTransform(SharedHeader header, const SortDescription & sort_description);
 
     String getName() const override { return "CheckSortedTransform"; }
-
+    void setDescription(const String & str) { description = str; }
 
 protected:
     void transform(Chunk & chunk) override;
 
 private:
-    SortDescriptionsWithPositions sort_description_map;
+    SortDescriptionWithPositions sort_description_map;
     Columns last_row;
-
-    /// Just checks, that all sort_descriptions has column_number
-    SortDescriptionsWithPositions addPositionsToSortDescriptions(const SortDescription & sort_description);
+    String description;
+    size_t chunk_num = 0;
+    size_t rows_read = 0;
 };
 }
