@@ -11,7 +11,7 @@ class DistinctTransform : public ISimpleTransform
 {
 public:
     DistinctTransform(
-        const Block & header_,
+        SharedHeader header_,
         const SizeLimits & set_size_limits_,
         UInt64 limit_hint_,
         const Names & columns_);
@@ -25,9 +25,7 @@ private:
     ColumnNumbers key_columns_pos;
     SetVariants data;
     Sizes key_sizes;
-    UInt64 limit_hint;
-
-    bool no_more_rows = false;
+    const UInt64 limit_hint;
 
     /// Restrictions on the maximum size of the output data.
     SizeLimits set_size_limits;
