@@ -11,9 +11,13 @@ class FinishSortingTransform : public SortingTransform
 {
 public:
     /// limit - if not 0, allowed to return just first 'limit' rows in sorted order.
-    FinishSortingTransform(const Block & header, const SortDescription & description_sorted_,
+    FinishSortingTransform(
+        SharedHeader header,
+        const SortDescription & description_sorted_,
         const SortDescription & description_to_sort_,
-        size_t max_merged_block_size_, UInt64 limit_);
+        size_t max_merged_block_size_,
+        UInt64 limit_,
+        bool increase_sort_description_compile_attempts);
 
     String getName() const override { return "FinishSortingTransform"; }
 
@@ -22,7 +26,7 @@ protected:
     void generate() override;
 
 private:
-    SortDescription description_sorted;
+    SortDescriptionWithPositions description_with_positions;
 
     Chunk tail_chunk;
 };
