@@ -1,5 +1,6 @@
 #pragma once
 #include <Processors/ISimpleTransform.h>
+#include <Processors/RowsBeforeStepCounter.h>
 #include <Common/HashTable/HashMap.h>
 
 
@@ -10,10 +11,12 @@ namespace DB
 class LimitByTransform : public ISimpleTransform
 {
 public:
-    LimitByTransform(const Block & header, UInt64 group_length_, UInt64 group_offset_, const Names & columns);
+    LimitByTransform(SharedHeader header, UInt64 group_length_, UInt64 group_offset_, const Names & columns);
 
     String getName() const override { return "LimitByTransform"; }
 
+    void setRowsBeforeLimitCounter(RowsBeforeStepCounterPtr counter) override { rows_before_limit_at_least.swap(counter); }
+
 protected:
     void transform(Chunk & chunk) override;
 
@@ -24,6 +27,8 @@ private:
     std::vector<size_t> key_positions;
     const UInt64 group_length;
     const UInt64 group_offset;
+
+    RowsBeforeStepCounterPtr rows_before_limit_at_least;
 };
 
 }
