@@ -8,7 +8,7 @@ namespace DB
 class ReverseTransform : public ISimpleTransform
 {
 public:
-    explicit ReverseTransform(const Block & header) : ISimpleTransform(header, header, false) {}
+    explicit ReverseTransform(SharedHeader header) : ISimpleTransform(header, header, false) {}
     String getName() const override { return "ReverseTransform"; }
 
 protected:
