@@ -1,21 +1,23 @@
 #pragma once
-#include <Processors/IAccumulatingTransform.h>
-#include <Storages/MergeTree/MergeTreeData.h>
-#include <Storages/MergeTree/IMergeTreeDataPart.h>
-#include <Core/Block.h>
-#include <Storages/MergeTree/MergeTreeDataPartTTLInfo.h>
-#include <Processors/TTL/ITTLAlgorithm.h>
 
-#include <Common/DateLUT.h>
+#include <Interpreters/PreparedSets.h>
+#include <Processors/IAccumulatingTransform.h>
+#include <Processors/TTL/ITTLAlgorithm.h>
+#include <Storages/MergeTree/IMergeTreeDataPart.h>
+#include <Storages/MergeTree/MergeTreeData.h>
+#include <Storages/MergeTree/MergeTreeDataPartTTLInfo.h>
 
 namespace DB
 {
 
+class Block;
+
 class TTLCalcTransform : public IAccumulatingTransform
 {
 public:
     TTLCalcTransform(
-        const Block & header_,
+        const ContextPtr & context,
+        SharedHeader header_,
         const MergeTreeData & storage_,
         const StorageMetadataPtr & metadata_snapshot_,
         const MergeTreeData::MutableDataPartPtr & data_part_,
@@ -23,6 +25,8 @@ public:
         bool force_
     );
 
+    PreparedSets::Subqueries getSubqueries() { return std::move(subqueries_for_sets); }
+
     String getName() const override { return "TTL_CALC"; }
     Status prepare() override;
 
@@ -35,10 +39,11 @@ protected:
 
 private:
     std::vector<TTLAlgorithmPtr> algorithms;
+    PreparedSets::Subqueries subqueries_for_sets;
 
     /// ttl_infos and empty_columns are updating while reading
     const MergeTreeData::MutableDataPartPtr & data_part;
-    Poco::Logger * log;
+    LoggerPtr log;
 };
 
 }
