@@ -26,24 +26,21 @@ class InterserverIOHTTPHandler : public HTTPRequestHandler
 public:
     explicit InterserverIOHTTPHandler(IServer & server_)
         : server(server_)
-        , log(&Poco::Logger::get("InterserverIOHTTPHandler"))
+        , log(getLogger("InterserverIOHTTPHandler"))
     {
     }
 
-    void handleRequest(HTTPServerRequest & request, HTTPServerResponse & response) override;
+    void handleRequest(HTTPServerRequest & request, HTTPServerResponse & response, const ProfileEvents::Event & write_event) override;
 
 private:
-    struct Output
-    {
-        std::shared_ptr<WriteBufferFromHTTPServerResponse> out;
-    };
+    using OutputPtr = std::shared_ptr<WriteBufferFromHTTPServerResponse>;
 
     IServer & server;
-    Poco::Logger * log;
+    LoggerPtr log;
 
     CurrentMetrics::Increment metric_increment{CurrentMetrics::InterserverConnection};
 
-    void processQuery(HTTPServerRequest & request, HTTPServerResponse & response, Output & used_output);
+    void processQuery(HTTPServerRequest & request, HTTPServerResponse & response, OutputPtr used_output);
 
     std::pair<String, bool> checkAuthentication(HTTPServerRequest & request) const;
 };
