@@ -2,36 +2,56 @@
 
 #if USE_NURAFT
 
-#include <Common/ZooKeeper/ZooKeeperIO.h>
-#include <Core/Types.h>
-#include <IO/WriteBufferFromPocoSocket.h>
-#include <IO/ReadBufferFromPocoSocket.h>
-#include <Poco/Net/NetException.h>
-#include <Common/CurrentThread.h>
-#include <Common/Stopwatch.h>
-#include <Common/NetException.h>
-#include <Common/setThreadName.h>
-#include <base/logger_useful.h>
-#include <chrono>
-#include <Common/PipeFDs.h>
-#include <Poco/Util/AbstractConfiguration.h>
-#include <IO/ReadBufferFromFileDescriptor.h>
-#include <queue>
-#include <mutex>
-#include <Coordination/FourLetterCommand.h>
-#include <Common/hex.h>
+#    include <mutex>
+#    include <Coordination/CoordinationSettings.h>
+#    include <Coordination/FourLetterCommand.h>
+#    include <Core/Types.h>
+#    include <IO/CompressionMethod.h>
+#    include <IO/ReadBufferFromFileDescriptor.h>
+#    include <IO/ReadBufferFromPocoSocket.h>
+#    include <IO/WriteBufferFromPocoSocket.h>
+#    include <IO/LimitReadBuffer.h>
+#    include <base/defines.h>
+#    include <base/hex.h>
+#    include <Poco/Net/NetException.h>
+#    include <Poco/Util/AbstractConfiguration.h>
+#    include <Common/CurrentThread.h>
+#    include <Common/NetException.h>
+#    include <Common/PipeFDs.h>
+#    include <Common/Stopwatch.h>
+#    include <Common/ZooKeeper/ZooKeeperCommon.h>
+#    include <Common/ZooKeeper/ZooKeeperConstants.h>
+#    include <Common/ZooKeeper/ZooKeeperIO.h>
+#    include <Common/logger_useful.h>
+#    include <Common/setThreadName.h>
+#    include <Compression/CompressionFactory.h>
+
+#    include <boost/algorithm/string/trim.hpp>
 
 
-#ifdef POCO_HAVE_FD_EPOLL
-    #include <sys/epoll.h>
-#else
-    #include <poll.h>
-#endif
+#    ifdef POCO_HAVE_FD_EPOLL
+#        include <sys/epoll.h>
+#    else
+#        include <poll.h>
+#    endif
+
+namespace ProfileEvents
+{
+    extern const Event KeeperTotalElapsedMicroseconds;
+}
 
 
 namespace DB
 {
 
+namespace CoordinationSetting
+{
+    extern const CoordinationSettingsUInt64 log_slow_connection_operation_threshold_ms;
+    extern const CoordinationSettingsUInt64 log_slow_total_threshold_ms;
+    extern const CoordinationSettingsUInt64 max_request_size;
+    extern const CoordinationSettingsBool use_xid_64;
+}
+
 struct LastOp
 {
 public:
@@ -49,6 +69,9 @@ namespace ErrorCodes
     extern const int LOGICAL_ERROR;
     extern const int UNEXPECTED_PACKET_FROM_CLIENT;
     extern const int TIMEOUT_EXCEEDED;
+    extern const int BAD_ARGUMENTS;
+    extern const int LIMIT_EXCEEDED;
+    extern const int AUTHENTICATION_FAILED;
 }
 
 struct PollResult
@@ -81,21 +104,25 @@ struct SocketInterruptablePollWrapper
 #if defined(POCO_HAVE_FD_EPOLL)
         epollfd = epoll_create(2);
         if (epollfd < 0)
-            throwFromErrno("Cannot epoll_create", ErrorCodes::SYSTEM_ERROR);
+            throw ErrnoException(ErrorCodes::SYSTEM_ERROR, "Cannot epoll_create");
 
         socket_event.events = EPOLLIN | EPOLLERR | EPOLLPRI;
         socket_event.data.fd = sockfd;
         if (epoll_ctl(epollfd, EPOLL_CTL_ADD, sockfd, &socket_event) < 0)
         {
-            ::close(epollfd);
-            throwFromErrno("Cannot insert socket into epoll queue", ErrorCodes::SYSTEM_ERROR);
+            [[maybe_unused]] int err = ::close(epollfd);
+            chassert(!err || errno == EINTR);
+
+            throw ErrnoException(ErrorCodes::SYSTEM_ERROR, "Cannot insert socket into epoll queue");
         }
         pipe_event.events = EPOLLIN | EPOLLERR | EPOLLPRI;
         pipe_event.data.fd = pipe.fds_rw[0];
         if (epoll_ctl(epollfd, EPOLL_CTL_ADD, pipe.fds_rw[0], &pipe_event) < 0)
         {
-            ::close(epollfd);
-            throwFromErrno("Cannot insert socket into epoll queue", ErrorCodes::SYSTEM_ERROR);
+            [[maybe_unused]] int err = ::close(epollfd);
+            chassert(!err || errno == EINTR);
+
+            throw ErrnoException(ErrorCodes::SYSTEM_ERROR, "Cannot insert socket into epoll queue");
         }
 #endif
     }
@@ -105,13 +132,13 @@ struct SocketInterruptablePollWrapper
         return pipe.fds_rw[1];
     }
 
-    PollResult poll(Poco::Timespan remaining_time, const std::shared_ptr<ReadBufferFromPocoSocket> & in)
+    PollResult poll(Poco::Timespan remaining_time, const ReadBufferFromPocoSocket & in)
     {
 
         bool socket_ready = false;
         bool fd_ready = false;
 
-        if (in->available() != 0)
+        if (in.available() != 0)
             socket_ready = true;
 
         if (response_in.available() != 0)
@@ -126,7 +153,8 @@ struct SocketInterruptablePollWrapper
             do
             {
                 Poco::Timestamp start;
-                rc = epoll_wait(epollfd, evout, 2, remaining_time.totalMilliseconds());
+                /// TODO: use epoll_pwait() for more precise timers
+                rc = epoll_wait(epollfd, evout, 2, static_cast<int>(remaining_time.totalMilliseconds()));
                 if (rc < 0 && errno == EINTR)
                 {
                     Poco::Timestamp end;
@@ -156,7 +184,7 @@ struct SocketInterruptablePollWrapper
             do
             {
                 Poco::Timestamp start;
-                rc = ::poll(poll_buf, 2, remaining_time.totalMilliseconds());
+                rc = ::poll(poll_buf, 2, static_cast<int>(remaining_time.totalMilliseconds()));
                 if (rc < 0 && errno == POCO_EINTR)
                 {
                     Poco::Timestamp end;
@@ -169,10 +197,13 @@ struct SocketInterruptablePollWrapper
             }
             while (rc < 0 && errno == POCO_EINTR);
 
-            if (rc >= 1 && poll_buf[0].revents & POLLIN)
-                socket_ready = true;
-            if (rc >= 1 && poll_buf[1].revents & POLLIN)
-                fd_ready = true;
+            if (rc >= 1)
+            {
+                if (poll_buf[0].revents & POLLIN)
+                    socket_ready = true;
+                if (poll_buf[1].revents & POLLIN)
+                    fd_ready = true;
+            }
 #endif
         }
 
@@ -197,42 +228,44 @@ struct SocketInterruptablePollWrapper
 #if defined(POCO_HAVE_FD_EPOLL)
     ~SocketInterruptablePollWrapper()
     {
-        ::close(epollfd);
+        [[maybe_unused]] int err = ::close(epollfd);
+        chassert(!err || errno == EINTR);
     }
 #endif
 };
 
-KeeperTCPHandler::KeeperTCPHandler(IServer & server_, const Poco::Net::StreamSocket & socket_)
+KeeperTCPHandler::KeeperTCPHandler(
+    const Poco::Util::AbstractConfiguration & config_ref,
+    std::shared_ptr<KeeperDispatcher> keeper_dispatcher_,
+    Poco::Timespan receive_timeout_,
+    Poco::Timespan send_timeout_,
+    const Poco::Net::StreamSocket & socket_)
     : Poco::Net::TCPServerConnection(socket_)
-    , server(server_)
-    , log(&Poco::Logger::get("KeeperTCPHandler"))
-    , global_context(Context::createCopy(server.context()))
-    , keeper_dispatcher(global_context->getKeeperDispatcher())
-    , operation_timeout(
-          0,
-          global_context->getConfigRef().getUInt(
-              "keeper_server.coordination_settings.operation_timeout_ms", Coordination::DEFAULT_OPERATION_TIMEOUT_MS) * 1000)
-    , min_session_timeout(
-          0,
-          global_context->getConfigRef().getUInt(
-              "keeper_server.coordination_settings.min_session_timeout_ms", Coordination::DEFAULT_MIN_SESSION_TIMEOUT_MS) * 1000)
-    , max_session_timeout(
-          0,
-          global_context->getConfigRef().getUInt(
-              "keeper_server.coordination_settings.session_timeout_ms", Coordination::DEFAULT_MAX_SESSION_TIMEOUT_MS) * 1000)
-    , poll_wrapper(std::make_unique<SocketInterruptablePollWrapper>(socket_))
+    , log(getLogger("KeeperTCPHandler"))
+    , keeper_dispatcher(keeper_dispatcher_)
+    , keeper_context(keeper_dispatcher->getKeeperContext())
+    , min_session_timeout(config_ref.getInt64("keeper_server.coordination_settings.min_session_timeout_ms", Coordination::DEFAULT_MIN_SESSION_TIMEOUT_MS) * 1000)
+    , max_session_timeout(config_ref.getInt64("keeper_server.coordination_settings.session_timeout_ms", Coordination::DEFAULT_MAX_SESSION_TIMEOUT_MS) * 1000)
+    , poll_wrapper(std::make_shared<SocketInterruptablePollWrapper>(socket_))
+    , send_timeout(send_timeout_)
+    , receive_timeout(receive_timeout_)
     , responses(std::make_unique<ThreadSafeResponseQueue>(std::numeric_limits<size_t>::max()))
     , last_op(std::make_unique<LastOp>(EMPTY_LAST_OP))
 {
     KeeperTCPHandler::registerConnection(this);
 }
 
-void KeeperTCPHandler::sendHandshake(bool has_leader)
+void KeeperTCPHandler::sendHandshake(bool has_leader, bool & use_compression)
 {
     Coordination::write(Coordination::SERVER_HANDSHAKE_LENGTH, *out);
     if (has_leader)
     {
-        Coordination::write(Coordination::ZOOKEEPER_PROTOCOL_VERSION, *out);
+        if (use_xid_64)
+            Coordination::write(Coordination::ZOOKEEPER_PROTOCOL_VERSION_WITH_XID_64, *out);
+        else if (use_compression)
+            Coordination::write(Coordination::ZOOKEEPER_PROTOCOL_VERSION_WITH_COMPRESSION, *out);
+        else
+            Coordination::write(Coordination::ZOOKEEPER_PROTOCOL_VERSION, *out);
     }
     else
     {
@@ -254,7 +287,7 @@ void KeeperTCPHandler::run()
     runImpl();
 }
 
-Poco::Timespan KeeperTCPHandler::receiveHandshake(int32_t handshake_length)
+Poco::Timespan KeeperTCPHandler::receiveHandshake(int32_t handshake_length, bool & use_compression)
 {
     int32_t protocol_version;
     int64_t last_zxid_seen;
@@ -263,41 +296,73 @@ Poco::Timespan KeeperTCPHandler::receiveHandshake(int32_t handshake_length)
     std::array<char, Coordination::PASSWORD_LENGTH> passwd {};
 
     if (!isHandShake(handshake_length))
-        throw Exception("Unexpected handshake length received: " + toString(handshake_length), ErrorCodes::UNEXPECTED_PACKET_FROM_CLIENT);
+        throw Exception(ErrorCodes::UNEXPECTED_PACKET_FROM_CLIENT, "Unexpected handshake length received: {}", toString(handshake_length));
 
     Coordination::read(protocol_version, *in);
 
-    if (protocol_version != Coordination::ZOOKEEPER_PROTOCOL_VERSION)
-        throw Exception("Unexpected protocol version: " + toString(protocol_version), ErrorCodes::UNEXPECTED_PACKET_FROM_CLIENT);
+    if (protocol_version != Coordination::ZOOKEEPER_PROTOCOL_VERSION
+        && protocol_version < Coordination::ZOOKEEPER_PROTOCOL_VERSION_WITH_COMPRESSION
+        && protocol_version > Coordination::ZOOKEEPER_PROTOCOL_VERSION_WITH_XID_64)
+    {
+        throw Exception(ErrorCodes::UNEXPECTED_PACKET_FROM_CLIENT, "Unexpected protocol version: {}", toString(protocol_version));
+    }
+
+    if (protocol_version == Coordination::ZOOKEEPER_PROTOCOL_VERSION_WITH_COMPRESSION)
+    {
+        use_compression = true;
+    }
+    else if (protocol_version >= Coordination::ZOOKEEPER_PROTOCOL_VERSION_WITH_XID_64)
+    {
+        if (!keeper_context->getCoordinationSettings()[CoordinationSetting::use_xid_64])
+            throw Exception(
+                ErrorCodes::UNEXPECTED_PACKET_FROM_CLIENT,
+                "keeper_server.coordination_settings.use_xid_64 is set to 'false' while client has it enabled");
+        close_xid = Coordination::CLOSE_XID_64;
+        use_xid_64 = true;
+        Coordination::read(use_compression, *in);
+    }
 
     Coordination::read(last_zxid_seen, *in);
     Coordination::read(timeout_ms, *in);
 
-    /// TODO Stop ignoring this value
     Coordination::read(previous_session_id, *in);
     Coordination::read(passwd, *in);
 
+
+    auto auth_data = keeper_dispatcher->getAuthenticationData();
+    if (auth_data)
+    {
+        String password{std::begin(passwd), std::end(passwd)};
+        /// It was padded to Coordination::PASSWORD_LENGTH with '\0'
+        boost::trim_right_if(password, [](char c) { return c == '\0'; });
+        AuthenticationData client_auth_data(auth_data->getType());
+        client_auth_data.setPassword(password, true);
+        if (client_auth_data != *auth_data)
+            throw Exception(ErrorCodes::AUTHENTICATION_FAILED, "Wrong password specified, authentication failed");
+    }
+
     int8_t readonly;
     if (handshake_length == Coordination::CLIENT_HANDSHAKE_LENGTH_WITH_READONLY)
         Coordination::read(readonly, *in);
 
-    return Poco::Timespan(0, timeout_ms * 1000);
+    return Poco::Timespan(static_cast<Poco::Timespan::TimeDiff>(timeout_ms) * 1000);
 }
 
 
 void KeeperTCPHandler::runImpl()
 {
     setThreadName("KeeperHandler");
-    ThreadStatus thread_status;
-    auto global_receive_timeout = global_context->getSettingsRef().receive_timeout;
-    auto global_send_timeout = global_context->getSettingsRef().send_timeout;
 
-    socket().setReceiveTimeout(global_receive_timeout);
-    socket().setSendTimeout(global_send_timeout);
+    socket().setReceiveTimeout(receive_timeout);
+    socket().setSendTimeout(send_timeout);
     socket().setNoDelay(true);
 
-    in = std::make_shared<ReadBufferFromPocoSocket>(socket());
-    out = std::make_shared<WriteBufferFromPocoSocket>(socket());
+    in.emplace(socket());
+    out.emplace(socket());
+    compressed_in.reset();
+    compressed_out.reset();
+
+    bool use_compression = false;
 
     if (in->eof())
     {
@@ -322,6 +387,7 @@ void KeeperTCPHandler::runImpl()
     int32_t four_letter_cmd = header;
     if (!isHandShake(four_letter_cmd))
     {
+        connected.store(true, std::memory_order_relaxed);
         tryExecuteFourLetterWordCmd(four_letter_cmd);
         return;
     }
@@ -329,10 +395,10 @@ void KeeperTCPHandler::runImpl()
     try
     {
         int32_t handshake_length = header;
-        auto client_timeout = receiveHandshake(handshake_length);
+        auto client_timeout = receiveHandshake(handshake_length, use_compression);
 
-        if (client_timeout == 0)
-            client_timeout = Coordination::DEFAULT_SESSION_TIMEOUT_MS;
+        if (client_timeout.totalMilliseconds() == 0)
+            client_timeout = Poco::Timespan(Coordination::DEFAULT_SESSION_TIMEOUT_MS * Poco::Timespan::MILLISECONDS);
         session_timeout = std::max(client_timeout, min_session_timeout);
         session_timeout = std::min(session_timeout, max_session_timeout);
     }
@@ -342,7 +408,7 @@ void KeeperTCPHandler::runImpl()
         return;
     }
 
-    if (keeper_dispatcher->checkInit() && keeper_dispatcher->hasLeader())
+    if (keeper_dispatcher->isServerActive())
     {
         try
         {
@@ -353,53 +419,51 @@ void KeeperTCPHandler::runImpl()
         catch (const Exception & e)
         {
             LOG_WARNING(log, "Cannot receive session id {}", e.displayText());
-            sendHandshake(false);
+            sendHandshake(/* has_leader */ false, use_compression);
             return;
 
         }
 
-        sendHandshake(true);
+        sendHandshake(/* has_leader */ true, use_compression);
     }
     else
     {
-        String reason;
-        if (!keeper_dispatcher->checkInit() && !keeper_dispatcher->hasLeader())
-            reason = "server is not initialized yet and no alive leader exists";
-        else if (!keeper_dispatcher->checkInit())
-            reason = "server is not initialized yet";
-        else
-            reason = "no alive leader exists";
-
-        LOG_WARNING(log, "Ignoring user request, because {}", reason);
-        sendHandshake(false);
+        LOG_WARNING(log, "Ignoring user request, because the server is not active yet");
+        sendHandshake(/* has_leader */ false, use_compression);
         return;
     }
 
-    auto response_fd = poll_wrapper->getResponseFD();
-    auto response_callback = [this, response_fd] (const Coordination::ZooKeeperResponsePtr & response)
+    if (use_compression)
     {
-        if (!responses->push(response))
-            throw Exception(ErrorCodes::SYSTEM_ERROR,
-                "Could not push response with xid {} and zxid {}",
-                response->xid,
-                response->zxid);
+        compressed_in.emplace(*in);
+        compressed_out.emplace(*out, CompressionCodecFactory::instance().get("LZ4",{}));
+    }
+
+    max_request_size = static_cast<UInt64>(keeper_context->getCoordinationSettings()[CoordinationSetting::max_request_size]);
+
+    auto response_callback = [my_responses = this->responses, my_poll_wrapper = this->poll_wrapper](
+                                 const Coordination::ZooKeeperResponsePtr & response, Coordination::ZooKeeperRequestPtr request)
+    {
+        if (!my_responses->push(RequestWithResponse{response, std::move(request)}))
+            throw Exception(ErrorCodes::SYSTEM_ERROR, "Could not push response with xid {} and zxid {}", response->xid, response->zxid);
 
         UInt8 single_byte = 1;
-        [[maybe_unused]] int result = write(response_fd, &single_byte, sizeof(single_byte));
+        [[maybe_unused]] ssize_t result = write(my_poll_wrapper->getResponseFD(), &single_byte, sizeof(single_byte));
     };
     keeper_dispatcher->registerSession(session_id, response_callback);
 
     Stopwatch logging_stopwatch;
+    auto operation_max_ms = keeper_context->getCoordinationSettings()[CoordinationSetting::log_slow_connection_operation_threshold_ms];
     auto log_long_operation = [&](const String & operation)
     {
-        constexpr UInt64 operation_max_ms = 500;
         auto elapsed_ms = logging_stopwatch.elapsedMilliseconds();
         if (operation_max_ms < elapsed_ms)
-            LOG_TEST(log, "{} for session {} took {} ms", operation, session_id, elapsed_ms);
+            LOG_INFO(log, "{} for session {} took {} ms", operation, session_id, elapsed_ms);
         logging_stopwatch.restart();
     };
 
     session_stopwatch.start();
+    connected.store(true, std::memory_order_release);
     bool close_received = false;
 
     try
@@ -408,10 +472,16 @@ void KeeperTCPHandler::runImpl()
         {
             using namespace std::chrono_literals;
 
-            PollResult result = poll_wrapper->poll(session_timeout, in);
-            log_long_operation("Polling socket");
+            PollResult result = poll_wrapper->poll(session_timeout, *in);
             if (result.has_requests && !close_received)
             {
+                if (in->eof())
+                {
+                    LOG_DEBUG(log, "Client closed connection, session id #{}", session_id);
+                    keeper_dispatcher->finishSession(session_id);
+                    break;
+                }
+
                 auto [received_op, received_xid] = receiveRequest();
                 packageReceived();
                 log_long_operation("Receiving request");
@@ -438,22 +508,24 @@ void KeeperTCPHandler::runImpl()
             /// became inconsistent and race condition is possible.
             while (result.responses_count != 0)
             {
-                Coordination::ZooKeeperResponsePtr response;
+                RequestWithResponse request_with_response;
 
-                if (!responses->tryPop(response))
+                if (!responses->tryPop(request_with_response))
                     throw Exception(ErrorCodes::LOGICAL_ERROR, "We must have ready response, but queue is empty. It's a bug.");
                 log_long_operation("Waiting for response to be ready");
 
+                auto & response = request_with_response.response;
                 if (response->xid == close_xid)
                 {
                     LOG_DEBUG(log, "Session #{} successfully closed", session_id);
                     return;
                 }
 
-                updateStats(response);
+                updateStats(response, request_with_response.request);
                 packageSent();
 
-                response->write(*out);
+                response->write(getWriteBuffer(), use_xid_64);
+                flushWriteBuffer();
                 log_long_operation("Sending response");
                 if (response->error == Coordination::Error::ZSESSIONEXPIRED)
                 {
@@ -466,7 +538,7 @@ void KeeperTCPHandler::runImpl()
             }
 
             if (result.error)
-                throw Exception("Exception happened while reading from socket", ErrorCodes::SYSTEM_ERROR);
+                throw Exception(ErrorCodes::SYSTEM_ERROR, "Exception happened while reading from socket");
 
             if (session_stopwatch.elapsedMicroseconds() > static_cast<UInt64>(session_timeout.totalMicroseconds()))
             {
@@ -475,12 +547,14 @@ void KeeperTCPHandler::runImpl()
                 break;
             }
         }
+        finalizeWriteBuffer();
     }
     catch (const Exception & ex)
     {
         log_long_operation("Unknown operation");
         LOG_TRACE(log, "Has {} responses in the queue", responses->size());
         LOG_INFO(log, "Got exception processing session #{}: {}", session_id, getExceptionMessage(ex, true));
+        cancelWriteBuffer();
         keeper_dispatcher->finishSession(session_id);
     }
 }
@@ -498,81 +572,160 @@ bool KeeperTCPHandler::tryExecuteFourLetterWordCmd(int32_t command)
         LOG_WARNING(log, "invalid four letter command {}", IFourLetterCommand::toName(command));
         return false;
     }
-    else if (!FourLetterCommandFactory::instance().isEnabled(command))
+    if (!FourLetterCommandFactory::instance().isEnabled(command))
     {
         LOG_WARNING(log, "Not enabled four letter command {}", IFourLetterCommand::toName(command));
         return false;
     }
-    else
+
+    auto command_ptr = FourLetterCommandFactory::instance().get(command);
+    LOG_DEBUG(log, "Receive four letter command {}", command_ptr->name());
+
+    try
     {
-        auto command_ptr = FourLetterCommandFactory::instance().get(command);
-        LOG_DEBUG(log, "Receive four letter command {}", command_ptr->name());
-
-        try
-        {
-            String res = command_ptr->run();
-            out->write(res.data(), res.size());
-            out->next();
-        }
-        catch (...)
-        {
-            tryLogCurrentException(log, "Error when executing four letter command " + command_ptr->name());
-        }
-
-        return true;
+        String res = command_ptr->run();
+        out->write(res.data(), res.size());
+        out->next();
     }
+    catch (...)
+    {
+        tryLogCurrentException(log, "Error when executing four letter command " + command_ptr->name());
+    }
+
+    return true;
+}
+
+WriteBuffer & KeeperTCPHandler::getWriteBuffer()
+{
+    if (compressed_out)
+        return *compressed_out;
+    return *out;
+}
+
+void KeeperTCPHandler::flushWriteBuffer()
+{
+    if (compressed_out)
+        compressed_out->next();
+    out->next();
+}
+
+void KeeperTCPHandler::finalizeWriteBuffer()
+{
+    if (compressed_out)
+        compressed_out->finalize();
+    out->finalize();
+}
+
+void KeeperTCPHandler::cancelWriteBuffer() noexcept
+{
+    if (compressed_out)
+        compressed_out->cancel();
+    if (out)
+        out->cancel();
+}
+
+ReadBuffer & KeeperTCPHandler::getReadBuffer()
+{
+    if (compressed_in)
+        return *compressed_in;
+    return *in;
 }
 
 std::pair<Coordination::OpNum, Coordination::XID> KeeperTCPHandler::receiveRequest()
 {
+    std::optional<LimitReadBuffer> limited_buffer_holder;
+    /// Wrap regular read buffer with LimitReadBuffer to apply max_request_size
+    /// (this should be done on per-request basis)
+    auto get_read_buffer_with_limit = [&]() -> ReadBuffer &
+    {
+        if (max_request_size == 0)
+            return getReadBuffer();
+        limited_buffer_holder.emplace(getReadBuffer(), LimitReadBuffer::Settings{.read_no_more = max_request_size, .expect_eof = false, .excetion_hint = "too long packet."});
+        return *limited_buffer_holder;
+    };
+    auto & read_buffer = get_read_buffer_with_limit();
     int32_t length;
-    Coordination::read(length, *in);
-    int32_t xid;
-    Coordination::read(xid, *in);
+    Coordination::read(length, read_buffer);
+    if (length < 0 || (max_request_size > 0 && static_cast<uint32_t>(length) > max_request_size))
+        throw Exception(ErrorCodes::LIMIT_EXCEEDED, "Request size {} is too big request (limit: {})", length, max_request_size);
+
+    int64_t xid;
+    if (use_xid_64)
+        Coordination::read(xid, read_buffer);
+    else
+    {
+        int32_t read_xid;
+        Coordination::read(read_xid, read_buffer);
+        xid = read_xid;
+    }
 
     Coordination::OpNum opnum;
-    Coordination::read(opnum, *in);
+    Coordination::read(opnum, read_buffer);
 
     Coordination::ZooKeeperRequestPtr request = Coordination::ZooKeeperRequestFactory::instance().get(opnum);
     request->xid = xid;
-    request->readImpl(*in);
 
-    if (!keeper_dispatcher->putRequest(request, session_id))
+    auto request_validator = [&](const Coordination::ZooKeeperRequest & current_request)
+    {
+        if (!keeper_context->isOperationSupported(current_request.getOpNum()))
+            throw Exception(ErrorCodes::BAD_ARGUMENTS, "Unsupported operation: {}, Path: '{}'", current_request.getOpNum(), current_request.getPath());
+    };
+
+    if (auto * multi_request = dynamic_cast<Coordination::ZooKeeperMultiRequest *>(request.get()))
+    {
+        multi_request->readImpl(read_buffer, request_validator);
+    }
+    else
+    {
+        request->readImpl(read_buffer);
+        request_validator(*request);
+    }
+
+
+    if (!keeper_dispatcher->putRequest(request, session_id, use_xid_64))
         throw Exception(ErrorCodes::TIMEOUT_EXCEEDED, "Session {} already disconnected", session_id);
+
+    if (keeper_context->shouldLogRequests())
+        LOG_TRACE(log, "Received request:\n{}", request->toString(/*short_format=*/true));
+
     return std::make_pair(opnum, xid);
 }
 
 void KeeperTCPHandler::packageSent()
 {
-    {
-        std::lock_guard lock(conn_stats_mutex);
-        conn_stats.incrementPacketsSent();
-    }
+    conn_stats.incrementPacketsSent();
     keeper_dispatcher->incrementPacketsSent();
 }
 
 void KeeperTCPHandler::packageReceived()
 {
-    {
-        std::lock_guard lock(conn_stats_mutex);
-        conn_stats.incrementPacketsReceived();
-    }
+    conn_stats.incrementPacketsReceived();
     keeper_dispatcher->incrementPacketsReceived();
 }
 
-void KeeperTCPHandler::updateStats(Coordination::ZooKeeperResponsePtr & response)
+void KeeperTCPHandler::updateStats(Coordination::ZooKeeperResponsePtr & response, const Coordination::ZooKeeperRequestPtr & request)
 {
     /// update statistics ignoring watch response and heartbeat.
     if (response->xid != Coordination::WATCH_XID && response->getOpNum() != Coordination::OpNum::Heartbeat)
     {
-        Int64 elapsed = (Poco::Timestamp() - operations[response->xid]) / 1000;
+        Int64 elapsed = (Poco::Timestamp() - operations[response->xid]);
+        ProfileEvents::increment(ProfileEvents::KeeperTotalElapsedMicroseconds, elapsed);
+        Int64 elapsed_ms = elapsed / 1000;
+
+        if (request && elapsed_ms > static_cast<Int64>(keeper_context->getCoordinationSettings()[CoordinationSetting::log_slow_total_threshold_ms]))
         {
-            std::lock_guard lock(conn_stats_mutex);
-            conn_stats.updateLatency(elapsed);
+            LOG_INFO(
+                log,
+                "Total time to process a request in session {} took too long ({}ms).\nRequest info:\n{}",
+                session_id,
+                elapsed_ms,
+                request->toString(/*short_format=*/true));
         }
 
+        conn_stats.updateLatency(elapsed_ms);
+
         operations.erase(response->xid);
-        keeper_dispatcher->updateKeeperStatLatency(elapsed);
+        keeper_dispatcher->updateKeeperStatLatency(elapsed_ms);
 
         last_op.set(std::make_unique<LastOp>(LastOp{
             .name = Coordination::toString(response->getOpNum()),
@@ -584,15 +737,17 @@ void KeeperTCPHandler::updateStats(Coordination::ZooKeeperResponsePtr & response
 
 }
 
-KeeperConnectionStats KeeperTCPHandler::getConnectionStats() const
+KeeperConnectionStats & KeeperTCPHandler::getConnectionStats()
 {
-    std::lock_guard lock(conn_stats_mutex);
     return conn_stats;
 }
 
 void KeeperTCPHandler::dumpStats(WriteBufferFromOwnString & buf, bool brief)
 {
-    KeeperConnectionStats stats = getConnectionStats();
+    if (!connected.load(std::memory_order_acquire))
+        return;
+
+    auto & stats = getConnectionStats();
 
     writeText(' ', buf);
     writeText(socket().peerAddress().toString(), buf);
@@ -641,15 +796,13 @@ void KeeperTCPHandler::dumpStats(WriteBufferFromOwnString & buf, bool brief)
 
 void KeeperTCPHandler::resetStats()
 {
-    {
-        std::lock_guard lock(conn_stats_mutex);
-        conn_stats.reset();
-    }
+    conn_stats.reset();
     last_op.set(std::make_unique<LastOp>(EMPTY_LAST_OP));
 }
 
 KeeperTCPHandler::~KeeperTCPHandler()
 {
+    cancelWriteBuffer();
     KeeperTCPHandler::unregisterConnection(this);
 }
 
