@@ -1,6 +1,5 @@
 #pragma once
 
-#include <Common/SipHash.h>
 #include <base/types.h>
 #include <unordered_set>
 
@@ -24,7 +23,10 @@ struct ColumnDependency
         TTL_EXPRESSION,
 
         /// TTL is set for @column_name.
-        TTL_TARGET
+        TTL_TARGET,
+
+        /// Exists any statistics, that requires @column_name
+        STATISTICS,
     };
 
     ColumnDependency(const String & column_name_, Kind kind_)
@@ -45,13 +47,7 @@ struct ColumnDependency
 
     struct Hash
     {
-        UInt64 operator()(const ColumnDependency & dependency) const
-        {
-            SipHash hash;
-            hash.update(dependency.column_name);
-            hash.update(dependency.kind);
-            return hash.get64();
-        }
+        UInt64 operator()(const ColumnDependency & dependency) const;
     };
 };
 
