@@ -3,6 +3,7 @@
 #include <Storages/MergeTree/MergeTreeIndices.h>
 #include <Storages/MergeTree/MergeTreeData.h>
 #include <Interpreters/ComparisonGraph.h>
+#include <Interpreters/TreeCNFConverter.h>
 
 namespace DB
 {
@@ -20,8 +21,8 @@ public:
 
 private:
     void addConstraints(const ConstraintsDescription & constraints_description);
-    std::unique_ptr<ComparisonGraph> buildGraph(const std::vector<bool> & values) const;
-    const ComparisonGraph * getGraph(const std::vector<bool> & values) const;
+    std::unique_ptr<ComparisonGraph<ASTPtr>> buildGraph(const std::vector<bool> & values) const;
+    const ComparisonGraph<ASTPtr> * getGraph(const std::vector<bool> & values) const;
 
     ASTPtr expression_ast;
     std::unique_ptr<CNFQuery> expression_cnf;
@@ -29,12 +30,12 @@ private:
     /// Part analysis can be done in parallel.
     /// So, we have shared answer and graph cache.
     mutable std::mutex cache_mutex;
-    mutable std::unordered_map<std::vector<bool>, std::unique_ptr<ComparisonGraph>> graph_cache;
+    mutable std::unordered_map<std::vector<bool>, std::unique_ptr<ComparisonGraph<ASTPtr>>> graph_cache;
     mutable std::unordered_map<std::vector<bool>, bool> answer_cache;
 
     std::vector<std::vector<ASTPtr>> index_to_compare_atomic_hypotheses;
     std::vector<std::vector<CNFQuery::OrGroup>> index_to_atomic_hypotheses;
-    std::vector<ASTPtr> atomic_constraints;
+    ASTs atomic_constraints;
 };
 
 }
