@@ -1,11 +1,14 @@
 #pragma once
 
+#include <Columns/IColumn.h>
 #include <Common/HashTable/HashMap.h>
 #include <Common/Arena.h>
+#include <Common/PODArray.h>
 #include <absl/container/flat_hash_map.h>
 #include <Processors/Sinks/SinkToStorage.h>
 #include <Interpreters/ExpressionAnalyzer.h>
 #include <Interpreters/Context_fwd.h>
+#include <Storages/IPartitionStrategy.h>
 
 
 namespace DB
@@ -16,11 +19,18 @@ class PartitionedSink : public SinkToStorage
 public:
     static constexpr auto PARTITION_ID_WILDCARD = "{_partition_id}";
 
-    PartitionedSink(const ASTPtr & partition_by, ContextPtr context_, const Block & sample_block_);
+    PartitionedSink(
+        std::shared_ptr<IPartitionStrategy> partition_strategy_,
+        ContextPtr context_,
+        SharedHeader source_header_);
+
+    ~PartitionedSink() override;
 
     String getName() const override { return "PartitionedSink"; }
 
-    void consume(Chunk chunk) override;
+    void consume(Chunk & chunk) override;
+
+    void onException(std::exception_ptr exception) override;
 
     void onFinish() override;
 
@@ -30,12 +40,12 @@ public:
 
     static String replaceWildcards(const String & haystack, const String & partition_id);
 
+protected:
+    std::shared_ptr<IPartitionStrategy> partition_strategy;
+
 private:
     ContextPtr context;
-    Block sample_block;
-
-    ExpressionActionsPtr partition_by_expr;
-    String partition_by_column_name;
+    SharedHeader source_header;
 
     absl::flat_hash_map<StringRef, SinkPtr> partition_id_to_sink;
     HashMapWithSavedHash<StringRef, size_t> partition_id_to_chunk_index;
@@ -43,7 +53,6 @@ private:
     Arena partition_keys_arena;
 
     SinkPtr getSinkForPartitionKey(StringRef partition_key);
-
 };
 
 }
