@@ -1,14 +1,25 @@
 #include <Storages/StorageInMemoryMetadata.h>
 
+#include <Access/AccessControl.h>
+#include <Access/User.h>
+
+#include <Core/Settings.h>
+
 #include <Common/HashTable/HashMap.h>
 #include <Common/HashTable/HashSet.h>
 #include <Common/quoteString.h>
-#include <Common/StringUtils/StringUtils.h>
+#include <Common/StringUtils.h>
 #include <Core/ColumnWithTypeAndName.h>
+#include <DataTypes/NestedUtils.h>
 #include <DataTypes/DataTypeEnum.h>
+#include <Interpreters/Context.h>
+#include <Interpreters/ExpressionActions.h>
 #include <IO/ReadBufferFromString.h>
 #include <IO/ReadHelpers.h>
 #include <IO/Operators.h>
+#include <Parsers/ASTSQLSecurity.h>
+#include <Parsers/ASTSetQuery.h>
+#include <Storages/MergeTree/MergeTreeVirtualColumns.h>
 
 
 namespace DB
@@ -22,6 +33,7 @@ namespace ErrorCodes
     extern const int NOT_FOUND_COLUMN_IN_BLOCK;
     extern const int TYPE_MISMATCH;
     extern const int EMPTY_LIST_OF_COLUMNS_PASSED;
+    extern const int LOGICAL_ERROR;
 }
 
 StorageInMemoryMetadata::StorageInMemoryMetadata(const StorageInMemoryMetadata & other)
@@ -39,7 +51,12 @@ StorageInMemoryMetadata::StorageInMemoryMetadata(const StorageInMemoryMetadata &
     , table_ttl(other.table_ttl)
     , settings_changes(other.settings_changes ? other.settings_changes->clone() : nullptr)
     , select(other.select)
+    , refresh(other.refresh ? other.refresh->clone() : nullptr)
+    , definer(other.definer)
+    , sql_security_type(other.sql_security_type)
     , comment(other.comment)
+    , metadata_version(other.metadata_version)
+    , datalake_table_state(other.datalake_table_state)
 {
 }
 
@@ -67,7 +84,13 @@ StorageInMemoryMetadata & StorageInMemoryMetadata::operator=(const StorageInMemo
     else
         settings_changes.reset();
     select = other.select;
+    refresh = other.refresh ? other.refresh->clone() : nullptr;
+    definer = other.definer;
+    sql_security_type = other.sql_security_type;
     comment = other.comment;
+    metadata_version = other.metadata_version;
+    datalake_table_state = other.datalake_table_state;
+
     return *this;
 }
 
@@ -76,10 +99,79 @@ void StorageInMemoryMetadata::setComment(const String & comment_)
     comment = comment_;
 }
 
+void StorageInMemoryMetadata::setSQLSecurity(const ASTSQLSecurity & sql_security)
+{
+    if (sql_security.definer)
+        definer = sql_security.definer->toString();
+    else
+        definer = std::nullopt;
+
+    sql_security_type = sql_security.type;
+}
+
+UUID StorageInMemoryMetadata::getDefinerID(DB::ContextPtr context) const
+{
+    if (!definer)
+    {
+        if (const auto definer_id = context->getUserID())
+            return *definer_id;
+
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "No user in context for sub query execution.");
+    }
+
+    const auto & access_control = context->getAccessControl();
+    return access_control.getID<User>(*definer);
+}
+
+ContextMutablePtr StorageInMemoryMetadata::getSQLSecurityOverriddenContext(ContextPtr context, const ClientInfo * client_info) const
+{
+    if (!sql_security_type)
+        return Context::createCopy(context);
+
+    if (sql_security_type == SQLSecurityType::INVOKER)
+        return Context::createCopy(context);
+
+    auto new_context = Context::createCopy(context->getGlobalContext());
+    if (client_info)
+        new_context->setClientInfo(*client_info);
+    else
+        new_context->setClientInfo(context->getClientInfo());
+    new_context->makeQueryContext();
+
+    const auto & database = context->getCurrentDatabase();
+    if (!database.empty() && database != new_context->getCurrentDatabase())
+        new_context->setCurrentDatabase(database);
+
+    new_context->setInsertionTable(context->getInsertionTable(), context->getInsertionTableColumnNames());
+    new_context->setProgressCallback(context->getProgressCallback());
+    new_context->setProcessListElement(context->getProcessListElement());
+
+    if (context->getCurrentTransaction())
+        new_context->setCurrentTransaction(context->getCurrentTransaction());
+
+    if (context->getZooKeeperMetadataTransaction())
+        new_context->initZooKeeperMetadataTransaction(context->getZooKeeperMetadataTransaction());
+
+    if (sql_security_type == SQLSecurityType::NONE)
+    {
+        new_context->applySettingsChanges(context->getSettingsRef().changes());
+        return new_context;
+    }
+
+    new_context->setUser(getDefinerID(context));
+
+    auto changed_settings = context->getSettingsRef().changes();
+    new_context->clampToSettingsConstraints(changed_settings, SettingSource::QUERY);
+    new_context->applySettingsChanges(changed_settings);
+    new_context->setSetting("allow_ddl", 1);
+
+    return new_context;
+}
+
 void StorageInMemoryMetadata::setColumns(ColumnsDescription columns_)
 {
     if (columns_.getAllPhysical().empty())
-        throw Exception("Empty list of columns passed", ErrorCodes::EMPTY_LIST_OF_COLUMNS_PASSED);
+        throw Exception(ErrorCodes::EMPTY_LIST_OF_COLUMNS_PASSED, "Empty list of columns passed");
     columns = std::move(columns_);
 }
 
@@ -121,6 +213,28 @@ void StorageInMemoryMetadata::setSelectQuery(const SelectQueryDescription & sele
     select = select_;
 }
 
+void StorageInMemoryMetadata::setRefresh(ASTPtr refresh_)
+{
+    refresh = refresh_;
+}
+
+void StorageInMemoryMetadata::setMetadataVersion(int32_t metadata_version_)
+{
+    metadata_version = metadata_version_;
+}
+
+void StorageInMemoryMetadata::setDataLakeTableState(const DataLakeTableStateSnapshot & datalake_table_state_)
+{
+    datalake_table_state = datalake_table_state_;
+}
+
+StorageInMemoryMetadata StorageInMemoryMetadata::withMetadataVersion(int32_t metadata_version_) const
+{
+    StorageInMemoryMetadata copy(*this);
+    copy.setMetadataVersion(metadata_version_);
+    return copy;
+}
+
 const ColumnsDescription & StorageInMemoryMetadata::getColumns() const
 {
     return columns;
@@ -161,6 +275,12 @@ bool StorageInMemoryMetadata::hasAnyTableTTL() const
     return hasAnyMoveTTL() || hasRowsTTL() || hasAnyRecompressionTTL() || hasAnyGroupByTTL() || hasAnyRowsWhereTTL();
 }
 
+bool StorageInMemoryMetadata::hasOnlyRowsTTL() const
+{
+    bool has_any_other_ttl = hasAnyMoveTTL() || hasAnyRecompressionTTL() || hasAnyGroupByTTL() || hasAnyRowsWhereTTL() || hasAnyColumnTTL();
+    return hasRowsTTL() && !has_any_other_ttl;
+}
+
 TTLColumnsDescription StorageInMemoryMetadata::getColumnTTLs() const
 {
     return column_ttls_by_name;
@@ -178,7 +298,7 @@ TTLDescription StorageInMemoryMetadata::getRowsTTL() const
 
 bool StorageInMemoryMetadata::hasRowsTTL() const
 {
-    return table_ttl.rows_ttl.expression != nullptr;
+    return table_ttl.rows_ttl.expression_ast != nullptr;
 }
 
 TTLDescriptions StorageInMemoryMetadata::getRowsWhereTTLs() const
@@ -221,7 +341,10 @@ bool StorageInMemoryMetadata::hasAnyGroupByTTL() const
     return !table_ttl.group_by_ttl.empty();
 }
 
-ColumnDependencies StorageInMemoryMetadata::getColumnDependencies(const NameSet & updated_columns, bool include_ttl_target) const
+ColumnDependencies StorageInMemoryMetadata::getColumnDependencies(
+    const NameSet & updated_columns,
+    bool include_ttl_target,
+    const HasDependencyCallback & has_dependency) const
 {
     if (updated_columns.empty())
         return {};
@@ -233,12 +356,18 @@ ColumnDependencies StorageInMemoryMetadata::getColumnDependencies(const NameSet
     NameSet required_ttl_columns;
     NameSet updated_ttl_columns;
 
-    auto add_dependent_columns = [&updated_columns](const auto & expression, auto & to_set)
+    auto add_dependent_columns = [&updated_columns](const Names & required_columns, auto & to_set, bool is_projection = false)
     {
-        auto required_columns = expression->getRequiredColumns();
         for (const auto & dependency : required_columns)
         {
-            if (updated_columns.count(dependency))
+            /// useful in the case of lightweight delete with wide part and option of rebuild projection
+            if (is_projection && updated_columns.contains(RowExistsColumn::name))
+            {
+                to_set.insert(required_columns.begin(), required_columns.end());
+                return true;
+            }
+
+            if (updated_columns.contains(dependency))
             {
                 to_set.insert(required_columns.begin(), required_columns.end());
                 return true;
@@ -249,14 +378,20 @@ ColumnDependencies StorageInMemoryMetadata::getColumnDependencies(const NameSet
     };
 
     for (const auto & index : getSecondaryIndices())
-        add_dependent_columns(index.expression, indices_columns);
+    {
+        if (has_dependency(index.name, ColumnDependency::SKIP_INDEX))
+            add_dependent_columns(index.expression->getRequiredColumns(), indices_columns);
+    }
 
     for (const auto & projection : getProjections())
-        add_dependent_columns(&projection, projections_columns);
+    {
+        if (has_dependency(projection.name, ColumnDependency::PROJECTION))
+            add_dependent_columns(projection.getRequiredColumns(), projections_columns, true);
+    }
 
     auto add_for_rows_ttl = [&](const auto & expression, auto & to_set)
     {
-        if (add_dependent_columns(expression, to_set) && include_ttl_target)
+        if (add_dependent_columns(expression.getNames(), to_set) && include_ttl_target)
         {
             /// Filter all columns, if rows TTL expression have to be recalculated.
             for (const auto & column : getColumns().getAllPhysical())
@@ -265,25 +400,25 @@ ColumnDependencies StorageInMemoryMetadata::getColumnDependencies(const NameSet
     };
 
     if (hasRowsTTL())
-        add_for_rows_ttl(getRowsTTL().expression, required_ttl_columns);
+        add_for_rows_ttl(getRowsTTL().expression_columns, required_ttl_columns);
 
     for (const auto & entry : getRowsWhereTTLs())
-        add_for_rows_ttl(entry.expression, required_ttl_columns);
+        add_for_rows_ttl(entry.expression_columns, required_ttl_columns);
 
     for (const auto & entry : getGroupByTTLs())
-        add_for_rows_ttl(entry.expression, required_ttl_columns);
+        add_for_rows_ttl(entry.expression_columns, required_ttl_columns);
 
     for (const auto & entry : getRecompressionTTLs())
-        add_dependent_columns(entry.expression, required_ttl_columns);
+        add_dependent_columns(entry.expression_columns.getNames(), required_ttl_columns);
 
     for (const auto & [name, entry] : getColumnTTLs())
     {
-        if (add_dependent_columns(entry.expression, required_ttl_columns) && include_ttl_target)
+        if (add_dependent_columns(entry.expression_columns.getNames(), required_ttl_columns) && include_ttl_target)
             updated_ttl_columns.insert(name);
     }
 
     for (const auto & entry : getMoveTTLs())
-        add_dependent_columns(entry.expression, required_ttl_columns);
+        add_dependent_columns(entry.expression_columns.getNames(), required_ttl_columns);
 
     //TODO what about rows_where_ttl and group_by_ttl ??
 
@@ -297,7 +432,6 @@ ColumnDependencies StorageInMemoryMetadata::getColumnDependencies(const NameSet
         res.emplace(column, ColumnDependency::TTL_TARGET);
 
     return res;
-
 }
 
 Block StorageInMemoryMetadata::getSampleBlockInsertable() const
@@ -342,35 +476,12 @@ Block StorageInMemoryMetadata::getSampleBlock() const
     return res;
 }
 
-Block StorageInMemoryMetadata::getSampleBlockForColumns(
-    const Names & column_names, const NamesAndTypesList & virtuals, const StorageID & storage_id) const
+Block StorageInMemoryMetadata::getSampleBlockWithSubcolumns() const
 {
     Block res;
 
-    HashMapWithSavedHash<StringRef, const DataTypePtr *, StringRefHash> virtuals_map;
-
-    /// Virtual columns must be appended after ordinary, because user can
-    /// override them.
-    for (const auto & column : virtuals)
-        virtuals_map[column.name] = &column.type;
-
-    for (const auto & name : column_names)
-    {
-        auto column = getColumns().tryGetColumnOrSubcolumn(ColumnsDescription::All, name);
-        if (column)
-        {
-            res.insert({column->type->createColumn(), column->type, column->name});
-        }
-        else if (auto * it = virtuals_map.find(name); it != virtuals_map.end())
-        {
-            const auto & type = *it->getMapped();
-            res.insert({type->createColumn(), type, name});
-        }
-        else
-            throw Exception(
-                "Column " + backQuote(name) + " not found in table " + (storage_id.empty() ? "" : storage_id.getNameForLogs()),
-                ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK);
-    }
+    for (const auto & column : getColumns().get(GetColumnsOptions(GetColumnsOptions::AllPhysical).withSubcolumns()))
+        res.insert({column.type->createColumn(), column.type, column.name});
 
     return res;
 }
@@ -427,6 +538,13 @@ Names StorageInMemoryMetadata::getSortingKeyColumns() const
     return {};
 }
 
+std::vector<bool> StorageInMemoryMetadata::getSortingKeyReverseFlags() const
+{
+    if (hasSortingKey())
+        return sorting_key.reverse_flags;
+    return {};
+}
+
 const KeyDescription & StorageInMemoryMetadata::getSamplingKey() const
 {
     return sampling_key;
@@ -484,6 +602,17 @@ ASTPtr StorageInMemoryMetadata::getSettingsChanges() const
         return settings_changes->clone();
     return nullptr;
 }
+
+Field StorageInMemoryMetadata::getSettingChange(const String & setting_name) const
+{
+    if (!settings_changes)
+        return Field();
+
+    const auto & changes = settings_changes->as<const ASTSetQuery &>().changes;
+    auto it = std::ranges::find_if(changes, [&setting_name](const SettingChange & change) { return change.name == setting_name; });
+    return it != changes.end() ? it->value : Field();
+}
+
 const SelectQueryDescription & StorageInMemoryMetadata::getSelectQuery() const
 {
     return select;
@@ -499,18 +628,6 @@ namespace
     using NamesAndTypesMap = HashMapWithSavedHash<StringRef, const IDataType *, StringRefHash>;
     using UniqueStrings = HashSetWithSavedHash<StringRef, StringRefHash>;
 
-    String listOfColumns(const NamesAndTypesList & available_columns)
-    {
-        WriteBufferFromOwnString ss;
-        for (auto it = available_columns.begin(); it != available_columns.end(); ++it)
-        {
-            if (it != available_columns.begin())
-                ss << ", ";
-            ss << it->name;
-        }
-        return ss.str();
-    }
-
     NamesAndTypesMap getColumnsMap(const NamesAndTypesList & columns)
     {
         NamesAndTypesMap res;
@@ -539,36 +656,16 @@ namespace
     }
 }
 
-void StorageInMemoryMetadata::check(const Names & column_names, const NamesAndTypesList & virtuals, const StorageID & storage_id) const
+String listOfColumns(const NamesAndTypesList & available_columns)
 {
-    if (column_names.empty())
+    WriteBufferFromOwnString ss;
+    for (auto it = available_columns.begin(); it != available_columns.end(); ++it)
     {
-        auto list_of_columns = listOfColumns(getColumns().getAllPhysicalWithSubcolumns());
-        throw Exception(ErrorCodes::EMPTY_LIST_OF_COLUMNS_QUERIED,
-            "Empty list of columns queried. There are columns: {}", list_of_columns);
-    }
-
-    const auto virtuals_map = getColumnsMap(virtuals);
-    UniqueStrings unique_names;
-
-    for (const auto & name : column_names)
-    {
-        bool has_column = getColumns().hasColumnOrSubcolumn(ColumnsDescription::AllPhysical, name)
-            || virtuals_map.find(name) != nullptr;
-
-        if (!has_column)
-        {
-            auto list_of_columns = listOfColumns(getColumns().getAllPhysicalWithSubcolumns());
-            throw Exception(ErrorCodes::NO_SUCH_COLUMN_IN_TABLE,
-                "There is no column with name {} in table {}. There are columns: {}",
-                backQuote(name), storage_id.getNameForLogs(), list_of_columns);
-        }
-
-        if (unique_names.end() != unique_names.find(name))
-            throw Exception(ErrorCodes::COLUMN_QUERIED_MORE_THAN_ONCE, "Column {} queried more than once", name);
-
-        unique_names.insert(name);
+        if (it != available_columns.begin())
+            ss << ", ";
+        ss << it->name;
     }
+    return ss.str();
 }
 
 void StorageInMemoryMetadata::check(const NamesAndTypesList & provided_columns) const
@@ -589,7 +686,10 @@ void StorageInMemoryMetadata::check(const NamesAndTypesList & provided_columns)
                 listOfColumns(available_columns));
 
         const auto * available_type = it->getMapped();
-        if (!column.type->equals(*available_type) && !isCompatibleEnumTypes(available_type, column.type.get()))
+
+        if (!available_type->hasDynamicSubcolumnsDeprecated()
+            && !column.type->equals(*available_type)
+            && !isCompatibleEnumTypes(available_type, column.type.get()))
             throw Exception(
                 ErrorCodes::TYPE_MISMATCH,
                 "Type mismatch for column {}. Column has type {}, got type {}",
@@ -613,9 +713,8 @@ void StorageInMemoryMetadata::check(const NamesAndTypesList & provided_columns,
     const auto & provided_columns_map = getColumnsMap(provided_columns);
 
     if (column_names.empty())
-        throw Exception(
-            "Empty list of columns queried. There are columns: " + listOfColumns(available_columns),
-            ErrorCodes::EMPTY_LIST_OF_COLUMNS_QUERIED);
+        throw Exception(ErrorCodes::EMPTY_LIST_OF_COLUMNS_QUERIED, "Empty list of columns queried. There are columns: {}",
+            listOfColumns(available_columns));
 
     UniqueStrings unique_names;
 
@@ -636,7 +735,9 @@ void StorageInMemoryMetadata::check(const NamesAndTypesList & provided_columns,
         const auto * provided_column_type = it->getMapped();
         const auto * available_column_type = jt->getMapped();
 
-        if (!provided_column_type->equals(*available_column_type) && !isCompatibleEnumTypes(available_column_type, provided_column_type))
+        if (!provided_column_type->hasDynamicSubcolumnsDeprecated()
+            && !provided_column_type->equals(*available_column_type)
+            && !isCompatibleEnumTypes(available_column_type, provided_column_type))
             throw Exception(
                 ErrorCodes::TYPE_MISMATCH,
                 "Type mismatch for column {}. Column has type {}, got type {}",
@@ -664,8 +765,8 @@ void StorageInMemoryMetadata::check(const Block & block, bool need_all) const
 
     for (const auto & column : block)
     {
-        if (names_in_block.count(column.name))
-            throw Exception("Duplicate column " + column.name + " in block", ErrorCodes::DUPLICATE_COLUMN);
+        if (names_in_block.contains(column.name))
+            throw Exception(ErrorCodes::DUPLICATE_COLUMN, "Duplicate column {} in block", column.name);
 
         names_in_block.insert(column.name);
 
@@ -678,7 +779,9 @@ void StorageInMemoryMetadata::check(const Block & block, bool need_all) const
                 listOfColumns(available_columns));
 
         const auto * available_type = it->getMapped();
-        if (!column.type->equals(*available_type) && !isCompatibleEnumTypes(available_type, column.type.get()))
+        if (!available_type->hasDynamicSubcolumnsDeprecated()
+            && !column.type->equals(*available_type)
+            && !isCompatibleEnumTypes(available_type, column.type.get()))
             throw Exception(
                 ErrorCodes::TYPE_MISMATCH,
                 "Type mismatch for column {}. Column has type {}, got type {}",
@@ -691,11 +794,28 @@ void StorageInMemoryMetadata::check(const Block & block, bool need_all) const
     {
         for (const auto & available_column : available_columns)
         {
-            if (!names_in_block.count(available_column.name))
-                throw Exception("Expected column " + available_column.name, ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK);
+            if (!names_in_block.contains(available_column.name))
+                throw Exception(ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK, "Expected column {}", available_column.name);
         }
     }
 }
 
+std::unordered_map<std::string, ColumnSize> StorageInMemoryMetadata::getFakeColumnSizes() const
+{
+    std::unordered_map<std::string, ColumnSize> sizes;
+    for (const auto & col : columns)
+        sizes[col.name] = ColumnSize {.marks = 1000, .data_compressed = 100000000, .data_uncompressed = 1000000000};
+    return sizes;
+}
+
+NameSet StorageInMemoryMetadata::getColumnsWithoutDefaultExpressions(const NamesAndTypesList & exclude) const
+{
+    auto exclude_map = exclude.getNameToTypeMap();
+    NameSet names;
+    for (const auto & col : columns)
+        if (!col.default_desc.expression && !exclude_map.contains(col.name))
+            names.insert(col.name);
+    return names;
+}
 
 }
