@@ -4,7 +4,7 @@
 namespace DB
 {
 
-enum class TTLMode
+enum class TTLMode : uint8_t
 {
     DELETE,
     MOVE,
