@@ -21,6 +21,8 @@ class IAST;
   * and WHERE contains subset of (AND-ed) conditions from original query,
   * that contain only compatible expressions.
   *
+  * If limit is passed additionally apply LIMIT in result query.
+  *
   * Compatible expressions are comparisons of identifiers, constants, and logical operations on them.
   *
   * Throws INCORRECT_QUERY if external_table_strict_query (from context settings)
@@ -28,10 +30,13 @@ class IAST;
   */
 String transformQueryForExternalDatabase(
     const SelectQueryInfo & query_info,
+    const Names & column_names,
     const NamesAndTypesList & available_columns,
     IdentifierQuotingStyle identifier_quoting_style,
+    LiteralEscapingStyle literal_escaping_style,
     const String & database,
     const String & table,
-    ContextPtr context);
+    ContextPtr context,
+    std::optional<size_t> limit = {});
 
 }
