@@ -20,13 +20,19 @@ public:
         return name;
     }
 
-    ColumnsDescription getActualTableStructure(ContextPtr context) const override;
+    ColumnsDescription getActualTableStructure(ContextPtr context, bool is_insert_query) const override;
+
+protected:
+    int fd = -1;
+    String path_to_archive;
+    void parseFirstArguments(const ASTPtr & arg, const ContextPtr & context) override;
+    std::optional<String> tryGetFormatFromFirstArgument() override;
 
 private:
     StoragePtr getStorage(
         const String & source, const String & format_, const ColumnsDescription & columns, ContextPtr global_context,
-        const std::string & table_name, const std::string & compression_method_) const override;
-    const char * getStorageTypeName() const override { return "File"; }
+        const std::string & table_name, const std::string & compression_method_, bool) const override;
+    const char * getStorageEngineName() const override { return "File"; }
 };
 
 }
