@@ -3,8 +3,8 @@
 import sys
 import time
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     for line in sys.stdin:
         time.sleep(25)
-        print("Key " + line, end='')
+        print("Key " + line, end="")
         sys.stdout.flush()
