@@ -6,4 +6,5 @@ ANY LEFT JOIN
 (
     SELECT number * 2 AS k, number AS joined FROM system.numbers LIMIT 10
 ) AS b
-USING k;
+USING k
+ORDER BY k;
