@@ -6,4 +6,5 @@ ALL LEFT JOIN
 (
     SELECT intDiv(number, 2) AS k, number AS joined FROM system.numbers LIMIT 10
 ) js2
-USING k;
+USING k
+ORDER BY ALL;
