@@ -12,7 +12,13 @@ INSERT INTO data2014(name,value) VALUES('Alice', 2000);
 INSERT INTO data2014(name,value) VALUES('Bob', 2000);
 INSERT INTO data2014(name,value) VALUES('Dennis', 35000);
 
-SELECT nn,vv FROM (SELECT name AS nn, value AS vv FROM data2013 UNION ALL SELECT name AS nn, value AS vv FROM data2014) ORDER BY nn,vv ASC;
+SELECT val FROM
+(SELECT value AS val FROM data2013 WHERE name = 'Alice'
+UNION ALL
+SELECT value AS val FROM data2014 WHERE name = 'Alice'
+UNION ALL
+SELECT value AS val FROM data2014 WHERE name = 'Dennis')
+ORDER BY val ASC;
 
 DROP TABLE data2013;
 DROP TABLE data2014;
