@@ -14,4 +14,5 @@ FROM
         arrayJoin([1, 3, 2]) AS key1,
         0 AS key2,
         999 AS table_1
-) js2 USING key2, key1;
+) js2 USING key2, key1
+ORDER BY key1;
