@@ -1,12 +1,12 @@
 -- Tags: long
 
-/* timestamp 1419800400 == 2014-12-29 00:00:00 (Europe/Moscow) */
-/* timestamp 1412106600 == 2014-09-30 23:50:00 (Europe/Moscow) */
-/* timestamp 1420102800 == 2015-01-01 12:00:00 (Europe/Moscow) */
-/* timestamp 1428310800 == 2015-04-06 12:00:00 (Europe/Moscow) */
-/* timestamp 1436956200 == 2015-07-15 13:30:00 (Europe/Moscow) */
-/* timestamp 1426415400 == 2015-03-15 13:30:00 (Europe/Moscow) */
-/* timestamp 1549483055 == 2019-02-06 22:57:35 (Europe/Moscow) */
+/* timestamp 1419800400 == 2014-12-29 00:00:00 (Asia/Istanbul) */
+/* timestamp 1412106600 == 2014-09-30 23:50:00 (Asia/Istanbul) */
+/* timestamp 1420102800 == 2015-01-01 12:00:00 (Asia/Istanbul) */
+/* timestamp 1428310800 == 2015-04-06 12:00:00 (Asia/Istanbul) */
+/* timestamp 1436956200 == 2015-07-15 13:30:00 (Asia/Istanbul) */
+/* timestamp 1426415400 == 2015-03-15 13:30:00 (Asia/Istanbul) */
+/* timestamp 1549483055 == 2019-02-06 22:57:35 (Asia/Istanbul) */
 /* date 16343 == 2014-09-30 */
 /* date 16433 == 2014-12-29 */
 /* date 17933 == 2019-02-06 */
@@ -14,12 +14,12 @@
 /* toStartOfDay */
 
 SELECT 'toStartOfDay';
-SELECT toStartOfDay(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toStartOfDay(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toStartOfDay(toDateTime(1412106600), 'Europe/Paris');
 SELECT toStartOfDay(toDateTime(1412106600), 'Europe/London');
 SELECT toStartOfDay(toDateTime(1412106600), 'Asia/Tokyo');
 SELECT toStartOfDay(toDateTime(1412106600), 'Pacific/Pitcairn');
-SELECT toStartOfDay(toDate(16343), 'Europe/Moscow');
+SELECT toStartOfDay(toDate(16343), 'Asia/Istanbul');
 SELECT toStartOfDay(toDate(16343), 'Europe/Paris');
 SELECT toStartOfDay(toDate(16343), 'Europe/London');
 SELECT toStartOfDay(toDate(16343), 'Asia/Tokyo');
@@ -28,72 +28,104 @@ SELECT toStartOfDay(toDate(16343), 'Pacific/Pitcairn');
 /* toMonday */
 
 SELECT 'toMonday';
-SELECT toMonday(toDateTime(1419800400), 'Europe/Moscow');
+SELECT toMonday(toDateTime(1419800400), 'Asia/Istanbul');
 SELECT toMonday(toDateTime(1419800400), 'Europe/Paris');
 SELECT toMonday(toDateTime(1419800400), 'Europe/London');
 SELECT toMonday(toDateTime(1419800400), 'Asia/Tokyo');
 SELECT toMonday(toDateTime(1419800400), 'Pacific/Pitcairn');
+SELECT toMonday(toDate(16433), 'Asia/Istanbul'); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
 SELECT toMonday(toDate(16433));
-SELECT toMonday(toDate(16433));
-SELECT toMonday(toDate(16433));
-SELECT toMonday(toDate(16433));
-SELECT toMonday(toDate(16433));
+
+/* toStartOfWeek (Sunday) */
+
+SELECT 'toStartOfWeek (Sunday)';
+SELECT toStartOfWeek(toDateTime(1419800400), 0, 'Asia/Istanbul');
+SELECT toStartOfWeek(toDateTime(1419800400), 0, 'Europe/Paris');
+SELECT toStartOfWeek(toDateTime(1419800400), 0, 'Europe/London');
+SELECT toStartOfWeek(toDateTime(1419800400), 0, 'Asia/Tokyo');
+SELECT toStartOfWeek(toDateTime(1419800400), 0, 'Pacific/Pitcairn');
+SELECT toStartOfWeek(toDate(16433), 0, 'Asia/Istanbul'); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
+SELECT toStartOfWeek(toDate(16433), 0);
+
+/* toStartOfWeek (Monday) */
+
+SELECT 'toStartOfWeek (Monday)';
+SELECT toStartOfWeek(toDateTime(1419800400), 1, 'Asia/Istanbul');
+SELECT toStartOfWeek(toDateTime(1419800400), 1, 'Europe/Paris');
+SELECT toStartOfWeek(toDateTime(1419800400), 1, 'Europe/London');
+SELECT toStartOfWeek(toDateTime(1419800400), 1, 'Asia/Tokyo');
+SELECT toStartOfWeek(toDateTime(1419800400), 1, 'Pacific/Pitcairn');
+SELECT toStartOfWeek(toDate(16433), 1, 'Asia/Istanbul'); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
+SELECT toStartOfWeek(toDate(16433), 1);
+
+/* toLastDayOfWeek (Sunday) */
+
+SELECT 'toLastDayOfWeek (Sunday)';
+SELECT toLastDayOfWeek(toDateTime(1419800400), 0, 'Asia/Istanbul');
+SELECT toLastDayOfWeek(toDateTime(1419800400), 0, 'Europe/Paris');
+SELECT toLastDayOfWeek(toDateTime(1419800400), 0, 'Europe/London');
+SELECT toLastDayOfWeek(toDateTime(1419800400), 0, 'Asia/Tokyo');
+SELECT toLastDayOfWeek(toDateTime(1419800400), 0, 'Pacific/Pitcairn');
+SELECT toLastDayOfWeek(toDate(16433), 0, 'Asia/Istanbul'); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
+SELECT toLastDayOfWeek(toDate(16433), 0);
+
+/* toLastDayOfWeek (Monday) */
+
+SELECT 'toLastDayOfWeek (Monday)';
+SELECT toLastDayOfWeek(toDateTime(1419800400), 1, 'Asia/Istanbul');
+SELECT toLastDayOfWeek(toDateTime(1419800400), 1, 'Europe/Paris');
+SELECT toLastDayOfWeek(toDateTime(1419800400), 1, 'Europe/London');
+SELECT toLastDayOfWeek(toDateTime(1419800400), 1, 'Asia/Tokyo');
+SELECT toLastDayOfWeek(toDateTime(1419800400), 1, 'Pacific/Pitcairn');
+SELECT toLastDayOfWeek(toDate(16433), 1, 'Asia/Istanbul'); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
+SELECT toLastDayOfWeek(toDate(16433), 1);
 
 /* toStartOfMonth */
 
 SELECT 'toStartOfMonth';
-SELECT toStartOfMonth(toDateTime(1419800400), 'Europe/Moscow');
+SELECT toStartOfMonth(toDateTime(1419800400), 'Asia/Istanbul');
 SELECT toStartOfMonth(toDateTime(1419800400), 'Europe/Paris');
 SELECT toStartOfMonth(toDateTime(1419800400), 'Europe/London');
 SELECT toStartOfMonth(toDateTime(1419800400), 'Asia/Tokyo');
 SELECT toStartOfMonth(toDateTime(1419800400), 'Pacific/Pitcairn');
-SELECT toStartOfMonth(toDate(16433));
-SELECT toStartOfMonth(toDate(16433));
-SELECT toStartOfMonth(toDate(16433));
-SELECT toStartOfMonth(toDate(16433));
+SELECT toStartOfMonth(toDate(16433), 'Asia/Istanbul'); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
 SELECT toStartOfMonth(toDate(16433));
 
 /* toStartOfQuarter */
 
 SELECT 'toStartOfQuarter';
-SELECT toStartOfQuarter(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toStartOfQuarter(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toStartOfQuarter(toDateTime(1412106600), 'Europe/Paris');
 SELECT toStartOfQuarter(toDateTime(1412106600), 'Europe/London');
 SELECT toStartOfQuarter(toDateTime(1412106600), 'Asia/Tokyo');
 SELECT toStartOfQuarter(toDateTime(1412106600), 'Pacific/Pitcairn');
-SELECT toStartOfQuarter(toDate(16343));
-SELECT toStartOfQuarter(toDate(16343));
-SELECT toStartOfQuarter(toDate(16343));
-SELECT toStartOfQuarter(toDate(16343));
+SELECT toStartOfQuarter(toDate(16343), 'Asia/Istanbul'); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
 SELECT toStartOfQuarter(toDate(16343));
 
 /* toStartOfYear */
 
 SELECT 'toStartOfYear';
-SELECT toStartOfYear(toDateTime(1419800400), 'Europe/Moscow');
+SELECT toStartOfYear(toDateTime(1419800400), 'Asia/Istanbul');
 SELECT toStartOfYear(toDateTime(1419800400), 'Europe/Paris');
 SELECT toStartOfYear(toDateTime(1419800400), 'Europe/London');
 SELECT toStartOfYear(toDateTime(1419800400), 'Asia/Tokyo');
 SELECT toStartOfYear(toDateTime(1419800400), 'Pacific/Pitcairn');
-SELECT toStartOfYear(toDate(16433));
-SELECT toStartOfYear(toDate(16433));
-SELECT toStartOfYear(toDate(16433));
-SELECT toStartOfYear(toDate(16433));
+SELECT toStartOfYear(toDate(16433), 'Asia/Istanbul'); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
 SELECT toStartOfYear(toDate(16433));
 
-/* toTime */
+/* toTimeWithFixedDate */
 
-SELECT 'toTime';
-SELECT toString(toTime(toDateTime(1420102800), 'Europe/Moscow'), 'Europe/Moscow'), toString(toTime(toDateTime(1428310800), 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toTime(toDateTime(1420102800), 'Europe/Paris'), 'Europe/Paris'), toString(toTime(toDateTime(1428310800), 'Europe/Paris'), 'Europe/Paris');
-SELECT toString(toTime(toDateTime(1420102800), 'Europe/London'), 'Europe/London'), toString(toTime(toDateTime(1428310800), 'Europe/London'), 'Europe/London');
-SELECT toString(toTime(toDateTime(1420102800), 'Asia/Tokyo'), 'Asia/Tokyo'), toString(toTime(toDateTime(1428310800), 'Asia/Tokyo'), 'Asia/Tokyo');
-SELECT toString(toTime(toDateTime(1420102800), 'Pacific/Pitcairn'), 'Pacific/Pitcairn'), toString(toTime(toDateTime(1428310800), 'Pacific/Pitcairn'), 'Pacific/Pitcairn');
+SELECT 'toTimeWithFixedDate';
+SELECT toString(toTimeWithFixedDate(toDateTime(1420102800), 'Asia/Istanbul'), 'Asia/Istanbul'), toString(toTimeWithFixedDate(toDateTime(1428310800), 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toTimeWithFixedDate(toDateTime(1420102800), 'Europe/Paris'), 'Europe/Paris'), toString(toTimeWithFixedDate(toDateTime(1428310800), 'Europe/Paris'), 'Europe/Paris');
+SELECT toString(toTimeWithFixedDate(toDateTime(1420102800), 'Europe/London'), 'Europe/London'), toString(toTimeWithFixedDate(toDateTime(1428310800), 'Europe/London'), 'Europe/London');
+SELECT toString(toTimeWithFixedDate(toDateTime(1420102800), 'Asia/Tokyo'), 'Asia/Tokyo'), toString(toTimeWithFixedDate(toDateTime(1428310800), 'Asia/Tokyo'), 'Asia/Tokyo');
+SELECT toString(toTimeWithFixedDate(toDateTime(1420102800), 'Pacific/Pitcairn'), 'Pacific/Pitcairn'), toString(toTimeWithFixedDate(toDateTime(1428310800), 'Pacific/Pitcairn'), 'Pacific/Pitcairn');
 
 /* toYear */
 
 SELECT 'toYear';
-SELECT toYear(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toYear(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toYear(toDateTime(1412106600), 'Europe/Paris');
 SELECT toYear(toDateTime(1412106600), 'Europe/London');
 SELECT toYear(toDateTime(1412106600), 'Asia/Tokyo');
@@ -102,7 +134,7 @@ SELECT toYear(toDateTime(1412106600), 'Pacific/Pitcairn');
 /* toMonth */
 
 SELECT 'toMonth';
-SELECT toMonth(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toMonth(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toMonth(toDateTime(1412106600), 'Europe/Paris');
 SELECT toMonth(toDateTime(1412106600), 'Europe/London');
 SELECT toMonth(toDateTime(1412106600), 'Asia/Tokyo');
@@ -111,7 +143,7 @@ SELECT toMonth(toDateTime(1412106600), 'Pacific/Pitcairn');
 /* toDayOfMonth */
 
 SELECT 'toDayOfMonth';
-SELECT toDayOfMonth(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toDayOfMonth(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toDayOfMonth(toDateTime(1412106600), 'Europe/Paris');
 SELECT toDayOfMonth(toDateTime(1412106600), 'Europe/London');
 SELECT toDayOfMonth(toDateTime(1412106600), 'Asia/Tokyo');
@@ -120,16 +152,16 @@ SELECT toDayOfMonth(toDateTime(1412106600), 'Pacific/Pitcairn');
 /* toDayOfWeek */
 
 SELECT 'toDayOfWeek';
-SELECT toDayOfWeek(toDateTime(1412106600), 'Europe/Moscow');
-SELECT toDayOfWeek(toDateTime(1412106600), 'Europe/Paris');
-SELECT toDayOfWeek(toDateTime(1412106600), 'Europe/London');
-SELECT toDayOfWeek(toDateTime(1412106600), 'Asia/Tokyo');
-SELECT toDayOfWeek(toDateTime(1412106600), 'Pacific/Pitcairn');
+SELECT toDayOfWeek(toDateTime(1412106600), 0, 'Asia/Istanbul');
+SELECT toDayOfWeek(toDateTime(1412106600), 0, 'Europe/Paris');
+SELECT toDayOfWeek(toDateTime(1412106600), 0, 'Europe/London');
+SELECT toDayOfWeek(toDateTime(1412106600), 0, 'Asia/Tokyo');
+SELECT toDayOfWeek(toDateTime(1412106600), 0, 'Pacific/Pitcairn');
 
 /* toHour */
 
 SELECT 'toHour';
-SELECT toHour(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toHour(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toHour(toDateTime(1412106600), 'Europe/Paris');
 SELECT toHour(toDateTime(1412106600), 'Europe/London');
 SELECT toHour(toDateTime(1412106600), 'Asia/Tokyo');
@@ -138,7 +170,7 @@ SELECT toHour(toDateTime(1412106600), 'Pacific/Pitcairn');
 /* toMinute */
 
 SELECT 'toMinute';
-SELECT toMinute(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toMinute(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toMinute(toDateTime(1412106600), 'Europe/Paris');
 SELECT toMinute(toDateTime(1412106600), 'Europe/London');
 SELECT toMinute(toDateTime(1412106600), 'Asia/Tokyo');
@@ -147,7 +179,7 @@ SELECT toMinute(toDateTime(1412106600), 'Pacific/Pitcairn');
 /* toSecond */
 
 SELECT 'toSecond';
-SELECT toSecond(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toSecond(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toSecond(toDateTime(1412106600), 'Europe/Paris');
 SELECT toSecond(toDateTime(1412106600), 'Europe/London');
 SELECT toSecond(toDateTime(1412106600), 'Asia/Tokyo');
@@ -156,25 +188,25 @@ SELECT toSecond(toDateTime(1412106600), 'Pacific/Pitcairn');
 /* toStartOfMinute */
 
 SELECT 'toStartOfMinute';
-SELECT toString(toStartOfMinute(toDateTime(1549483055), 'Europe/Moscow'), 'Europe/Moscow');
+SELECT toString(toStartOfMinute(toDateTime(1549483055), 'Asia/Istanbul'), 'Asia/Istanbul');
 SELECT toString(toStartOfMinute(toDateTime(1549483055), 'Europe/Paris'), 'Europe/Paris');
 SELECT toString(toStartOfMinute(toDateTime(1549483055), 'Europe/London'), 'Europe/London');
 SELECT toString(toStartOfMinute(toDateTime(1549483055), 'Asia/Tokyo'), 'Asia/Tokyo');
 SELECT toString(toStartOfMinute(toDateTime(1549483055), 'Pacific/Pitcairn'), 'Pacific/Pitcairn');
 
-/* toStartOfFiveMinute */
+/* toStartOfFiveMinutes */
 
-SELECT 'toStartOfFiveMinute';
-SELECT toString(toStartOfFiveMinute(toDateTime(1549483055), 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfFiveMinute(toDateTime(1549483055), 'Europe/Paris'), 'Europe/Paris');
-SELECT toString(toStartOfFiveMinute(toDateTime(1549483055), 'Europe/London'), 'Europe/London');
-SELECT toString(toStartOfFiveMinute(toDateTime(1549483055), 'Asia/Tokyo'), 'Asia/Tokyo');
-SELECT toString(toStartOfFiveMinute(toDateTime(1549483055), 'Pacific/Pitcairn'), 'Pacific/Pitcairn');
+SELECT 'toStartOfFiveMinutes';
+SELECT toString(toStartOfFiveMinutes(toDateTime(1549483055), 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfFiveMinutes(toDateTime(1549483055), 'Europe/Paris'), 'Europe/Paris');
+SELECT toString(toStartOfFiveMinutes(toDateTime(1549483055), 'Europe/London'), 'Europe/London');
+SELECT toString(toStartOfFiveMinutes(toDateTime(1549483055), 'Asia/Tokyo'), 'Asia/Tokyo');
+SELECT toString(toStartOfFiveMinutes(toDateTime(1549483055), 'Pacific/Pitcairn'), 'Pacific/Pitcairn');
 
 /* toStartOfTenMinutes */
 
 SELECT 'toStartOfTenMinutes';
-SELECT toString(toStartOfTenMinutes(toDateTime(1549483055), 'Europe/Moscow'), 'Europe/Moscow');
+SELECT toString(toStartOfTenMinutes(toDateTime(1549483055), 'Asia/Istanbul'), 'Asia/Istanbul');
 SELECT toString(toStartOfTenMinutes(toDateTime(1549483055), 'Europe/Paris'), 'Europe/Paris');
 SELECT toString(toStartOfTenMinutes(toDateTime(1549483055), 'Europe/London'), 'Europe/London');
 SELECT toString(toStartOfTenMinutes(toDateTime(1549483055), 'Asia/Tokyo'), 'Asia/Tokyo');
@@ -183,7 +215,7 @@ SELECT toString(toStartOfTenMinutes(toDateTime(1549483055), 'Pacific/Pitcairn'),
 /* toStartOfFifteenMinutes */
 
 SELECT 'toStartOfFifteenMinutes';
-SELECT toString(toStartOfFifteenMinutes(toDateTime(1549483055), 'Europe/Moscow'), 'Europe/Moscow');
+SELECT toString(toStartOfFifteenMinutes(toDateTime(1549483055), 'Asia/Istanbul'), 'Asia/Istanbul');
 SELECT toString(toStartOfFifteenMinutes(toDateTime(1549483055), 'Europe/Paris'), 'Europe/Paris');
 SELECT toString(toStartOfFifteenMinutes(toDateTime(1549483055), 'Europe/London'), 'Europe/London');
 SELECT toString(toStartOfFifteenMinutes(toDateTime(1549483055), 'Asia/Tokyo'), 'Asia/Tokyo');
@@ -192,7 +224,7 @@ SELECT toString(toStartOfFifteenMinutes(toDateTime(1549483055), 'Pacific/Pitcair
 /* toStartOfHour */
 
 SELECT 'toStartOfHour';
-SELECT toString(toStartOfHour(toDateTime(1549483055), 'Europe/Moscow'), 'Europe/Moscow');
+SELECT toString(toStartOfHour(toDateTime(1549483055), 'Asia/Istanbul'), 'Asia/Istanbul');
 SELECT toString(toStartOfHour(toDateTime(1549483055), 'Europe/Paris'), 'Europe/Paris');
 SELECT toString(toStartOfHour(toDateTime(1549483055), 'Europe/London'), 'Europe/London');
 SELECT toString(toStartOfHour(toDateTime(1549483055), 'Asia/Tokyo'), 'Asia/Tokyo');
@@ -201,33 +233,33 @@ SELECT toString(toStartOfHour(toDateTime(1549483055), 'Pacific/Pitcairn'), 'Paci
 /* toStartOfInterval */
 
 SELECT 'toStartOfInterval';
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 1 year, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 2 year, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 5 year, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 1 quarter, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 2 quarter, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 3 quarter, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 1 month, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 2 month, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 5 month, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 1 week, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 2 week, 'Europe/Moscow');
-SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 6 week, 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 1 day, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 2 day, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 5 day, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 1 hour, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 2 hour, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 6 hour, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 24 hour, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 1 minute, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 2 minute, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 5 minute, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 20 minute, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 90 minute, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 1 second, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 2 second, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 5 second, 'Europe/Moscow'), 'Europe/Moscow');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 1 year, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 2 year, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 5 year, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 1 quarter, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 2 quarter, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 3 quarter, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 1 month, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 2 month, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 5 month, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 1 week, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 2 week, 'Asia/Istanbul');
+SELECT toStartOfInterval(toDateTime(1549483055), INTERVAL 6 week, 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 1 day, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 2 day, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 5 day, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 1 hour, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 2 hour, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 6 hour, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 24 hour, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 1 minute, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 2 minute, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 5 minute, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 20 minute, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 90 minute, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 1 second, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 2 second, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDateTime(1549483055), INTERVAL 5 second, 'Asia/Istanbul'), 'Asia/Istanbul');
 SELECT toStartOfInterval(toDate(17933), INTERVAL 1 year);
 SELECT toStartOfInterval(toDate(17933), INTERVAL 2 year);
 SELECT toStartOfInterval(toDate(17933), INTERVAL 5 year);
@@ -240,14 +272,14 @@ SELECT toStartOfInterval(toDate(17933), INTERVAL 5 month);
 SELECT toStartOfInterval(toDate(17933), INTERVAL 1 week);
 SELECT toStartOfInterval(toDate(17933), INTERVAL 2 week);
 SELECT toStartOfInterval(toDate(17933), INTERVAL 6 week);
-SELECT toString(toStartOfInterval(toDate(17933), INTERVAL 1 day, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDate(17933), INTERVAL 2 day, 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toString(toStartOfInterval(toDate(17933), INTERVAL 5 day, 'Europe/Moscow'), 'Europe/Moscow');
+SELECT toString(toStartOfInterval(toDate(17933), INTERVAL 1 day, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDate(17933), INTERVAL 2 day, 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toString(toStartOfInterval(toDate(17933), INTERVAL 5 day, 'Asia/Istanbul'), 'Asia/Istanbul');
 
 /* toRelativeYearNum */
 
 SELECT 'toRelativeYearNum';
-SELECT toRelativeYearNum(toDateTime(1412106600), 'Europe/Moscow') - toRelativeYearNum(toDateTime(0), 'Europe/Moscow');
+SELECT toRelativeYearNum(toDateTime(1412106600), 'Asia/Istanbul') - toRelativeYearNum(toDateTime(0), 'Asia/Istanbul');
 SELECT toRelativeYearNum(toDateTime(1412106600), 'Europe/Paris') - toRelativeYearNum(toDateTime(0), 'Europe/Paris');
 SELECT toRelativeYearNum(toDateTime(1412106600), 'Europe/London') - toRelativeYearNum(toDateTime(0), 'Europe/London');
 SELECT toRelativeYearNum(toDateTime(1412106600), 'Asia/Tokyo') - toRelativeYearNum(toDateTime(0), 'Asia/Tokyo');
@@ -256,7 +288,7 @@ SELECT toRelativeYearNum(toDateTime(1412106600), 'Pacific/Pitcairn') - toRelativ
 /* toRelativeMonthNum */
 
 SELECT 'toRelativeMonthNum';
-SELECT toRelativeMonthNum(toDateTime(1412106600), 'Europe/Moscow') - toRelativeMonthNum(toDateTime(0), 'Europe/Moscow');
+SELECT toRelativeMonthNum(toDateTime(1412106600), 'Asia/Istanbul') - toRelativeMonthNum(toDateTime(0), 'Asia/Istanbul');
 SELECT toRelativeMonthNum(toDateTime(1412106600), 'Europe/Paris') - toRelativeMonthNum(toDateTime(0), 'Europe/Paris');
 SELECT toRelativeMonthNum(toDateTime(1412106600), 'Europe/London') - toRelativeMonthNum(toDateTime(0), 'Europe/London');
 SELECT toRelativeMonthNum(toDateTime(1412106600), 'Asia/Tokyo') - toRelativeMonthNum(toDateTime(0), 'Asia/Tokyo');
@@ -265,7 +297,7 @@ SELECT toRelativeMonthNum(toDateTime(1412106600), 'Pacific/Pitcairn') - toRelati
 /* toRelativeWeekNum */
 
 SELECT 'toRelativeWeekNum';
-SELECT toRelativeWeekNum(toDateTime(1412106600), 'Europe/Moscow') - toRelativeWeekNum(toDateTime(0), 'Europe/Moscow');
+SELECT toRelativeWeekNum(toDateTime(1412106600), 'Asia/Istanbul') - toRelativeWeekNum(toDateTime(0), 'Asia/Istanbul');
 SELECT toRelativeWeekNum(toDateTime(1412106600), 'Europe/Paris') - toRelativeWeekNum(toDateTime(0), 'Europe/Paris');
 SELECT toRelativeWeekNum(toDateTime(1412106600), 'Europe/London') - toRelativeWeekNum(toDateTime(0), 'Europe/London');
 SELECT toRelativeWeekNum(toDateTime(1412106600), 'Asia/Tokyo') - toRelativeWeekNum(toDateTime(0), 'Asia/Tokyo');
@@ -274,7 +306,7 @@ SELECT toRelativeWeekNum(toDateTime(1412106600), 'Pacific/Pitcairn') - toRelativ
 /* toRelativeDayNum */
 
 SELECT 'toRelativeDayNum';
-SELECT toRelativeDayNum(toDateTime(1412106600), 'Europe/Moscow') - toRelativeDayNum(toDateTime(0), 'Europe/Moscow');
+SELECT toRelativeDayNum(toDateTime(1412106600), 'Asia/Istanbul') - toRelativeDayNum(toDateTime(0), 'Asia/Istanbul');
 SELECT toRelativeDayNum(toDateTime(1412106600), 'Europe/Paris') - toRelativeDayNum(toDateTime(0), 'Europe/Paris');
 SELECT toRelativeDayNum(toDateTime(1412106600), 'Europe/London') - toRelativeDayNum(toDateTime(0), 'Europe/London');
 SELECT toRelativeDayNum(toDateTime(1412106600), 'Asia/Tokyo') - toRelativeDayNum(toDateTime(0), 'Asia/Tokyo');
@@ -284,7 +316,7 @@ SELECT toUInt16(toRelativeDayNum(toDateTime(1412106600), 'Pacific/Pitcairn') - t
 /* toRelativeHourNum */
 
 SELECT 'toRelativeHourNum';
-SELECT toRelativeHourNum(toDateTime(1412106600), 'Europe/Moscow') - toRelativeHourNum(toDateTime(0), 'Europe/Moscow');
+SELECT toRelativeHourNum(toDateTime(1412106600), 'Asia/Istanbul') - toRelativeHourNum(toDateTime(0), 'Asia/Istanbul');
 SELECT toRelativeHourNum(toDateTime(1412106600), 'Europe/Paris') - toRelativeHourNum(toDateTime(0), 'Europe/Paris');
 SELECT toRelativeHourNum(toDateTime(1412106600), 'Europe/London') - toRelativeHourNum(toDateTime(0), 'Europe/London');
 SELECT toRelativeHourNum(toDateTime(1412106600), 'Asia/Tokyo') - toRelativeHourNum(toDateTime(0), 'Asia/Tokyo');
@@ -293,7 +325,7 @@ SELECT toRelativeHourNum(toDateTime(1412106600), 'Pacific/Pitcairn') - toRelativ
 /* toRelativeMinuteNum */
 
 SELECT 'toRelativeMinuteNum';
-SELECT toRelativeMinuteNum(toDateTime(1412106600), 'Europe/Moscow') - toRelativeMinuteNum(toDateTime(0), 'Europe/Moscow');
+SELECT toRelativeMinuteNum(toDateTime(1412106600), 'Asia/Istanbul') - toRelativeMinuteNum(toDateTime(0), 'Asia/Istanbul');
 SELECT toRelativeMinuteNum(toDateTime(1412106600), 'Europe/Paris') - toRelativeMinuteNum(toDateTime(0), 'Europe/Paris');
 SELECT toRelativeMinuteNum(toDateTime(1412106600), 'Europe/London') - toRelativeMinuteNum(toDateTime(0), 'Europe/London');
 SELECT toRelativeMinuteNum(toDateTime(1412106600), 'Asia/Tokyo') - toRelativeMinuteNum(toDateTime(0), 'Asia/Tokyo');
@@ -302,7 +334,7 @@ SELECT toRelativeMinuteNum(toDateTime(1412106600), 'Pacific/Pitcairn') - toRelat
 /* toRelativeSecondNum */
 
 SELECT 'toRelativeSecondNum';
-SELECT toRelativeSecondNum(toDateTime(1412106600), 'Europe/Moscow') - toRelativeSecondNum(toDateTime(0), 'Europe/Moscow');
+SELECT toRelativeSecondNum(toDateTime(1412106600), 'Asia/Istanbul') - toRelativeSecondNum(toDateTime(0), 'Asia/Istanbul');
 SELECT toRelativeSecondNum(toDateTime(1412106600), 'Europe/Paris') - toRelativeSecondNum(toDateTime(0), 'Europe/Paris');
 SELECT toRelativeSecondNum(toDateTime(1412106600), 'Europe/London') - toRelativeSecondNum(toDateTime(0), 'Europe/London');
 SELECT toRelativeSecondNum(toDateTime(1412106600), 'Asia/Tokyo') - toRelativeSecondNum(toDateTime(0), 'Asia/Tokyo');
@@ -311,13 +343,13 @@ SELECT toRelativeSecondNum(toDateTime(1412106600), 'Pacific/Pitcairn') - toRelat
 /* toDate */
 
 SELECT 'toDate';
-SELECT toDate(toDateTime(1412106600), 'Europe/Moscow');
+SELECT toDate(toDateTime(1412106600), 'Asia/Istanbul');
 SELECT toDate(toDateTime(1412106600), 'Europe/Paris');
 SELECT toDate(toDateTime(1412106600), 'Europe/London');
 SELECT toDate(toDateTime(1412106600), 'Asia/Tokyo');
 SELECT toDate(toDateTime(1412106600), 'Pacific/Pitcairn');
 
-SELECT toDate(1412106600, 'Europe/Moscow');
+SELECT toDate(1412106600, 'Asia/Istanbul');
 SELECT toDate(1412106600, 'Europe/Paris');
 SELECT toDate(1412106600, 'Europe/London');
 SELECT toDate(1412106600, 'Asia/Tokyo');
@@ -328,7 +360,7 @@ SELECT toDate(16343);
 /* toString */
 
 SELECT 'toString';
-SELECT toString(toDateTime(1436956200), 'Europe/Moscow');
+SELECT toString(toDateTime(1436956200), 'Asia/Istanbul');
 SELECT toString(toDateTime(1436956200), 'Europe/Paris');
 SELECT toString(toDateTime(1436956200), 'Europe/London');
 SELECT toString(toDateTime(1436956200), 'Asia/Tokyo');
@@ -337,13 +369,13 @@ SELECT toString(toDateTime(1436956200), 'Pacific/Pitcairn');
 /* toUnixTimestamp */
 
 SELECT 'toUnixTimestamp';
-SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Europe/Moscow'), 'Europe/Moscow');
-SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Europe/Moscow'), 'Europe/Paris');
-SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Europe/Moscow'), 'Europe/London');
-SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Europe/Moscow'), 'Asia/Tokyo');
-SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Europe/Moscow'), 'Pacific/Pitcairn');
+SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Asia/Istanbul'), 'Asia/Istanbul');
+SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Asia/Istanbul'), 'Europe/Paris');
+SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Asia/Istanbul'), 'Europe/London');
+SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Asia/Istanbul'), 'Asia/Tokyo');
+SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Asia/Istanbul'), 'Pacific/Pitcairn');
 
-SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Europe/Moscow'), 'Europe/Moscow');
+SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Asia/Istanbul'), 'Asia/Istanbul');
 SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Europe/Paris'), 'Europe/Paris');
 SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Europe/London'), 'Europe/London');
 SELECT toUnixTimestamp(toString(toDateTime(1426415400), 'Asia/Tokyo'), 'Asia/Tokyo');
