@@ -1,5 +1,4 @@
--- Tags: no-parallel
-
+set allow_deprecated_syntax_for_merge_tree=1;
 DROP TABLE IF EXISTS array_pk;
 CREATE TABLE array_pk (key Array(UInt8), s String, n UInt64, d Date MATERIALIZED '2000-01-01') ENGINE = MergeTree(d, (key, s, n), 1);
 
