@@ -7,3 +7,6 @@ CURDIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
 $CLICKHOUSE_CLIENT -q "SHOW PROCESSLIST" &>/dev/null
 $CLICKHOUSE_CLIENT -q "SHOW DATABASES" &>/dev/null
 $CLICKHOUSE_CLIENT -q "SHOW TABLES" &>/dev/null
+$CLICKHOUSE_CLIENT -q "SHOW ENGINES" &>/dev/null
+$CLICKHOUSE_CLIENT -q "SHOW FUNCTIONS" &>/dev/null
+$CLICKHOUSE_CLIENT -q "SHOW MERGES" &>/dev/null
