@@ -13,11 +13,11 @@ DROP TABLE set;
 DROP TABLE IF EXISTS join;
 CREATE TABLE join (k UInt8, x String) ENGINE = Memory;
 INSERT INTO join VALUES (1, 'hello');
-SELECT k, x FROM (SELECT arrayJoin([1, 2]) AS k) js1 ANY LEFT JOIN join USING k;
+SELECT k, x FROM (SELECT arrayJoin([1, 2]) AS k) js1 ANY LEFT JOIN join USING k ORDER BY ALL;
 
 DROP TABLE join;
 CREATE TABLE join (k UInt8, x String) ENGINE = Join(ANY, LEFT, k);
 INSERT INTO join VALUES (1, 'hello');
-SELECT k, x FROM (SELECT arrayJoin([1, 2]) AS k) js1 ANY LEFT JOIN join USING k;
+SELECT k, x FROM (SELECT arrayJoin([1, 2]) AS k) js1 ANY LEFT JOIN join USING k ORDER BY ALL;
 
 DROP TABLE join;
