@@ -1,7 +1,7 @@
 drop table if exists lc_00752;
 drop table if exists lc_mv_00752;
 
-create table lc_00752 (str StringWithDictionary) engine = MergeTree order by tuple();
+create table lc_00752 (str LowCardinality(String)) engine = MergeTree order by tuple();
 
 insert into lc_00752 values ('a'), ('bbb'), ('ab'), ('accccc'), ('baasddas'), ('bcde');
 
@@ -12,4 +12,3 @@ select * from lc_mv_00752 order by letter;
 
 drop table if exists lc_00752;
 drop table if exists lc_mv_00752;
-
