@@ -5,9 +5,9 @@ SELECT number as x, roundDown(x, [6, 5, 4]) FROM system.numbers LIMIT 10;
 SELECT 1 as x, roundDown(x, [6, 5, 4]);
 
 SET send_logs_level = 'fatal';
-SELECT 1 as x, roundDown(x, []); -- { serverError 43 }
-SELECT 1 as x, roundDown(x, emptyArrayUInt8()); -- { serverError 44 }
-SELECT roundDown(number, [number]) FROM system.numbers LIMIT 10; -- { serverError 44 }
+SELECT 1 as x, roundDown(x, []); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
+SELECT 1 as x, roundDown(x, emptyArrayUInt8()); -- { serverError ILLEGAL_COLUMN }
+SELECT roundDown(number, [number]) FROM system.numbers LIMIT 10; -- { serverError ILLEGAL_COLUMN }
 
 SELECT 1 as x, roundDown(x, [1]);
 SELECT 1 as x, roundDown(x, [1.5]);
