@@ -1,7 +1,3 @@
--- Tags: no-parallel
-
-CREATE DATABASE IF NOT EXISTS test_00815;
-USE test_00815;
 
 DROP TABLE IF EXISTS fact_cpc_clicks;
 DROP TABLE IF EXISTS dim_model;
@@ -16,6 +12,6 @@ select f.model_id from fact_cpc_clicks as f left join dim_model as d on f.model_
 
 USE default;
 
-select f.model_id from test_00815.fact_cpc_clicks as f left join test_00815.dim_model as d on f.model_id=d.model_id limit 10;
+select f.model_id from {CLICKHOUSE_DATABASE:Identifier}.fact_cpc_clicks as f left join {CLICKHOUSE_DATABASE:Identifier}.dim_model as d on f.model_id=d.model_id limit 10;
 
-DROP DATABASE test_00815;
+DROP DATABASE {CLICKHOUSE_DATABASE:Identifier};
