@@ -10,7 +10,7 @@ INSERT INTO test1_00843 VALUES (1);
 CREATE VIEW view_00843 AS SELECT * FROM test1_00843;
 SELECT * FROM view_00843;
 RENAME TABLE test1_00843 TO test2_00843;
-SELECT * FROM view_00843; -- { serverError 60 }
+SELECT * FROM view_00843; -- { serverError UNKNOWN_TABLE }
 RENAME TABLE test2_00843 TO test1_00843;
 SELECT * FROM view_00843;
 
