@@ -1,5 +1,3 @@
--- Tags: no-parallel
-
 set send_logs_level = 'fatal';
 
 drop table if exists ttl;
@@ -15,14 +13,14 @@ optimize table ttl partition 10 final;
 
 select * from ttl order by d, a;
 
-alter table ttl modify ttl a; -- { serverError 450 }
+alter table ttl modify ttl a; -- { serverError BAD_TTL_EXPRESSION }
 
 drop table if exists ttl;
 
 create table ttl (d Date, a Int) engine = MergeTree order by tuple() partition by toDayOfMonth(d) settings remove_empty_parts = 0;
 alter table ttl modify column a Int ttl d + interval 1 day;
 desc table ttl;
-alter table ttl modify column d Int ttl d + interval 1 day; -- { serverError 43 }
-alter table ttl modify column d DateTime ttl d + interval 1 day; -- { serverError 524 }
+alter table ttl modify column d Int ttl d + interval 1 day; -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
+alter table ttl modify column d DateTime ttl d + interval 1 day; -- { serverError ALTER_OF_COLUMN_IS_FORBIDDEN }
 
 drop table if exists ttl;
