@@ -22,13 +22,13 @@ SELECT 'a' IN splitByChar('c', 'abcdef');
 
 SELECT 'errors:';
 -- non-constant expressions in the right side of IN
-SELECT count() FROM samples WHERE 1 IN range(samples.value); -- { serverError 47 }
-SELECT count() FROM samples WHERE 1 IN range(rand() % 1000); -- { serverError 36 }
+SELECT count() FROM samples WHERE 1 IN range(samples.value); -- { serverError UNSUPPORTED_METHOD, 47 }
+SELECT count() FROM samples WHERE 1 IN range(rand() % 1000); -- { serverError UNSUPPORTED_METHOD, 36 }
 
 -- index is not used
-SELECT count() FROM samples WHERE value IN range(3); -- { serverError 277 }
+SELECT count() FROM samples WHERE value IN range(3); -- { serverError INDEX_NOT_USED }
 
 -- wrong type
-SELECT 123 IN splitByChar('c', 'abcdef'); -- { serverError 53 }
+SELECT 123 IN splitByChar('c', 'abcdef'); -- { serverError TYPE_MISMATCH }
 
-DROP TABLE samples;
\ No newline at end of file
+DROP TABLE samples;
