@@ -1,6 +1,6 @@
 drop table if exists limit_by;
 
-create table limit_by(id Int, val Int) engine = Memory;
+create table limit_by(id Int, val Int) engine = MergeTree ORDER BY tuple();
 
 insert into limit_by values(1, 100), (1, 110), (1, 120), (1, 130), (2, 200), (2, 210), (2, 220), (3, 300);
 
