@@ -6,7 +6,7 @@ CURDIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
 
 $CLICKHOUSE_CLIENT --query="DROP TABLE IF EXISTS minmax_idx;"
 
-$CLICKHOUSE_CLIENT -n --query="
+$CLICKHOUSE_CLIENT --query="
 CREATE TABLE minmax_idx
 (
     u64 UInt64,
@@ -15,7 +15,7 @@ CREATE TABLE minmax_idx
     INDEX idx (i64, u64 * i64) TYPE minmax GRANULARITY 1
 ) ENGINE = MergeTree()
 ORDER BY u64
-SETTINGS index_granularity = 2;"
+SETTINGS index_granularity = 2, index_granularity_bytes = '10Mi';"
 
 $CLICKHOUSE_CLIENT --query="INSERT INTO minmax_idx VALUES
 (0, 1, 1),
